//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:
//  $Header:
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "BooleanTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "GenApi/impl/PolyReference.h"
#include "GenApi/impl/Value2String.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( BooleanTestSuite );

void BooleanTestSuite::TestValueAccess()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Boolean Name=\"Trigger\">                \n"
    "    <Value>1</Value>                    \n"
    "    <OnValue>1</OnValue>                \n"
    "    <OffValue>0</OffValue>                \n"
    "</Boolean>                                \n"
    "                                        \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CBooleanPtr ptrValue = Camera._GetNode("Trigger");
    CPPUNIT_ASSERT( ptrValue.IsValid() );
    CPPUNIT_ASSERT_EQUAL (gcstring("IBoolean"), GetInterfaceName(ptrValue));

    CPPUNIT_ASSERT_EQUAL((bool)1, ptrValue->GetValue() );
    ptrValue->SetValue( 0);
    CPPUNIT_ASSERT_EQUAL((bool)0, ptrValue->GetValue() );

    ptrValue->SetValue( 1 );
    CPPUNIT_ASSERT_EQUAL((bool)1, ptrValue->GetValue() );

    ptrValue->FromString("0");
    gcstring gstr( "0" );
    CPPUNIT_ASSERT_EQUAL(gstr, ptrValue->ToString() );

    ptrValue->FromString("1");
    gstr = "1";
    CPPUNIT_ASSERT_EQUAL(gstr, ptrValue->ToString() );

    CPPUNIT_ASSERT_THROW_EX(
        ptrValue->FromString("X"),
        GenICam::InvalidArgumentException
    );

    CPPUNIT_ASSERT_EQUAL( RW, ptrValue->GetAccessMode() );

    // operators
    ptrValue->operator=(0);
    CPPUNIT_ASSERT_EQUAL((bool)0, ptrValue->operator()());

    // Check IBoolean operators
    ptrValue->IBoolean::operator=(1);
    CPPUNIT_ASSERT_EQUAL((bool)1,ptrValue->IBoolean::operator()());

    // Reference
    CBooleanRef refBoolean;

    CPPUNIT_ASSERT_THROW_EX(
        refBoolean.SetValue(1),
        GenICam::AccessException
    );
    CPPUNIT_ASSERT_THROW_EX(
        refBoolean.GetValue(),
        GenICam::AccessException
    );
    CPPUNIT_ASSERT_THROW_EX(
        refBoolean.operator=(1),
        GenICam::AccessException
    );
    CPPUNIT_ASSERT_THROW_EX(
        refBoolean.operator()(),
        GenICam::AccessException
    );

    CPPUNIT_ASSERT_NO_THROW(refBoolean.SetReference( ptrValue ) );

    refBoolean.SetValue(1);
    CPPUNIT_ASSERT_EQUAL((bool)1, refBoolean.GetValue() );

    refBoolean.operator=(0);
    CPPUNIT_ASSERT_EQUAL((bool)0, refBoolean.operator()() );


    // access without verification
    // (currently just to reach happy path, invalid access is not possible for boolean)
    ptrValue->SetValue( true, false );
    ptrValue->FromString("0", false);

    // and test also the properties
    CNodePtr ptrAnode = (CNodePtr) ptrValue;
    gcstring ValueStr;
    gcstring AttributeStr;
    ptrValue->SetValue (true);
    CPPUNIT_ASSERT( ptrAnode->GetProperty("Value", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("1"), ValueStr );

}


void BooleanTestSuite::TestRegAccess()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Boolean Name=\"Trigger\">                \n"
    "    <pValue>TriggerReg</pValue>            \n"
    "    <OnValue>1</OnValue>                \n"
    "    <OffValue>0</OffValue>                \n"
    "</Boolean>                                \n"
    "                                        \n"
    "<IntReg Name=\"TriggerReg\">            \n"
    "    <Address>0x00ff</Address>            \n"
    "    <Length>1</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Unsigned</Sign>                \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<Port Name=\"Port\"/>                    \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x00ff, 0x0UL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CBooleanPtr ptrValue = Camera._GetNode("Trigger");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    CPPUNIT_ASSERT_EQUAL (false, ptrValue->IsValueCacheValid());
    CPPUNIT_ASSERT_EQUAL((bool)0, ptrValue->GetValue() );
    ////FAILS////CPPUNIT_ASSERT_EQUAL (true, ptrValue->IsValueCacheValid());
    ptrValue->SetValue( 1 );
    CPPUNIT_ASSERT_EQUAL((bool)1, ptrValue->GetValue() );

    ptrValue->SetValue( 0 );
    CPPUNIT_ASSERT_EQUAL((bool)0, ptrValue->GetValue() );

    ptrValue->FromString("1");
    gcstring gstr( "1" );
    CPPUNIT_ASSERT_EQUAL(gstr, ptrValue->ToString() );

    ptrValue->FromString("0");
    gstr = "0";
    CPPUNIT_ASSERT_EQUAL(gstr, ptrValue->ToString() );

    CPPUNIT_ASSERT_EQUAL( RW, ptrValue->GetAccessMode() );
}



void BooleanTestSuite::TestRegAccess1()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Boolean Name=\"Trigger\">                \n"
    "    <pValue>TriggerReg</pValue>            \n"
    "    <OnValue>0</OnValue>                \n"
    "    <OffValue>1</OffValue>                \n"
    "</Boolean>                                \n"
    "                                        \n"
    "<IntReg Name=\"TriggerReg\">            \n"
    "    <Address>0x00ff</Address>            \n"
    "    <Length>1</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Unsigned</Sign>                \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<Port Name=\"Port\"/>                    \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x00ff, 0x3UL, RW);  // for LOGICAL_ERROR_EXCEPTION in GetValue

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CBooleanPtr ptrValue = Camera._GetNode("Trigger");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    CPPUNIT_ASSERT_THROW_EX(
        ptrValue->GetValue(),
        GenICam::LogicalErrorException
    );
}


void BooleanTestSuite::TestAccessMode()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Boolean Name=\"Trigger\">                \n"
    "    <ImposedAccessMode>RO</ImposedAccessMode>  \n"
    "    <Value>1</Value>                    \n"
    "    <OnValue>1</OnValue>                \n"
    "    <OffValue>0</OffValue>                \n"
    "</Boolean>                                \n"
    "                                        \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CBooleanPtr ptrValue = Camera._GetNode("Trigger");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    CPPUNIT_ASSERT_EQUAL((bool)1, ptrValue->GetValue() );
    CPPUNIT_ASSERT_THROW (ptrValue->SetValue( 0), GenICam::OutOfRangeException);

    // just to follow happy path
    CPPUNIT_ASSERT_EQUAL (true, ptrValue->IsValueCacheValid());

}


void BooleanTestSuite::TestCornerCases()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    // JB: Value=7 is not allowed by the schema, changed to correct one
    //"    <Value>7</Value>                    \n"
    "<Boolean Name=\"Invalid\">                \n"
    "    <Value>1</Value>                    \n"
    "    <OnValue>7</OnValue>                \n"
    "    <OffValue>7</OffValue>                \n"
    "</Boolean>                                \n"
    "                                        \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_THROW (Camera._LoadXMLFromString(CameraDescriptionFile), GenICam::RuntimeException);

}

void BooleanTestSuite::TestPolyReference()
{
    // need to test the value access to boolean polyreference artificially
    // because as of this writing, it's not used in GenApi
    CBooleanPolyRef poly;
    CPPUNIT_ASSERT_EQUAL (false, poly.IsInitialized());
    CPPUNIT_ASSERT_EQUAL (false, poly.IsPointer());
    CPPUNIT_ASSERT_THROW (poly.SetValue (false), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW (poly.GetValue (), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW (poly.GetCachingMode (), GenICam::RuntimeException);
    poly = false;
    CPPUNIT_ASSERT_EQUAL (true, poly.IsInitialized());
    CPPUNIT_ASSERT_EQUAL (false, poly.IsPointer());
    CPPUNIT_ASSERT_EQUAL ((INode*)NULL, poly.GetPointer());
    gcstring str;
    Value2String (poly, str);
    CPPUNIT_ASSERT_EQUAL (gcstring("0"), str);
    String2Value (gcstring("1"), &poly);
    CPPUNIT_ASSERT_EQUAL (true, poly.GetValue());
    CPPUNIT_ASSERT_NO_THROW (poly.SetValue (false));
    CPPUNIT_ASSERT_EQUAL (false, poly.GetValue());
    CPPUNIT_ASSERT_EQUAL (WriteThrough, poly.GetCachingMode());
    CPPUNIT_ASSERT (!String2Value (gcstring("grrrgh"), &poly));

}

void BooleanTestSuite::TestOnOffValue()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Boolean Name=\"BoolValue\">                \n"
    "    <pValue>IntValue</pValue>                    \n"
    "    <OnValue>5</OnValue>                \n"
    "    <OffValue>3</OffValue>                \n"
    "</Boolean>                                \n"
    "                                        \n"
    "<Integer Name=\"IntValue\">                \n"
    "    <Value>1</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // test value which is not On or Off-value
    CBooleanPtr ptrBoolValue = Camera._GetNode("BoolValue");
    CPPUNIT_ASSERT( ptrBoolValue.IsValid() );
    CIntegerPtr ptrIntValue = Camera._GetNode("IntValue");
    CPPUNIT_ASSERT( ptrIntValue.IsValid() );

    CPPUNIT_ASSERT_THROW(ptrBoolValue->GetValue(), LogicalErrorException );

    ptrBoolValue->SetValue(false);
    CPPUNIT_ASSERT_EQUAL((int64_t)3LL, ptrIntValue->GetValue() );
    ptrBoolValue->SetValue(true);
    CPPUNIT_ASSERT_EQUAL((int64_t)5LL, ptrIntValue->GetValue() );


}

void BooleanTestSuite::TestPolyPointers()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Integer Name=\"Int\">            \n"
    "    <Value>1</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Node Name=\"SimpleNode\">            \n"
    "</Node>                                \n"
    "                                        \n"
    "<Boolean Name=\"Bool\">                \n"
    "    <Value>1</Value>                    \n"
    "</Boolean>                                \n"
    "                                        \n"
    "<Enumeration Name=\"Enum\">            \n"
    "    <EnumEntry Name=\"EnumValue0\">     \n"
    "        <Value>0</Value>                \n"
    "   </EnumEntry>                        \n"
    "    <EnumEntry Name=\"EnumValue1\">     \n"
    "        <Value>1</Value>                \n"
    "   </EnumEntry>                        \n"
    "    <Value>1</Value>                \n"
    "</Enumeration>                            \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // test bool->int
    CIntegerPtr ptrInt = Camera._GetNode("Int");
    CPPUNIT_ASSERT( ptrInt.IsValid() );
    CBooleanPolyRef polyBoolFromInt;
    polyBoolFromInt = (IInteger*)ptrInt;
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromInt.IsInitialized());
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromInt.IsPointer());
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromInt.GetValue());
    polyBoolFromInt.SetValue (false);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0LL, ptrInt->GetValue());
    polyBoolFromInt.SetValue (true);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1LL, ptrInt->GetValue());

    CNodePtr ptrNode = Camera._GetNode ("SimpleNode");
    CPPUNIT_ASSERT( ptrNode.IsValid() );
    CBooleanPolyRef polyBoolFromNode;
    CPPUNIT_ASSERT_THROW (polyBoolFromNode = (INode*)ptrNode, RuntimeException);
    CPPUNIT_ASSERT_EQUAL (false, polyBoolFromNode.IsInitialized());

    // test bool->bool
    CBooleanPtr ptrBool = Camera._GetNode("Bool");
    CPPUNIT_ASSERT( ptrBool.IsValid() );
    CBooleanPolyRef polyBoolFromBool;
    polyBoolFromBool = (IBoolean*)ptrBool;
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromBool.IsInitialized());
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromBool.IsPointer());
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromBool.GetValue());
    polyBoolFromBool.SetValue (false);
    CPPUNIT_ASSERT_EQUAL (false, ptrBool->GetValue());
    polyBoolFromBool.SetValue (true);
    CPPUNIT_ASSERT_EQUAL (true, ptrBool->GetValue());

    // test bool->enum
    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CPPUNIT_ASSERT( ptrEnum.IsValid() );
    CBooleanPolyRef polyBoolFromEnum;
    polyBoolFromEnum = (IEnumeration*)ptrEnum;
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromEnum.IsInitialized());
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromEnum.IsPointer());
    CPPUNIT_ASSERT_EQUAL (true, polyBoolFromEnum.GetValue());
    CPPUNIT_ASSERT_EQUAL (dynamic_cast<INode*>((IEnumeration*)ptrEnum), polyBoolFromEnum.GetPointer());
    polyBoolFromEnum.SetValue (false);
    CPPUNIT_ASSERT_EQUAL (gcstring("EnumValue0"), ptrEnum->ToString());
    polyBoolFromEnum.SetValue (true);
    CPPUNIT_ASSERT_EQUAL (gcstring("EnumValue1"), ptrEnum->ToString());
    CPPUNIT_ASSERT_EQUAL (WriteThrough, polyBoolFromEnum.GetCachingMode());

}
