//-----------------------------------------------------------------------------
//  (c) 2009 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Utility for testing callbacks
*/
#if _MSC_VER > 1000
#pragma once
#endif

#ifndef _STRUCTPORT_H_
#define _STRUCTPORT_H_

#include <strstream>

namespace GenApi
{
    //! Helper class to test callbacks
    class CCallbackTestTarget
    {
    public:
        //! Constructor
        CCallbackTestTarget()
        {
            Reset();
        }

        //! Constructor registering automatically
        CCallbackTestTarget( IBase* pBase )
        {
            Reset();
            CNodePtr ptrNode(pBase);
            assert(ptrNode.IsValid());
            Register(ptrNode, *this, &CCallbackTestTarget::Callback);
        }

        //! Reset the callback counter
        void Reset()
        {
            m_Count=0;
        }

        //! the callback function to be registers
        /*! The registration code looks like this;
            Register( ptrFeature->GetNode(), Callback, &CCallbackTestTarget::Callback );
        */
        void Callback( INode* )
        {
            ++m_Count;
        }

        //! true if the callback has fired exactly once
        /*! If true Reset() is called implicitely */
        bool HasFiredOnce()
        {
            if(m_Count == 1)
            {
                Reset();
                return true;
            }
            else
            {
#ifdef WIN32
                std::ostrstream Buffer;
                Buffer << "WARNING : CCallbackTestTarget::HasFiredOnce : Count = " << Count() << std::endl << std::ends;
                OutputDebugStr( Buffer.str() );
#endif
                return false;
            }
        }

        //! true if the callback has nof fired
        bool HasNotFired()
        {
            if(m_Count == 0)
            {
                return true;
            }
            else
            {
#ifdef WIN32
                std::ostrstream Buffer;
                Buffer << "WARNING : CCallbackTestTarget::HasNotFired : Count = " << Count() << std::endl << std::ends;
                OutputDebugStr( Buffer.str() );
#endif
                return false;
            }
        }

        //! Tells the number of callbacks,
        uint32_t Count()
        {
            return m_Count;
        }
    private:
        //! counts the number of callbakcs
        unsigned m_Count;
    };
}

#endif // _STRUCTPORT_H_
