@echo off

setlocal

REM ============================================
:RunCMake

set BUILDDIR=Linux32_i86_
echo Running CMake 
echo Using BUILDDIR='%BUILDDIR%'

set errorstring="Error while running CMake"
if exist %BUILDDIR% goto :PrepareForBuild
md %BUILDDIR%
cd %BUILDDIR%
if /i "%BUILDDIR%" equ "win32_i86_VS71" cmake -G "Visual Studio 7 .NET 2003" ../
if /i "%BUILDDIR%" equ "win32_i86_VS80" cmake -G "Visual Studio 8 2005" ../
if /i "%BUILDDIR%" equ "win64_x64_VS80" cmake -G "Visual Studio 8 2005 Win64" ../
if /i "%BUILDDIR%" equ "win32_i86_VS90" cmake -G "Visual Studio 9 2008" ../
if /i "%BUILDDIR%" equ "win64_x64_VS90" cmake -G "Visual Studio 9 2008 Win64" ../
cd..
if %errorlevel% NEQ 0 goto error

REM ===========================================
:PrepareForBuild
if exist GenApiTest_BuildLog.txt del GenApiTest_BuildLog.txt > nul
if exist GenApiTest_TestLog.txt del GenApiTest_TestLog.txt > nul

REM ===========================================
:RunIncredibuild

REM check if Incredibuild is available
BuildConsole.exe /? > nul
if %errorlevel% NEQ 0 goto RunStudio

echo Running VisualStudio via Incredibuild
set errorstring="Error while running VisualStudio via Incredibuild"
BuildConsole.exe "%BUILDDIR%/GenApiTest.sln" /Build /cfg="Release|" /beep /Log="GenApiTest_BuildLog.txt"
if %errorlevel% NEQ 0 goto error

goto RunTests

REM ===========================================
:RunStudio


if /i "%BUILDDIR%" equ "win32_i86_VS71" set STUDIO=VS71
if /i "%BUILDDIR%" equ "win32_i86_VS80" set STUDIO=VS80
if /i "%BUILDDIR%" equ "win64_x64_VS80" set STUDIO=VS80
if /i "%BUILDDIR%" equ "win32_i86_VS90" set STUDIO=VS90
if /i "%BUILDDIR%" equ "win64_x64_VS90" set STUDIO=VS90
echo Running VisualStudio
echo STUDIO='%STUDIO%'

set errorstring="Error while running VisualStudio"

if /i "%STUDIO%" == "VS90" (
	if exist "%VS90COMNTOOLS%..\..\VC\bin\x86_amd64\vcvarsx86_amd64.bat" (
			call "%VS90COMNTOOLS%..\..\VC\vcvarsall.bat" x86_amd64 
		) ELSE (
			call "%VS90COMNTOOLS%..\..\VC\vcvarsall.bat" x86 & ECHO !!! Warning : the x64 compiler is not installed on this machine
		)	
	)
	
if /i "%STUDIO%" == "VS80" (
	if exist "%VS80COMNTOOLS%..\..\VC\bin\x86_amd64\vcvarsx86_amd64.bat" (
			call "%VS80COMNTOOLS%..\..\VC\vcvarsall.bat" x86_amd64 
		) ELSE (
			call "%VS80COMNTOOLS%..\..\VC\vcvarsall.bat" x86 & ECHO !!! Warning : the x64 compiler is not installed on this machine
		)	
	)
	
if /i "%STUDIO%" == "VS80express" call "%VS80COMNTOOLS%vsvars32.bat"
if /i "%STUDIO%" == "VS80express" @set DevEnvDir=%VS80COMNTOOLS%..\IDE

if /i "%STUDIO%" == "VS71" call "%VS71COMNTOOLS%vsvars32.bat"

ECHO Running Visual Studio
del GenApiTest_BuildLog.txt
if "%STUDIO%" == "VS71" (
	"%DevEnvDir%\devenv.exe" "%BUILDDIR%/GenApiTest.sln" /Build "Release" /out GenApiTest_BuildLog.txt
) else (
	"%DevEnvDir%\devenv.exe" "%BUILDDIR%/GenApiTest.sln" /Build "Release|" /out GenApiTest_BuildLog.txt
)
if %errorlevel% NEQ 0 goto error

goto RunTests

REM ===========================================
:RunTests

ECHO Running GenApiTests...
set errorstring="Error while running GenApiTest"
DllPluginTester_gcc40_v2_0.exe -w "%GENICAM_ROOT_V2_0%/bin/Linux32_i86/GenApiTest_gcc40_v2_0.dll" 
if %errorlevel% NEQ 0 goto error

ECHO GenApiTest successfull

goto end

REM ===========================================
:error

echo Fatal error! 
echo %errorstring%

pause
goto end

REM ===========================================
:end


endlocal

REM Just to remember how it is done:
REM This Version provides a copy of the test output in GenApiTest_TestLog.txt but requires Cygwin because of tee.exe
REM note the output trick "1> GenApiTest_TestLog.txt 2>&1", see http://support.microsoft.com/kb/110930
REM and the useage ot tee like dir file.xxx 2>&1 | tee bla.txt
REM DllPluginTester_gcc40_v2_0.exe %GENICAM_ROOT_V1_2%/bin/Win32_i86/GenApiTest_gcc40_v2_0.dll 2>&1 | tee GenApiTest_TestLog.txt 

