//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Hartmut Nebelung
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of IConfigurationRom
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "ConfRomTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

#include "GenApi/NodeCallbackImpl.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

#undef min

CPPUNIT_TEST_SUITE_REGISTRATION( ConfigurationRomTestSuite );

struct cb
{
  unsigned m_Count;
  cb() : m_Count(0) {}
  void doCount( INode* ) { ++m_Count;    }
  unsigned Count() const { return m_Count; }
};

// Test a selector of integer type
void ConfigurationRomTestSuite::TestConfigurationRom01()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +

    "<ConfRom Name=\"ConfRom\"> \n"
    "   <ToolTip>Configuration Rom of IIDC Digital Camera</ToolTip> \n"
    "    <Unit>0x00A02D</Unit> \n"
    "    <Address>0x0</Address> \n"
    "    <Length>0x400</Length> \n"
    "    <pPort>MyPort</pPort> \n"
    "    <TextDesc Name=\"VendorName\">0x81</TextDesc> \n"
    "   <IntKey Name=\"BaseAdr\">0x40</IntKey> \n"
    "</ConfRom> \n"

    "<Port Name=\"MyPort\"/>\n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;

    uint32_t ConfRom[400];           //address 0x0 to 0x639 (refer to SampleCamera.xml)
    ConfRom[0] = 0x00000404; // len; crc
    ConfRom[1] = 0x34393331; // 0x31333934; // bus
    ConfRom[2] = 0x00ff0001; // 0x0100ff00; // bus dep
    ConfRom[3] = 0x12555030; // 0x30505512; // com id; chip id hi
    ConfRom[4] = 0x80785434; // 0x34567880; // chip id lo
    ConfRom[5] = 0x00000300; // 0x00030000; // root directory
    ConfRom[6] = 0x56341204; // 0x03123456; // vendor id
    ConfRom[7] = 0x02000081; // 0x81000002; // text descriptor offset
    ConfRom[8] = 0x050000D1; // 0xD1000005; // unit dir offset
    ConfRom[9] = 0x00000300; // 0x00030000; // Text descriptor: len; crc
    ConfRom[10]= 0x00000000; // type, specifier
    ConfRom[11]= 0x00000000; // width, char set, lang
    ConfRom[12]= 0x454D4341; // ACME (reverse because of endianess)
    ConfRom[13]= 0x00000300; // 0x00030000; // Unit Directory: len, crc
    ConfRom[14]= 0x2da00012; // 0x1200a02d; // DCAM
    ConfRom[15]= 0x02010013; // 0x13000102; // Version
    ConfRom[16]= 0x010000D4; // 0xD4000001; // UnitDep
    ConfRom[17]= 0x00000100; // 0x00010000; // Unit Dependent Directory: len, crc
    ConfRom[18]= 0x00040040; // 0x40000400; // Command Register Base

    Port.CreateEntry(0x0, sizeof ConfRom, ConfRom, RO);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

    CStringPtr ptrVendor = Camera._GetNode( "VendorName" );
    CPPUNIT_ASSERT( ptrVendor.IsValid() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrVendor->GetAccessMode() );
    gcstring vendor( "ACME" );
    CPPUNIT_ASSERT_EQUAL( vendor, ptrVendor->GetValue() );
    CPPUNIT_ASSERT_THROW_EX( ptrVendor->SetValue( "Foo" ), GenICam::AccessException );
     CPPUNIT_ASSERT_EQUAL( vendor, ptrVendor->ToString() );

    // get the base adr
    CIntegerPtr ptrBaseAdr = Camera._GetNode( "BaseAdr" );
    CPPUNIT_ASSERT( ptrBaseAdr.IsValid() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrBaseAdr->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0x400, ptrBaseAdr->GetValue() );
    CPPUNIT_ASSERT_THROW_EX( ptrBaseAdr->SetValue( 0x0 ), GenICam::AccessException );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrBaseAdr->GetRepresentation() );
}




//
// Test a configuration with several descriptor for the vendor name
// in different languages. The Language is selected by a language
// attribute. Currently only the default language is supported by
// the implementation.
// This is not compatible to the IIDC specification.
//
void ConfigurationRomTestSuite::TestConfigurationRom02()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +

    "<ConfRom Name=\"ConfRom\"> \n"
    "   <ToolTip>Configuration Rom of IIDC Digital Camera</ToolTip> \n"
    "    <Unit>0x00A02D</Unit> \n"
    "    <Address>0x0</Address> \n"
    "    <Length>0x400</Length> \n"
    "    <pPort>MyPort</pPort> \n"
    "    <TextDesc Name=\"VendorName\">0x03</TextDesc> \n"
    "    <TextDesc Name=\"ModelName\">0x82</TextDesc> \n"
    "   <IntKey Name=\"BaseAdr\">0x40</IntKey> \n"
    "</ConfRom> \n"

    "<Port Name=\"MyPort\"/>\n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;

    uint32_t ConfRom[400];           //address 0x0 to 0x639 (refer to SampleCamera.xml)
    ConfRom[0] = 0x00000404; // 0x04040000; // len; crc
    ConfRom[1] = 0x34393331; // 0x31333934; // bus
    ConfRom[2] = 0x00ff0001; // 0x0100ff00; // bus dep
    ConfRom[3] = 0x12555030; // 0x30505512; // com id; chip id hi
    ConfRom[4] = 0x80785634; // 0x34567880; // chip id lo
    ConfRom[5] = 0x00000300; // 0x00030000; // root directory
    ConfRom[6] = 0x56351203; // 0x03123456; // vendor id
    ConfRom[7] = 0x020000C1; // 0xC1000002; // text descriptor directory offset
    ConfRom[8] = 0x110000D1; // 0xD1000011; // unit dir offset

    ConfRom[9] = 0x00000300; // 0x00030000; // descriptor directory
    ConfRom[10]= 0x03000081; // 0x81000003; // text descriptor directory offset
    ConfRom[11]= 0x06000081; // 0x81000006; // text descriptor directory offset
    ConfRom[12]= 0x09000081; // 0x81000009; // text descriptor directory offset

    ConfRom[13]= 0x00000300; // 0x00030000; // Text descriptor: len; crc
    ConfRom[14]= 0x00000000; // type, specifier
    ConfRom[15]= 0xA0100000; // 0x00000000 | (4<<10) | (5<<5); // width, char set, lang=DE
    ConfRom[16]= 0x48424D47; // GMBH (reverse because of endianess)

    ConfRom[17]= 0x00000300; // 0x00030000; // Text descriptor: len; crc
    ConfRom[18]= 0x00000000; // type, specifier
    ConfRom[19]= 0x00000000; // width, char set, lang
    ConfRom[20]= 0x454D4341; // ACME (reverse because of endianess)

    ConfRom[21]= 0x00000300; // 0x00030000; // Text descriptor: len; crc
    ConfRom[22]= 0x00000000; // type, specifier
    ConfRom[23]= 0xA0280000;// 0x00000000 | (5<<10) | (14<<5); // width, char set, lang=EN
    ConfRom[24]= 0x48424D47; // GMBH (reverse because of endianess)

    ConfRom[25]= 0x00000300; // 0x00030000; // Unit Directory: len, crc
    ConfRom[26]= 0x2da00012; // 0x1200a02d; // DCAM
    ConfRom[27]= 0x02020013; // 0x13000102; // Version
    ConfRom[28]= 0x010000D4; // 0xD4000001; // UnitDep
    ConfRom[29]= 0x00000100; // 0x00010000; // Unit Dependent Directory: len, crc
    ConfRom[30]= 0x00040040; // 0x40000400; // Command Register Base
    Port.CreateEntry(0x0, sizeof ConfRom, ConfRom, RO);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

    // get the vendor name
    CStringPtr ptrVendor = Camera._GetNode( "VendorName" );
    CPPUNIT_ASSERT( ptrVendor.IsValid() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrVendor->GetAccessMode() );
    gcstring vendor( "ACME" );
    CPPUNIT_ASSERT_EQUAL( vendor, ptrVendor->GetValue() );
    CPPUNIT_ASSERT_THROW_EX( ptrVendor->SetValue( "Foo" ), GenICam::AccessException );

    // get the base adr
    CIntegerPtr ptrBaseAdr = Camera._GetNode( "BaseAdr" );
    CPPUNIT_ASSERT( ptrBaseAdr.IsValid() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrBaseAdr->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0x400, ptrBaseAdr->GetValue() );
    CPPUNIT_ASSERT_THROW_EX( ptrBaseAdr->SetValue( 0x0 ), GenICam::AccessException );

    // get the model name
    CStringPtr ptrModel = Camera._GetNode( "ModelName" );
    CPPUNIT_ASSERT( ptrModel.IsValid() );
    CPPUNIT_ASSERT_THROW_EX( ptrModel->GetValue(), GenICam::RuntimeException );

}

