//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#pragma warning ( disable : 4068 ) // unknown pragma; refers tio BullsEyeCoverage
#pragma warning ( disable : 4251 ) // XXX needs to have dll-interface to be used by clients of class YYY

#include "ConverterTest.h"
#include "NodeMapTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include <limits>
#include "GenApi/impl/Converter.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( ConverterTestSuite );

void ConverterTestSuite::TestPixelFormat()
{
    CNodeMapRef Camera;
    Camera._LoadXMLFromFile(XMLPATH "Test/IntConverterTest.xml");

    CIntegerPtr ptrColorCode = Camera._GetNode("ColorCode");
    CPPUNIT_ASSERT( ptrColorCode.IsValid() );

    CIntegerPtr ptrColorFilterID = Camera._GetNode("ColorFilterID");
    CPPUNIT_ASSERT( ptrColorFilterID.IsValid() );

    CEnumerationPtr ptrPixelFormat = Camera._GetNode("PixelFormat");
    CPPUNIT_ASSERT( ptrPixelFormat.IsValid() );

    CIntegerPtr ptrSensorColorFilter_RG = Camera._GetNode("SensorColorFilter_RG");
    CPPUNIT_ASSERT( ptrSensorColorFilter_RG.IsValid() );

    CIntegerPtr ptrSensorColorFilter_GR = Camera._GetNode("SensorColorFilter_GR");
    CPPUNIT_ASSERT( ptrSensorColorFilter_GR.IsValid() );

    CIntegerPtr ptrSensorColorFilter_BG = Camera._GetNode("SensorColorFilter_BG");
    CPPUNIT_ASSERT( ptrSensorColorFilter_BG.IsValid() );

    CIntegerPtr ptrSensorColorFilter_GB = Camera._GetNode("SensorColorFilter_GB");
    CPPUNIT_ASSERT( ptrSensorColorFilter_GB.IsValid() );


    *ptrColorCode = 0;
    CPPUNIT_ASSERT_EQUAL( gcstring("Mono8"), *ptrPixelFormat());

    *ptrColorCode = 1;
    CPPUNIT_ASSERT_EQUAL( gcstring("YUV411Packed"), *ptrPixelFormat());

    *ptrColorCode = 2;
    CPPUNIT_ASSERT_EQUAL( gcstring("YUV422Packed"), *ptrPixelFormat());

    *ptrColorCode = 3;
    CPPUNIT_ASSERT_EQUAL( gcstring("YUV444Packed"), *ptrPixelFormat());

    *ptrColorCode = 4;
    CPPUNIT_ASSERT_EQUAL( gcstring("RGB8Packed"), *ptrPixelFormat());

    *ptrColorCode = 5;
    CPPUNIT_ASSERT_EQUAL( gcstring("Mono14"), *ptrPixelFormat());

    *ptrColorCode = 6;
    CPPUNIT_ASSERT_EQUAL( gcstring("RGB14Packed"), *ptrPixelFormat());

    *ptrColorCode = 129;
    CPPUNIT_ASSERT_EQUAL( gcstring("Mono12Packed"), *ptrPixelFormat());

    // Raw8
    *ptrColorCode = 9;

    *ptrColorFilterID = 0;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerRG8"), *ptrPixelFormat());

    *ptrColorFilterID = 1;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerGB8"), *ptrPixelFormat());

    *ptrColorFilterID = 2;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerGR8"), *ptrPixelFormat());

    *ptrColorFilterID = 3;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerBG8"), *ptrPixelFormat());

    // Raw16
    *ptrColorCode = 10;

    *ptrColorFilterID = 0;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerRG14"), *ptrPixelFormat());

    *ptrColorFilterID = 1;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerGB14"), *ptrPixelFormat());

    *ptrColorFilterID = 2;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerGR14"), *ptrPixelFormat());

    *ptrColorFilterID = 3;
    CPPUNIT_ASSERT_EQUAL( gcstring("BayerBG14"), *ptrPixelFormat());

    /* -------------------- */

    *ptrPixelFormat = "Mono8";
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, *ptrColorCode());

    *ptrPixelFormat = "YUV411Packed";
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, *ptrColorCode());

    *ptrPixelFormat = "YUV422Packed";
    CPPUNIT_ASSERT_EQUAL( (int64_t)2, *ptrColorCode());

    *ptrPixelFormat = "YUV444Packed";
    CPPUNIT_ASSERT_EQUAL( (int64_t)3, *ptrColorCode());

    *ptrPixelFormat = "RGB8Packed";
    CPPUNIT_ASSERT_EQUAL( (int64_t)4, *ptrColorCode());

    *ptrPixelFormat = "Mono14";
    CPPUNIT_ASSERT_EQUAL( (int64_t)5, *ptrColorCode());

    *ptrPixelFormat = "RGB14Packed";
    CPPUNIT_ASSERT_EQUAL( (int64_t)6, *ptrColorCode());

    *ptrPixelFormat = "Mono12Packed";
    CPPUNIT_ASSERT_EQUAL( (int64_t)129, *ptrColorCode());

    // Raw8
    *ptrColorFilterID = 0;  // the color filter stayes unchanged

    *ptrPixelFormat = "BayerRG8";
    CPPUNIT_ASSERT_EQUAL( (int64_t)9, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, *ptrColorFilterID());

    *ptrPixelFormat = "BayerGB8";
    CPPUNIT_ASSERT_EQUAL( (int64_t)9, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, *ptrColorFilterID());

    *ptrPixelFormat = "BayerGR8";
    CPPUNIT_ASSERT_EQUAL( (int64_t)9, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, *ptrColorFilterID());

    *ptrPixelFormat = "BayerBG8";
    CPPUNIT_ASSERT_EQUAL( (int64_t)9, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, *ptrColorFilterID());

    // Raw16
    *ptrColorFilterID = 1;  // the color filter stayes unchanged

    *ptrPixelFormat = "BayerRG14";
    CPPUNIT_ASSERT_EQUAL( (int64_t)10, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, *ptrColorFilterID());

    *ptrPixelFormat = "BayerGB14";
    CPPUNIT_ASSERT_EQUAL( (int64_t)10, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, *ptrColorFilterID());

    *ptrPixelFormat = "BayerGR14";
    CPPUNIT_ASSERT_EQUAL( (int64_t)10, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, *ptrColorFilterID());

    *ptrPixelFormat = "BayerBG14";
    CPPUNIT_ASSERT_EQUAL( (int64_t)10, *ptrColorCode());
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, *ptrColorFilterID());

}


void ConverterTestSuite::TestReadModifyWrite()
{

    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                            \n"
        "<Converter Name=\"ShutterAbs\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> TO * TIMEBASE </FormulaFrom>                \n"
        "    <pValue>ShutterRaw</pValue>                                \n"
      "  <Unit>s</Unit>                   \n"
        "    <Slope>Increasing</Slope>                                \n"
        "</Converter>                                                \n"
        "                                                            \n"
        "<Integer Name=\"ShutterRaw\">                                \n"
        "    <Value>2</Value>                                        \n"
        "</Integer>                                                    \n"
        "                                                            \n"
        "<Integer Name=\"TimeBase\">                                \n"
        "    <Value>10</Value>                                        \n"
        "</Integer>                                                    \n"
        "                                                            \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrShutterAbs = Camera._GetNode("ShutterAbs");
    CPPUNIT_ASSERT( ptrShutterAbs.IsValid() );
    CPPUNIT_ASSERT_EQUAL (intfIFloat, ptrShutterAbs->GetNode()->GetPrincipalInterfaceType());

    CIntegerPtr ptrShutterRaw = Camera._GetNode("ShutterRaw");
    CPPUNIT_ASSERT( ptrShutterRaw.IsValid() );

    CIntegerPtr ptrTimeBase = Camera._GetNode("TimeBase");
    CPPUNIT_ASSERT( ptrTimeBase.IsValid() );

   // check the converter's units
   CPPUNIT_ASSERT_EQUAL( ::gcstring("s") , ptrShutterAbs->GetUnit());

    // reading
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 20.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

    *ptrShutterRaw = 3;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 30.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

    *ptrTimeBase = 100;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 300.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

    // Writing
    *ptrShutterAbs = 400.0;
    CPPUNIT_ASSERT_EQUAL( (int64_t)4, *ptrShutterRaw());
}

void ConverterTestSuite::TestReadModifyWriteInt()
{
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                            \n"
        "<IntConverter Name=\"ShutterAbs\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> TO * TIMEBASE </FormulaFrom>                \n"
        "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Slope>Increasing</Slope>                                \n"
        "</IntConverter>                                            \n"
        "                                                            \n"
        "<Integer Name=\"ShutterRaw\">                                \n"
        "    <Value>2</Value>                                        \n"
        "    <Min>0</Min>                                            \n"
        "    <Max>4096</Max>                                            \n"
        "</Integer>                                                    \n"
        "                                                            \n"
        "<Integer Name=\"TimeBase\">                                \n"
        "    <Value>10</Value>                                        \n"
        "</Integer>                                                    \n"
      "                                                            \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrShutterAbs = Camera._GetNode("ShutterAbs");
    CPPUNIT_ASSERT( ptrShutterAbs.IsValid() );
    CPPUNIT_ASSERT_EQUAL (intfIInteger, ptrShutterAbs->GetNode()->GetPrincipalInterfaceType());

    CIntegerPtr ptrShutterRaw = Camera._GetNode("ShutterRaw");
    CPPUNIT_ASSERT( ptrShutterRaw.IsValid() );

    CIntegerPtr ptrTimeBase = Camera._GetNode("TimeBase");
    CPPUNIT_ASSERT( ptrTimeBase.IsValid() );

    // reading
    CPPUNIT_ASSERT_EQUAL( (int64_t)20, *ptrShutterAbs());

    *ptrShutterRaw = 3;
    CPPUNIT_ASSERT_EQUAL( (int64_t)30, *ptrShutterAbs());

    *ptrTimeBase = 100;
    CPPUNIT_ASSERT_EQUAL( (int64_t)300, *ptrShutterAbs());

    // Writing
    *ptrShutterAbs = 400;
    CPPUNIT_ASSERT_EQUAL( (int64_t)4, *ptrShutterRaw());

}

void ConverterTestSuite::TestAutomatic()
{

    if(GenApiSchemaVersion == v1_0)
        return;


    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                            \n"
        "<Converter Name=\"IntInc\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> TO * TIMEBASE </FormulaFrom>                \n"
        "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                \n"
        "</Converter>                                            \n"
        "                                                            \n"
        "<Converter Name=\"IntDec\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> 0 - TO + 10 </FormulaFrom>                \n"
      "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                 \n"
        "</Converter>                                            \n"
      "                                                            \n"
        "                                                            \n"
        "<Converter Name=\"FloatInc\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> TO * TIMEBASE </FormulaFrom>                \n"
      "    <pValue>FloatShutterRaw</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                \n"
        "</Converter>                                            \n"
        "                                                            \n"
        "<Converter Name=\"FloatDec\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> 0 - TO + 10 </FormulaFrom>                \n"
      "    <pValue>FloatShutterRaw</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                \n"
        "</Converter>                                            \n"
      "                                                            \n"
        "                                                            \n"
        "<Converter Name=\"Wobble\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> 0 - TO + 10 </FormulaFrom>                \n"
      "    <pValue>FloatShutterRaw</pValue>                                \n"
        "    <Slope>Varying</Slope>                                \n"
        "</Converter>                                            \n"
      "                                                            \n"
        "<Integer Name=\"ShutterRaw\">                                \n"
        "    <Value>2</Value>                                        \n"
        "    <Min>1</Min>                                            \n"
        "    <Max>5</Max>                                            \n"
      "    <Representation>Linear</Representation>\n"
        "</Integer>                                                    \n"
        "                                                            \n"
      "                                                            \n"
        "<Float Name=\"FloatShutterRaw\">                                \n"
        "    <Value>2.0</Value>                                        \n"
        "    <Min>1.0</Min>                                            \n"
        "    <Max>5.0</Max>                                            \n"
      "    <Representation>PureNumber</Representation>\n"
        "</Float>                                                    \n"
        "                                                            \n"
        "<Integer Name=\"TimeBase\">                                \n"
        "    <Value>2</Value>                                        \n"
        "</Integer>                                                    \n"
      "                                                            \n"
        "                                                            \n"
        "<Converter Name=\"IntDec_OwnRepre\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> 0 - TO + 10 </FormulaFrom>                \n"
      "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Unit>my_unit</Unit>\n"
        "    <Representation>Logarithmic</Representation>                                 \n"
        "    <Slope>Automatic</Slope>                                 \n"
        "</Converter>                                            \n"
      "                                                            \n"
        "<Float Name=\"FloatWithUnit\">                                \n"
        "    <Value>2.0</Value>                                        \n"
        "    <Unit>fltu</Unit>\n"
        "</Float>                                                    \n"
        "                                                            \n"
        "<Float Name=\"FloatNoUnit\">                                \n"
        "    <Value>2.0</Value>                                        \n"
        "</Float>                                                    \n"
        "                                                            \n"
        "<Integer Name=\"IntWithUnit\">                                \n"
        "    <Value>2</Value>                                        \n"
        "    <Unit>intu</Unit>\n"
        "</Integer>                                                    \n"
      "                                                            \n"
        "<Converter Name=\"IntCToFloat\">                            \n"
        "    <FormulaTo> FROM </FormulaTo>                \n"
        "    <FormulaFrom> TO </FormulaFrom>                \n"
      "    <pValue>FloatWithUnit</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                 \n"
        "</Converter>                                            \n"
      "                                                            \n"
        "<Converter Name=\"IntCToInt\">                            \n"
        "    <FormulaTo> FROM </FormulaTo>                \n"
        "    <FormulaFrom> TO </FormulaFrom>                \n"
      "    <pValue>IntWithUnit</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                 \n"
        "</Converter>                                            \n"
      "                                                            \n"
        "<Converter Name=\"IntCToFloat2\">                            \n"
        "    <FormulaTo> FROM </FormulaTo>                \n"
        "    <FormulaFrom> TO </FormulaFrom>                \n"
      "    <pValue>FloatNoUnit</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                 \n"
        "</Converter>                                            \n"
      "                                                            \n"
        + CDFFooter;

   CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

   CFloatPtr ptrIntInc = Camera._GetNode("IntInc");
   CPPUNIT_ASSERT( ptrIntInc.IsValid() );

   CFloatPtr ptrIntDec = Camera._GetNode("IntDec");
   CPPUNIT_ASSERT( ptrIntDec.IsValid() );

   CFloatPtr ptrFloatInc = Camera._GetNode("FloatInc");
   CPPUNIT_ASSERT( ptrFloatInc.IsValid() );

   CFloatPtr ptrFloatDec = Camera._GetNode("FloatDec");
   CPPUNIT_ASSERT( ptrFloatDec.IsValid() );

   CFloatPtr ptrWobble = Camera._GetNode("Wobble");
   CPPUNIT_ASSERT( ptrWobble.IsValid() );

    CIntegerPtr ptrShutterRaw = Camera._GetNode("ShutterRaw");
    CPPUNIT_ASSERT( ptrShutterRaw.IsValid() );

   CFloatPtr ptrFloatShutterRaw = Camera._GetNode("FloatShutterRaw");
   CPPUNIT_ASSERT( ptrFloatShutterRaw.IsValid() );

    CIntegerPtr ptrTimeBase = Camera._GetNode("TimeBase");
    CPPUNIT_ASSERT( ptrTimeBase.IsValid() );

   //Test automatic slopes
   CPPUNIT_ASSERT( ptrIntInc->GetMin() == 2 );
   CPPUNIT_ASSERT( ptrIntInc->GetMax() == 10 );

   CPPUNIT_ASSERT( ptrIntDec->GetMin() == 5 );
   CPPUNIT_ASSERT( ptrIntDec->GetMax() == 9 );

   CPPUNIT_ASSERT( ptrFloatInc->GetMin() == 2 );
   CPPUNIT_ASSERT( ptrFloatInc->GetMax() == 10 );

   CPPUNIT_ASSERT( ptrFloatDec->GetMin() == 5 );
   CPPUNIT_ASSERT( ptrFloatDec->GetMax() == 9 );

   CPPUNIT_ASSERT( ptrWobble->GetMin() < ptrWobble->GetMax() );

   //Do some gets and sets
   CPPUNIT_ASSERT( ptrFloatInc->GetValue() == 4 );
   ptrFloatInc->SetValue(6);
   CPPUNIT_ASSERT( ptrFloatInc->GetValue() == 6 );

   CPPUNIT_ASSERT( ptrFloatDec->GetValue() == 7 );
   ptrFloatInc->SetValue(8);
   CPPUNIT_ASSERT( ptrFloatDec->GetValue() == 6 );

   //Check the representations
   CPPUNIT_ASSERT( ptrIntDec->GetRepresentation() == Linear );
   CPPUNIT_ASSERT( ptrFloatInc->GetRepresentation() == PureNumber );
   CPPUNIT_ASSERT( ptrFloatInc->GetRepresentation() == PureNumber ); // second call for happy path
   CFloatPtr ptrIntDec_OwnRepre = Camera._GetNode("IntDec_OwnRepre");
   CPPUNIT_ASSERT( ptrIntDec_OwnRepre.IsValid() );
   CPPUNIT_ASSERT( ptrIntDec_OwnRepre->GetRepresentation() == Logarithmic );

   // Check the units
   CFloatPtr ptrIntCToFloat = Camera._GetNode("IntCToFloat");
   CPPUNIT_ASSERT( ptrIntCToFloat.IsValid() );
   CFloatPtr ptrIntCToInt = Camera._GetNode("IntCToInt");
   CPPUNIT_ASSERT( ptrIntCToInt.IsValid() );
   CFloatPtr ptrIntCToFloat2 = Camera._GetNode("IntCToFloat2");
   CPPUNIT_ASSERT( ptrIntCToFloat2.IsValid() );
   CPPUNIT_ASSERT_EQUAL (gcstring("fltu"), ptrIntCToFloat->GetUnit());
   CPPUNIT_ASSERT_EQUAL (gcstring("intu"), ptrIntCToInt->GetUnit());
   CPPUNIT_ASSERT_EQUAL (gcstring("my_unit"), ptrIntDec_OwnRepre->GetUnit());
   CPPUNIT_ASSERT_EQUAL (gcstring(""), ptrIntCToFloat2->GetUnit());

}

void ConverterTestSuite::TestIntAutomatic()
{
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                            \n"
        "<IntConverter Name=\"IntInc\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> TO * TIMEBASE </FormulaFrom>                \n"
        "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                \n"
        "</IntConverter>                                            \n"
        "                                                            \n"
        "<IntConverter Name=\"IntDec\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> 0 - TO + 10 </FormulaFrom>                \n"
      "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                 \n"
        "</IntConverter>                                            \n"
      "                                                            \n"
      "<IntConverter Name=\"Wobble\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> 0 - TO + 10 </FormulaFrom>                \n"
      "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Slope>Varying</Slope>                                \n"
        "</IntConverter>                                            \n"
      "                                                            \n"
        "<Integer Name=\"ShutterRaw\">                                \n"
        "    <Value>2</Value>                                        \n"
        "    <Min>1</Min>                                            \n"
        "    <Max>5</Max>                                            \n"
      "    <Representation>Linear</Representation>\n"
        "</Integer>                                                    \n"
        "                                                            \n"
        "<Integer Name=\"TimeBase\">                                \n"
        "    <Value>2</Value>                                        \n"
        "</Integer>                                                    \n"
      "                                                            \n"
        "<IntConverter Name=\"IntDec_OwnRepre\">                            \n"
        "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
        "    <FormulaTo> FROM / TIMEBASE </FormulaTo>                \n"
        "    <FormulaFrom> 0 - TO + 10 </FormulaFrom>                \n"
      "    <pValue>ShutterRaw</pValue>                                \n"
        "    <Representation>HexNumber</Representation>                                 \n"
        "    <Slope>Automatic</Slope>                                 \n"
        "</IntConverter>                                            \n"
      "                                                            \n"
        + CDFFooter;

   CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

   CIntegerPtr ptrIntInc = Camera._GetNode("IntInc");
   CPPUNIT_ASSERT( ptrIntInc.IsValid() );

   CIntegerPtr ptrIntDec = Camera._GetNode("IntDec");
   CPPUNIT_ASSERT( ptrIntDec.IsValid() );

   //Test automatic slopes
   CPPUNIT_ASSERT( ptrIntInc->GetMin() == 2 );
   CPPUNIT_ASSERT( ptrIntInc->GetMax() == 10 );

   CPPUNIT_ASSERT( ptrIntDec->GetMin() == 5 );
   CPPUNIT_ASSERT( ptrIntDec->GetMax() == 9 );

   CPPUNIT_ASSERT( ptrIntDec->GetRepresentation() == Linear );
   CPPUNIT_ASSERT( ptrIntDec->GetRepresentation() == Linear ); // second call for happy path
   CIntegerPtr ptrIntDec_OwnRepre = Camera._GetNode("IntDec_OwnRepre");
   CPPUNIT_ASSERT( ptrIntDec_OwnRepre.IsValid() );
   CPPUNIT_ASSERT( ptrIntDec_OwnRepre->GetRepresentation() == HexNumber);

}

void ConverterTestSuite::TestSlope()
{
    ESlope SlopVal;
    bool bOk;

    // Test FromString
    bOk = ESlopeClass::FromString("Increasing", 0);
    CPPUNIT_ASSERT_EQUAL( bOk, false);

    bOk = ESlopeClass::FromString("Increasing", &SlopVal);
    CPPUNIT_ASSERT_EQUAL( bOk, true);
    CPPUNIT_ASSERT_EQUAL( SlopVal, Increasing);

    bOk = ESlopeClass::FromString("Decreasing", &SlopVal);
    CPPUNIT_ASSERT_EQUAL( bOk, true);
    CPPUNIT_ASSERT_EQUAL( SlopVal, Decreasing);

    bOk = ESlopeClass::FromString("Varying", &SlopVal);
    CPPUNIT_ASSERT_EQUAL( bOk, true);
    CPPUNIT_ASSERT_EQUAL( SlopVal, Varying);

    bOk = ESlopeClass::FromString("Automatic", &SlopVal);
    CPPUNIT_ASSERT_EQUAL( bOk, true);
    CPPUNIT_ASSERT_EQUAL( SlopVal, Automatic);

    bOk = ESlopeClass::FromString("BadVal", &SlopVal);
    CPPUNIT_ASSERT_EQUAL( bOk, false);


    // Test ToString
    ESlope * pSlopVal = 0;
    gcstring strVal;
    CPPUNIT_ASSERT_THROW_EX(
        ESlopeClass::ToString(strVal, pSlopVal),
        GenICam::InvalidArgumentException
    );

    CPPUNIT_ASSERT_EQUAL(gcstring("Increasing"), ESlopeClass::ToString(Increasing));
    CPPUNIT_ASSERT_EQUAL(gcstring("Decreasing"), ESlopeClass::ToString(Decreasing));
    CPPUNIT_ASSERT_EQUAL(gcstring("Varying"), ESlopeClass::ToString(Varying));
    CPPUNIT_ASSERT_EQUAL(gcstring("Automatic"), ESlopeClass::ToString(Automatic));
    CPPUNIT_ASSERT_EQUAL(gcstring("_UndefinedESlope"), ESlopeClass::ToString(_UndefinedESlope));
}


void ConverterTestSuite::TestInvalidFormulas()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
        "                                                            \n"
        "<Converter Name=\"Result\">                            \n"
        "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
        "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
        "    <FormulaTo> FROM / Z </FormulaTo>                \n"
        "    <FormulaFrom> TO * Z </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                \n"
        "</Converter>                                                \n"
        "                                                            \n"
        "<IntConverter Name=\"IntResult\">                            \n"
        "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
        "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
        "    <FormulaTo> FROM / Z </FormulaTo>                \n"
        "    <FormulaFrom> TO * Z </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                                \n"
        "    <Slope>Automatic</Slope>                                \n"
        "</IntConverter>                                                \n"
        "                                                            \n"
        "<Integer Name=\"ConvValue\">                                \n"
        "    <Value>2</Value>                                        \n"
        "</Integer>                                                    \n"
        "                                                            \n"
    "                                                \n"
    "<Integer Name=\"ValueX\">                        \n"
    "    <Value>3</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueY\">                        \n"
    "    <Value>10</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrResult(Camera._GetNode("Result"));
    CPPUNIT_ASSERT( ptrResult.IsValid() );
    CPPUNIT_ASSERT_THROW (ptrResult->GetValue(), GenICam::LogicalErrorException);

    CIntegerPtr ptrIntResult(Camera._GetNode("IntResult"));
    CPPUNIT_ASSERT( ptrIntResult.IsValid() );
    CPPUNIT_ASSERT_THROW (ptrIntResult->GetValue(), GenICam::LogicalErrorException);
}


void ConverterTestSuite::TestCaseInsensitive()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
        "                                                   \n"
        "<IntConverter Name=\"IntResult1\">                  \n"
        "    <pVariable Name=\"VALUE\">ConvValue</pVariable>\n"
        "    <FormulaTo> VALUE + FROM </FormulaTo>          \n"
        "    <FormulaFrom> TO </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                     \n"
        "    <Slope>Varying</Slope>                         \n"
        "</IntConverter>                                    \n"
        "                                                   \n"
        "<IntConverter Name=\"IntResult2\">                  \n"
        "    <pVariable Name=\"VALUE\">ConvValue</pVariable>\n"
        "    <FormulaTo> VALUE + from </FormulaTo>          \n"
        "    <FormulaFrom> to </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                     \n"
        "    <Slope>Varying</Slope>                         \n"
        "</IntConverter>                                    \n"
        "                                                   \n"
        "<IntConverter Name=\"IntResult3\">                  \n"
        "    <pVariable Name=\"VALUE\">ConvValue</pVariable>\n"
        "    <FormulaTo> VALUE + FRoM </FormulaTo>          \n"
        "    <FormulaFrom> tO </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                     \n"
        "    <Slope>Varying</Slope>                         \n"
        "</IntConverter>                                    \n"
        "                                                   \n"
        "<Integer Name=\"ConvValue\">                       \n"
        "    <Value>0</Value>                               \n"
        "</Integer>                                         \n"
        "                                                   \n"
        "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // upper case
    CIntegerPtr ptrIntResult(Camera._GetNode("IntResult1"));
    CPPUNIT_ASSERT_EQUAL( int64_t(0), *ptrIntResult() );
    *ptrIntResult = 1;
    CPPUNIT_ASSERT_EQUAL( int64_t(1), *ptrIntResult() );
    *ptrIntResult = 1;
    CPPUNIT_ASSERT_EQUAL( int64_t(2), *ptrIntResult() );
    *ptrIntResult = 1;
    CPPUNIT_ASSERT_EQUAL( int64_t(3), *ptrIntResult() );

    // lower case
    ptrIntResult = Camera._GetNode("IntResult2");
    CPPUNIT_ASSERT( ptrIntResult.IsValid() );
    CPPUNIT_ASSERT_THROW( *ptrIntResult = 1, GenICam::LogicalErrorException );
    CPPUNIT_ASSERT_THROW( *ptrIntResult(), GenICam::LogicalErrorException );

    // mixed case
    ptrIntResult = Camera._GetNode("IntResult3");
    CPPUNIT_ASSERT( ptrIntResult.IsValid() );
    CPPUNIT_ASSERT_THROW( *ptrIntResult = 1, GenICam::LogicalErrorException );
    CPPUNIT_ASSERT_THROW( *ptrIntResult(), GenICam::LogicalErrorException );
}

void ConverterTestSuite::TestUseTOinFROM()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
        "                                                   \n"
        "<IntConverter Name=\"IntResult\">                  \n"
        "    <pVariable Name=\"OLDTO\">ConvValue</pVariable>\n"
        "    <FormulaTo> OLDTO + FROM </FormulaTo>          \n"
        "    <FormulaFrom> TO </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                     \n"
        "    <Slope>Varying</Slope>                         \n"
        "</IntConverter>                                    \n"
        "                                                   \n"
        "<Integer Name=\"ConvValue\">                       \n"
        "    <Value>0</Value>                               \n"
        "</Integer>                                         \n"
        "                                                   \n"
        "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrIntResult(Camera._GetNode("IntResult"));
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, *ptrIntResult() );
    *ptrIntResult = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, *ptrIntResult() );
    *ptrIntResult = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 2LL, *ptrIntResult() );
    *ptrIntResult = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 3LL, *ptrIntResult() );

}

void ConverterTestSuite::TestFormulaProperty()
{

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                   \n"
        "<IntConverter Name=\"IntResult\">                  \n"
        "    <pVariable Name=\"OLDTO\">ConvValue</pVariable>\n"
        "    <FormulaTo> OLDTO + FROM </FormulaTo>          \n"
        "    <FormulaFrom> TO </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                     \n"
        "    <Slope>Varying</Slope>                         \n"
        "</IntConverter>                                    \n"
        "                                                   \n"
        "<Converter Name=\"Result\">                  \n"
        "    <pVariable Name=\"OLDTO\">ConvValue</pVariable>\n"
        "    <FormulaTo> OLDTO + FROM </FormulaTo>          \n"
        "    <FormulaFrom> TO </FormulaFrom>                \n"
        "    <pValue>ConvValue</pValue>                     \n"
        "    <Slope>Varying</Slope>                         \n"
        "</Converter>                                    \n"
        "                                                   \n"
        "<Integer Name=\"ConvValue\">                       \n"
        "    <Value>0</Value>                               \n"
        "</Integer>                                         \n"
        "                                                   \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrIntResult(Camera._GetNode("IntResult"));
    CNodePtr ptrResult(Camera._GetNode("Result"));

    gcstring ValueStr;
    gcstring AttributeStr;
    bool res;

    res = ptrIntResult->GetProperty("FormulaTo", ValueStr, AttributeStr);
    CPPUNIT_ASSERT_EQUAL( true, res );
    cout << "FormulaTo = '" << ValueStr << "'" << endl;
    res = ptrIntResult->GetProperty("FormulaFrom", ValueStr, AttributeStr);
    CPPUNIT_ASSERT_EQUAL( true, res );
    cout << "FormulaTo = '" << ValueStr << "'" << endl;

    res = ptrResult->GetProperty("FormulaTo", ValueStr, AttributeStr);
    CPPUNIT_ASSERT_EQUAL( true, res );
    cout << "FormulaTo = '" << ValueStr << "'" << endl;
    res = ptrResult->GetProperty("FormulaFrom", ValueStr, AttributeStr);
    CPPUNIT_ASSERT_EQUAL( true, res );
    cout << "FormulaTo = '" << ValueStr << "'" << endl;

}

void ConverterTestSuite::TestConstantAndExpression()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
        "<Converter Name=\"FloatResult\">                            \n"
        "    <pVariable Name=\"X\">FloatVar</pVariable>    \n"
        "    <Constant Name=\"Two\">2.0</Constant>    \n"
        "    <Expression Name=\"TwoX\">2.0*X</Expression>    \n"
        "    <FormulaTo> FROM - TwoX - Two </FormulaTo>                \n"
        "    <FormulaFrom> TO + TwoX + Two </FormulaFrom>                \n"
        "    <pValue>FloatVal</pValue>                                \n"
        "    <Slope>Increasing</Slope>                                \n"
        "</Converter>                                                \n"
    "                                                \n"
    "<Float Name=\"FloatVal\">                        \n"
    "    <Value>1.0</Value>                            \n"
    "</Float>                                        \n"
    "                                                \n"
    "<Float Name=\"FloatVar\">                        \n"
    "    <Value>5.0</Value>                            \n"
    "</Float>                                        \n"
    "                                                \n"
    "                                                \n"
        "<IntConverter Name=\"IntResult\">                            \n"
        "    <pVariable Name=\"X\">IntVar</pVariable>    \n"
        "    <Constant Name=\"Two\">2</Constant>    \n"
        "    <Expression Name=\"TwoX\">2*X</Expression>    \n"
        "    <FormulaTo> FROM - TwoX - Two </FormulaTo>                \n"
        "    <FormulaFrom> TO + TwoX + Two </FormulaFrom>                \n"
        "    <pValue>IntVal</pValue>                                \n"
        "    <Slope>Increasing</Slope>                                \n"
        "</IntConverter>                                                \n"
    "                                                \n"
    "<Integer Name=\"IntVal\">                        \n"
    "    <Value>1</Value>                            \n"
    "    <Min>0</Min>                                            \n"
        "    <Max>4096</Max>                                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"IntVar\">                        \n"
    "    <Value>5</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);


    CIntegerPtr ptrIntResult = Camera._GetNode("IntResult");
    CPPUNIT_ASSERT( ptrIntResult.IsValid() );
    CIntegerPtr ptrIntVal = Camera._GetNode("IntVal");
    CPPUNIT_ASSERT( ptrIntVal.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t) 13LL, ptrIntResult->GetValue());
    ptrIntResult->SetValue(15);
    CPPUNIT_ASSERT_EQUAL( (int64_t) 3LL, ptrIntVal->GetValue());

    CFloatPtr ptrFloatResult = Camera._GetNode("FloatResult");
    CPPUNIT_ASSERT( ptrFloatResult.IsValid() );
    CFloatPtr ptrFloatVal = Camera._GetNode("FloatVal");
    CPPUNIT_ASSERT( ptrFloatVal.IsValid() );

    CPPUNIT_ASSERT_EQUAL( 13.0, ptrFloatResult->GetValue());
    ptrFloatResult->SetValue(15.0);
    CPPUNIT_ASSERT_EQUAL( 3.0, ptrFloatVal->GetValue());

}

void ConverterTestSuite::TestLimits()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "                                                \n"
    "<IntConverter Name=\"IntResult\">               \n"
    "    <pVariable Name=\"X\">IntVar</pVariable>    \n"
    "    <FormulaTo> FROM / 2 </FormulaTo>           \n"
    "    <FormulaFrom> TO * 2 </FormulaFrom>         \n"
    "    <pValue>IntVal</pValue>                     \n"
    "    <Slope>Varying</Slope>                      \n"
    "</IntConverter>                                 \n"
    "                                                \n"
    "<Integer Name=\"IntVal\">                       \n"
    "    <Value>1</Value>                            \n"
    "    <Min>0</Min>                                \n"
    "    <Max>9223372036854775807</Max>              \n"
    "</Integer>                                      \n"
    "                                                \n"
    "<Integer Name=\"IntVar\">                       \n"
    "    <Value>5</Value>                            \n"
    "</Integer>                                      \n"
    "                                                \n"
    "                                                \n"
    "<IntConverter Name=\"IntResult2\">              \n"
    "    <pVariable Name=\"X\">IntVar2</pVariable>   \n"
    "    <FormulaTo> FROM / 2 </FormulaTo>           \n"
    "    <FormulaFrom> TO * 2 </FormulaFrom>         \n"
    "    <pValue>IntVal2</pValue>                    \n"
    "    <Slope>Increasing</Slope>                   \n"
    "</IntConverter>                                 \n"
    "                                                \n"
    "<Integer Name=\"IntVal2\">                      \n"
    "    <Value>1</Value>                            \n"
    "</Integer>                                      \n"
    "                                                \n"
    "<Integer Name=\"IntVar2\">                      \n"
    "    <Value>5</Value>                            \n"
    "</Integer>                                      \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);


    CIntegerPtr ptrIntResult = Camera._GetNode("IntResult");
    CPPUNIT_ASSERT( ptrIntResult.IsValid() );
    CIntegerPtr ptrIntVal = Camera._GetNode("IntVal");
    CPPUNIT_ASSERT( ptrIntVal.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t) 9223372036854775807LL, ptrIntResult->GetMax());
    CPPUNIT_ASSERT_NO_THROW (ptrIntResult->SetValue(15));
}


void ConverterTestSuite::TestUpperLowerCase()
{

    if(GenApiSchemaVersion == v1_0)
    {

        // In schema version 1.0 the variable names inside the formula are
        // converted to upper case. Thus the variable names in the pVariable entry
        // must always be upper case

        gcstring CameraDescriptionFile =
            CDFHeader +
            "                                                            \n"
            "<Converter Name=\"ShutterAbs\">                            \n"
            "    <pVariable Name=\"TIMEBASE\">TimeBase</pVariable>        \n"
            // note the mixture of upper/lower case letters in the two formulas
            "    <FormulaTo> FroM / TiMeBaSe </FormulaTo>                \n"
            "    <FormulaFrom> TO * timeBASE </FormulaFrom>                \n"
            "    <pValue>ShutterRaw</pValue>                                \n"
            "  <Unit>s</Unit>                   \n"
            "    <Slope>Increasing</Slope>                                \n"
            "</Converter>                                                \n"
            "                                                            \n"
            "<Integer Name=\"ShutterRaw\">                                \n"
            "    <Value>2</Value>                                        \n"
            "</Integer>                                                    \n"
            "                                                            \n"
            "<Integer Name=\"TimeBase\">                                \n"
            "    <Value>10</Value>                                        \n"
            "</Integer>                                                    \n"
            "                                                            \n"
            + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        CFloatPtr ptrShutterAbs = Camera._GetNode("ShutterAbs");
        CPPUNIT_ASSERT( ptrShutterAbs.IsValid() );
        CPPUNIT_ASSERT_EQUAL (intfIFloat, ptrShutterAbs->GetNode()->GetPrincipalInterfaceType());

        CIntegerPtr ptrShutterRaw = Camera._GetNode("ShutterRaw");
        CPPUNIT_ASSERT( ptrShutterRaw.IsValid() );

        CIntegerPtr ptrTimeBase = Camera._GetNode("TimeBase");
        CPPUNIT_ASSERT( ptrTimeBase.IsValid() );

        // check the converter's units
        CPPUNIT_ASSERT_EQUAL( ::gcstring("s") , ptrShutterAbs->GetUnit());

        // reading
        CPPUNIT_ASSERT_DOUBLES_EQUAL( 20.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

        *ptrShutterRaw = 3;
        CPPUNIT_ASSERT_DOUBLES_EQUAL( 30.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

        *ptrTimeBase = 100;
        CPPUNIT_ASSERT_DOUBLES_EQUAL( 300.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

        // Writing
        *ptrShutterAbs = 400.0;
        CPPUNIT_ASSERT_EQUAL( (int64_t)4, *ptrShutterRaw());
    }
    else
    {

        // In schema version 1.1 ff the variable names can be mixed upper/lower case
        // but must be the same in the formula and the pVariable statement

        gcstring CameraDescriptionFile =
            CDFHeader +
            "                                                            \n"
            "<Converter Name=\"ShutterAbs\">                            \n"
            "    <pVariable Name=\"TiMeBaSe\">TimeBase</pVariable>        \n"
            "    <FormulaTo> FROM / TiMeBaSe </FormulaTo>                \n"
            "    <FormulaFrom> TO * TiMeBaSe </FormulaFrom>                \n"
            "    <pValue>ShutterRaw</pValue>                                \n"
            "  <Unit>s</Unit>                   \n"
            "    <Slope>Increasing</Slope>                                \n"
            "</Converter>                                                \n"
            "                                                            \n"
            "<Integer Name=\"ShutterRaw\">                                \n"
            "    <Value>2</Value>                                        \n"
            "</Integer>                                                    \n"
            "                                                            \n"
            "<Integer Name=\"TimeBase\">                                \n"
            "    <Value>10</Value>                                        \n"
            "</Integer>                                                    \n"
            "                                                            \n"
            + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        CFloatPtr ptrShutterAbs = Camera._GetNode("ShutterAbs");
        CPPUNIT_ASSERT( ptrShutterAbs.IsValid() );
        CPPUNIT_ASSERT_EQUAL (intfIFloat, ptrShutterAbs->GetNode()->GetPrincipalInterfaceType());

        CIntegerPtr ptrShutterRaw = Camera._GetNode("ShutterRaw");
        CPPUNIT_ASSERT( ptrShutterRaw.IsValid() );

        CIntegerPtr ptrTimeBase = Camera._GetNode("TimeBase");
        CPPUNIT_ASSERT( ptrTimeBase.IsValid() );

        // check the converter's units
        CPPUNIT_ASSERT_EQUAL( ::gcstring("s") , ptrShutterAbs->GetUnit());

        // reading
        CPPUNIT_ASSERT_DOUBLES_EQUAL( 20.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

        *ptrShutterRaw = 3;
        CPPUNIT_ASSERT_DOUBLES_EQUAL( 30.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

        *ptrTimeBase = 100;
        CPPUNIT_ASSERT_DOUBLES_EQUAL( 300.0, *ptrShutterAbs(), std::numeric_limits<double>::epsilon());

        // Writing
        *ptrShutterAbs = 400.0;
        CPPUNIT_ASSERT_EQUAL( (int64_t)4, *ptrShutterRaw());
    }
}
