//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "NodeMapTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "GenApi/GenApiVersion.h"
#include "GenApi/impl/DllManager.h"
#include "GenApi/DLLLoad.h"

// hn ++ 10012007
//#include "Base/GCTypes.h"
//#pragma message( "include GCTypes.h" )
// hn --

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( NodeMapTestSuite );



void NodeMapTestSuite::TestNodeMap()
{
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<Integer Name=\"TestInt\">                \n"
        "    <Value>0</Value>                    \n"
        "</Integer>                                \n"
        "                                        \n"
        "<Register Name=\"TestReg\">            \n"
        "    <Address>0x0000</Address>            \n"
        "    <Length>1</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "</Register>                            \n"
        "                                        \n"
        "<Port Name=\"Port\"/>                    \n"
        + CDFFooter;

    CNodeMapRef Camera;
    CTestPort Port1, Port2;
    NodeList_t NodeList;
    size_t Size;

    // test uninitialzied map
    CPPUNIT_ASSERT_THROW( Camera._Connect(&Port1, "Port"), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW( Camera._Connect(&Port2), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW( Camera._GetNode("TestInt"), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW( Camera._GetNodes(NodeList), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW( Camera._GetDeviceName(), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW( Camera._InvalidateNodes(), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW( Camera._Poll(500), GenICam::AccessException );

    // now test initialized map
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CNodeMapDynPtr pNodeMapDyn(Camera._Ptr);
    CPPUNIT_ASSERT_NO_THROW( Camera._Connect(&Port1, "Port") );
    CPPUNIT_ASSERT_NO_THROW( Camera._Connect(&Port2) );
    CPPUNIT_ASSERT_NO_THROW( Camera._GetNode("TestInt") );
    CPPUNIT_ASSERT_NO_THROW( Camera._GetNodes(NodeList) );
    Size = NodeList.size();
    CPPUNIT_ASSERT_NO_THROW( Camera._GetNodes(NodeList) );
    CPPUNIT_ASSERT_EQUAL ( Size, NodeList.size() );
    CPPUNIT_ASSERT_NO_THROW( Camera._GetDeviceName() );
    CPPUNIT_ASSERT_NO_THROW( Camera._InvalidateNodes() );
    CPPUNIT_ASSERT_EQUAL ( gcstring("Device"), Camera._GetDeviceName() );
    CPPUNIT_ASSERT_EQUAL ( gcstring("Device"), Camera._GetNode("TestInt")->GetDeviceName() );


    {
        // test loading from file
        CNodeMapRef Camera2, Camera3, Camera4, Camera4b;
        CPPUNIT_ASSERT_NO_THROW( Camera2._LoadXMLFromFile("Test", "TestCamera_v1_0") );
        CPPUNIT_ASSERT ( Camera2._GetNode("MyNode") != NULL );
        gcstring FileName(XMLPATH "Test/TestCamera_v1_0.xml");
        GenICam::ReplaceEnvironmentVariables(FileName);
        CPPUNIT_ASSERT_NO_THROW( Camera3._LoadXMLFromFile(FileName) );
        CPPUNIT_ASSERT ( Camera3._GetNode("MyNode") != NULL );
        gcstring DllFileName(GenApi::_MakeDLLFileName("grzzz", "barghhh"));
        CPPUNIT_ASSERT_THROW( Camera4._LoadDLL(DllFileName), GenICam::RuntimeException );
        DllFileName = _MakeDLLFileName("Test", "TestCamera_v1_0");
        CPPUNIT_ASSERT_NO_THROW( Camera4._LoadDLL(DllFileName) );
        CPPUNIT_ASSERT ( Camera4._GetNode("MyNode") != NULL );
        CPPUNIT_ASSERT_THROW( Camera4b._LoadXMLFromFile("vrrrmm", "psssst"), GenICam::RuntimeException );
        CPPUNIT_ASSERT_THROW( Camera4b._LoadDLL(DllFileName), GenICam::RuntimeException  );
        CPPUNIT_ASSERT_THROW( Camera4b._LoadDLL("vrrrmm", "psssst"), GenICam::RuntimeException  );
        CPPUNIT_ASSERT_THROW( Camera4b._LoadXMLFromFile(DllFileName), GenICam::RuntimeException );
        CPPUNIT_ASSERT_THROW( Camera4b._LoadXMLFromStringInject("vrrrmm", "psssst"), GenICam::RuntimeException );
        CPPUNIT_ASSERT_THROW( Camera4b._LoadXMLFromFileInject("vrrrmm", "psssst"), GenICam::RuntimeException );
    }
    {
        // test loading from file
        CNodeMapRef Camera2, Camera3, Camera4, Camera4b;
        CPPUNIT_ASSERT_NO_THROW( Camera2._LoadXMLFromFile("Test", "TestCamera_v1_1") );
        CPPUNIT_ASSERT ( Camera2._GetNode("MyNode") != NULL );
        gcstring FileName(XMLPATH "Test/TestCamera_v1_1.xml");
        GenICam::ReplaceEnvironmentVariables(FileName);
        CPPUNIT_ASSERT_NO_THROW( Camera3._LoadXMLFromFile(FileName) );
        CPPUNIT_ASSERT ( Camera3._GetNode("MyNode") != NULL );
        gcstring DllFileName(_MakeDLLFileName("grzzz", "barghhh"));
        CPPUNIT_ASSERT_THROW( Camera4._LoadDLL(DllFileName), GenICam::RuntimeException );
        DllFileName = _MakeDLLFileName("Test", "TestCamera_v1_1");
        CPPUNIT_ASSERT_NO_THROW( Camera4._LoadDLL(DllFileName) );
        CPPUNIT_ASSERT ( Camera4._GetNode("MyNode") != NULL );
        CPPUNIT_ASSERT_THROW( Camera4b._LoadXMLFromFile("vrrrmm", "psssst"), GenICam::RuntimeException );
    }
    // test empty map
    pNodeMapDyn->ClearAllNodes ();
    CPPUNIT_ASSERT_EQUAL ( (GenApi::INode*)NULL, Camera._GetNode("FooBar") );

    // try loading again without first cleaning up
    CPPUNIT_ASSERT_THROW(
        Camera._LoadXMLFromString(CameraDescriptionFile),
        GenICam::RuntimeException
    );


    // cover all paths in node factory (namely converter and invalid nodes)
    CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<Integer Name=\"TestInt\">                \n"
        "    <Value>0</Value>                    \n"
        "</Integer>                                \n"
        "                                        \n"
        "<Converter Name=\"TestConv\">            \n"
        "    <FormulaTo>(FROM &#47; 20)</FormulaTo>    \n"
        "    <FormulaFrom>(TO * 20)</FormulaFrom>    \n"
        "    <pValue>TestInt</pValue>            \n"
        "</Converter>                            \n"
        "                                        \n"
        "<FooBar Name=\"TestInvalid\">                \n"
        "    <ToolTip>Invalid node</ToolTip>        \n"
        "</FooBar>                                \n"
        "                                        \n"
        "<Port Name=\"Port\"/>                    \n"
        + CDFFooter;

    CNodeMapRef Camera5;
    CPPUNIT_ASSERT_THROW( Camera5._LoadXMLFromString(CameraDescriptionFile), GenICam::RuntimeException );

    // pass invalid XML so taht the Xalan transformer fails
    CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "</Integer>                                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera6;
    CPPUNIT_ASSERT_THROW( Camera6._LoadXMLFromString(CameraDescriptionFile), GenICam::RuntimeException );
}


void NodeMapTestSuite::TestTheRest()
{
    using namespace Test_TestCamera_v1_0;
    using namespace Test_TestCamera_v1_1;

    gcstring DllFileName(_MakeDLLFileName("Test", "TestCamera_v1_0"));
    DllManager::Handle _hDLL(DllManager::LoadLib(DllFileName));
    CPPUNIT_ASSERT ( DllManager::INVALID_HANDLE != _hDLL );
    typedef void (__cdecl* SetGenApiVersion_f)(Version_t &Version);
    GetGenApiVersion_f fGetGenApiVersion((GetGenApiVersion_f) DllManager::FindSymbol(_hDLL, "GetGenApiVersion"));
    CPPUNIT_ASSERT ( DllManager::INVALID_SYMBOL != fGetGenApiVersion );
    SetGenApiVersion_f fSetGenApiVersion = (SetGenApiVersion_f) DllManager::FindSymbol(_hDLL, "SetGenApiVersion");
    CPPUNIT_ASSERT ( DllManager::INVALID_SYMBOL != fSetGenApiVersion );
    SetGenApiVersion_f fUnknownSymbol = (SetGenApiVersion_f) DllManager::FindSymbol(_hDLL, "ThisFunctionDoesNotExist");
    CPPUNIT_ASSERT ( DllManager::INVALID_SYMBOL == fUnknownSymbol );

    {
        // test incorrect GenApi versions
        CTestCamera_v1_0 Camera1, Camera2, Camera3, Camera4, Camera5;

        // with default values should be OK
        CPPUNIT_ASSERT_NO_THROW ( Camera1._LoadDLL() );

        // now the incorrect ones
        Version_t GenApiVersion2 = {GENAPI_VERSION_MAJOR+1, GENAPI_VERSION_MINOR, GENAPI_VERSION_SUBMINOR};
        (fSetGenApiVersion)(GenApiVersion2);
        CPPUNIT_ASSERT_THROW( Camera2._LoadDLL(), GenICam::RuntimeException );

        Version_t GenApiVersion3 = {GENAPI_VERSION_MAJOR, GENAPI_VERSION_MINOR+1, GENAPI_VERSION_SUBMINOR};
        (fSetGenApiVersion)(GenApiVersion3);
        CPPUNIT_ASSERT_THROW( Camera3._LoadDLL(), GenICam::RuntimeException );

        // and again these ones should pass
        Version_t GenApiVersion4 = {GENAPI_VERSION_MAJOR, GENAPI_VERSION_MINOR, GENAPI_VERSION_SUBMINOR+1};
        (fSetGenApiVersion)(GenApiVersion4);
        CPPUNIT_ASSERT_NO_THROW( Camera4._LoadDLL() );

        Version_t GenApiVersion5 = {GENAPI_VERSION_MAJOR, GENAPI_VERSION_MINOR, GENAPI_VERSION_SUBMINOR};
        (fSetGenApiVersion)(GenApiVersion5);
        CPPUNIT_ASSERT_NO_THROW( Camera5._LoadDLL() );
    }

    // Free the library
    DllManager::FreeLib( _hDLL );
    // And try to free one not previously loaded
    CPPUNIT_ASSERT_EQUAL ( false, DllManager::FreeLib( DllManager::INVALID_HANDLE ) );
}

void NodeMapTestSuite::TestSchemaLocation()
{

    gcstring SubTest( XMLPATH "Test/SubTest/SubTest.xml" );
    ReplaceEnvironmentVariables(SubTest);

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile(SubTest);

    CIntegerPtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT_EQUAL( (int64_t)3, ptrValue->GetValue());
}
// Mantis #127: XMLLoader needs the optional ToolTip to be
// present in RegisterDescription element
void NodeMapTestSuite::TestWithoutTooltip()
{
    const GenICam::gcstring CDFHeader =
    "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
    "<RegisterDescription\n"
    "      ModelName=\"GenApiTest\"\n"
    "      VendorName=\"Generic\"\n"
//    "      ToolTip=\"nodes for testing the GenApi reference implementation\"\n"
    "      StandardNameSpace=\"GEV\"\n"
    "      SchemaMajorVersion=\"1\"\n"
    "      SchemaMinorVersion=\"0\"\n"
    "      SchemaSubMinorVersion=\"1\"\n"
    "      MajorVersion=\"2\"\n"
    "      MinorVersion=\"3\"\n"
    "      SubMinorVersion=\"4\"\n"
    "      ProductGuid=\"2D932CC6-EB68-40bd-B6CC-F03B55B7D653\"\n"
    "      VersionGuid=\"02A8C268-BEE8-463b-A6C0-53ED8256E3D8\"\n"
    "      xmlns=\"http://www.genicam.org/GenApi/Version_1_0\"\n"
    "      xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
    "      xsi:schemaLocation=\"http://www.genicam.org/GenApi/Version_1_0\n"
    "            http://www.genicam.org/GenApi/GenApiSchema_Version_1_0.xsd\">\n";


    gcstring CameraDescriptionFile =
        CDFHeader +
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString( CameraDescriptionFile );

}

void NodeMapTestSuite::TestSchemaVersions()
{
    CNodeMapRef Camera;

    GenICam::gcstring_vector SchemaVersions;
    Camera._GetSupportedSchemaVersions( SchemaVersions );

    CPPUNIT_ASSERT_EQUAL( (size_t)2, SchemaVersions.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("1.1"), SchemaVersions[0] );
    CPPUNIT_ASSERT_EQUAL( gcstring("1.2"), SchemaVersions[1] );
}

void NodeMapTestSuite::TestExposeStatic()
{
    using namespace Test_TestCamera_v1_1;

    CTestCamera_v1_1 Camera;
    Camera._LoadDLL();

    cout << Camera.Root.GetNode()->GetName().c_str() << "\n";
    cout << Camera.NoFeatureButExposed.GetNode()->GetName().c_str() << "\n";
    //cout << Camera.FeatureButNotExposed.GetNode()->GetName().c_str() << "\n";
    cout << Camera.CategoryButExposed.GetNode()->GetName().c_str() << "\n";
    //cout << Camera.CategoryNotExposed.GetNode()->GetName().c_str() << "\n";
}
