//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------

/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "NodeTest.h"
#include "TestPort.h"
#include "CallbackHelper.h"
#include "GenApi/GenApi.h"
#include <GenICamVersion.h>
#include "GenApi/StructPort.h"
#include "GenApi/impl/PointerImpl.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

#include <limits>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( NodeTestSuite );

void NodeTestSuite::TestAccessMode()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                           \n"
    "    <Category Name=\"Node0\">                              \n"
    "        <pFeature>Node00</pFeature>                        \n"
    "        <pFeature>Node01</pFeature>                        \n"
    "    </Category>                                            \n"
    "                                                           \n"
    "    <Category Name=\"Cat\">                              \n"
    "        <pIsImplemented>CatImplemented</pIsImplemented>    \n"
    "        <pIsAvailable>CatAvailable</pIsAvailable>          \n"
    "        <pFeature>CatNode</pFeature>                        \n"
    "    </Category>                                            \n"
    "                                                           \n"
    "    <Node Name=\"CatNode\">                                 \n"
    "    </Node>                                                \n"
    "                                                           \n"
    "    <Category Name=\"CatFtrNI\">                           \n"
    "        <pFeature>FtrNI</pFeature>                         \n"
    "    </Category>                                            \n"
    "                                                           \n"
    "    <Integer Name=\"FtrNI\">                               \n"
    "    <pIsImplemented>AMCtrl</pIsImplemented>                \n"
    "    <Value>0</Value>                                       \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Category Name=\"CatFtrNA\">                           \n"
    "        <pFeature>FtrNA</pFeature>                         \n"
    "    </Category>                                            \n"
    "                                                           \n"
    "    <Integer Name=\"FtrNA\">                               \n"
    "    <pIsAvailable>AMCtrl</pIsAvailable>                    \n"
    "    <Value>0</Value>                                       \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"AMCtrl\">                              \n"
    "    <Value>0</Value>                                       \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Node Name=\"Node00\">                                 \n"
    "        <pIsImplemented>Implemented0</pIsImplemented>      \n"
    "        <pIsAvailable>Available</pIsAvailable>             \n"
    "        <pIsLocked>Locked</pIsLocked>                      \n"
    "    </Node>                                                \n"
    "                                                           \n"
    "    <Category Name=\"Node01\">                             \n"
    "        <pFeature>Node010</pFeature>                       \n"
    "    </Category>                                            \n"
    "                                                           \n"
    "    <Node Name=\"Node010\">                                \n"
    "        <pIsImplemented>Implemented010</pIsImplemented>    \n"
    "    </Node>                                                \n"
    "                                                           \n"
    "    <Integer Name=\"Implemented0\">                        \n"
    "        <pIsAvailable>Implemented0Available</pIsAvailable> \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"Implemented010\">                      \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"Available\">                           \n"
    "        <pIsAvailable>AvailableAvailable</pIsAvailable>    \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"Locked\">                              \n"
    "        <pIsAvailable>LockedAvailable</pIsAvailable>       \n"
    "        <Value>0</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"Implemented0Available\">               \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"AvailableAvailable\">                  \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"LockedAvailable\">                     \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"CatImplemented\">                       \n"
    "        <pIsAvailable>CatImplementedAvailable</pIsAvailable> \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"CatAvailable\">                           \n"
    "        <pIsAvailable>CatAvailableAvailable</pIsAvailable> \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"CatImplementedAvailable\">               \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"CatAvailableAvailable\">                  \n"
    "        <Value>1</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Node Name=\"NodeImplWO\">                             \n"
    "        <pIsImplemented>WONode</pIsImplemented>            \n"
    "    </Node>                                                \n"
    "                                                           \n"
    "    <Node Name=\"NodeAvailWO\">                            \n"
    "        <pIsAvailable>WONode</pIsAvailable>                \n"
    "    </Node>                                                \n"
    "                                                           \n"
    "    <Node Name=\"NodeLockedWO\">                           \n"
    "        <pIsLocked>WONode</pIsLocked>                      \n"
    "    </Node>                                                \n"
    "                                                           \n"
    "    <Integer Name=\"WONode\">                              \n"
    "        <ImposedAccessMode>WO</ImposedAccessMode>          \n"
    "        <Value>0</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    "    <Integer Name=\"WONode2\">                             \n"
    "        <pIsLocked>LockedAvailable</pIsLocked>             \n"
    "        <ImposedAccessMode>WO</ImposedAccessMode>          \n"
    "        <Value>0</Value>                                   \n"
    "    </Integer>                                             \n"
    "                                                           \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePrivatePtr ptrNode0 = Camera._GetNode("Node0");
    CNodePrivatePtr ptrCat = Camera._GetNode("Cat");
    CNodePrivatePtr ptrCatFtrNI = Camera._GetNode("CatFtrNI");
    CNodePrivatePtr ptrCatFtrNA = Camera._GetNode("CatFtrNA");
    CNodePrivatePtr ptrNode00 = Camera._GetNode("Node00");
    CNodePrivatePtr ptrNode01 = Camera._GetNode("Node01");
    CNodePrivatePtr ptrNode010 = Camera._GetNode("Node010");
    CIntegerPtr ptrImplemented0 = Camera._GetNode("Implemented0");
    CIntegerPtr ptrImplemented010 = Camera._GetNode("Implemented010");
    CIntegerPtr ptrAvailable = Camera._GetNode("Available");
    CIntegerPtr ptrLocked = Camera._GetNode("Locked");
    CIntegerPtr ptrCatImplemented = Camera._GetNode("CatImplemented");
    CIntegerPtr ptrCatAvailable = Camera._GetNode("CatAvailable");
    CIntegerPtr ptrCatLocked = Camera._GetNode("CatLocked");
    CNodePtr ptrCatNode = Camera._GetNode("CatNode");
    CNodePrivatePtr ptrNodeImplWO = Camera._GetNode("NodeImplWO");
    CNodePrivatePtr ptrNodeAvailWO = Camera._GetNode("NodeAvailWO");
    CNodePrivatePtr ptrNodeLockedWO = Camera._GetNode("NodeLockedWO");
    CNodePrivatePtr ptrWONode = Camera._GetNode("WONode");
    CNodePrivatePtr ptrWONode2 = Camera._GetNode("WONode2");


    CPPUNIT_ASSERT_EQUAL (gcstring("IValue"), GetInterfaceName(ptrCatNode));
    CPPUNIT_ASSERT_EQUAL (intfIValue, ptrNode00->GetPrincipalInterfaceType());
    CPPUNIT_ASSERT_EQUAL (intfICategory, ptrNode0->GetPrincipalInterfaceType());
    CNodePtr ptrMyNode = Camera._GetNode("Node00");
    CPPUNIT_ASSERT_EQUAL (intfIValue, ptrMyNode->GetPrincipalInterfaceType());

    // Check default access mode
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode0->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( RW, ptrNode00->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode01->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( RW, ptrNode010->GetAccessMode() );

    // Use cache
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode0->GetAccessMode() );

    // Check Implemented
    *ptrImplemented0 = 0;
    CPPUNIT_ASSERT_EQUAL( NI, ptrNode00->GetAccessMode() );
    *ptrImplemented0 = 1;

    // Check Available
    *ptrAvailable = 0;
    CPPUNIT_ASSERT_EQUAL( NA, ptrNode00->GetAccessMode() );
    *ptrAvailable = 1;

    // Check Locked
    *ptrLocked = 1;
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode00->GetAccessMode() );
    *ptrLocked = 0;

    // Check Implemented of a <pFeature> node
    *ptrImplemented010 = 0;
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode0->GetAccessMode() );

    // note that the access mode is either NI or RO; since NI does not change
    // the access mode can be always(!) cacheable
    // as a consequence this test case is not valid for categories
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode01->GetAccessMode() );
    *ptrImplemented010 = 1;

    // Check Category access mode implied by pFeature
    CPPUNIT_ASSERT_EQUAL( NI, ptrCatFtrNI->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrCatFtrNA->GetAccessMode() );

    // Check flag access
    *ptrImplemented0 = 0;
    *ptrAvailable = 0;
    *ptrLocked = 1;
    CPPUNIT_ASSERT_EQUAL( false, IsImplemented( *ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( *ptrNode00 ) );
    *ptrImplemented0 = 1;
    *ptrAvailable = 0;
    CPPUNIT_ASSERT_EQUAL( true, IsImplemented( ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsReadable( *ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsWritable( *ptrNode00 ) );
    *ptrAvailable = 1;
    CPPUNIT_ASSERT_EQUAL( true, IsAvailable( ptrNode00 ) );
    *ptrLocked = 1;
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode00->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( true, IsReadable( ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsWritable( ptrNode00 ) );
    *ptrLocked = 0;
    CPPUNIT_ASSERT_EQUAL( true, IsWritable( ptrNode00 ) );
    // and again with the flags made "not available"
    CIntegerPtr ptrImplemented0Available = Camera._GetNode("Implemented0Available");
    CIntegerPtr ptrAvailableAvailable = Camera._GetNode("AvailableAvailable");
    CIntegerPtr ptrLockedAvailable = Camera._GetNode("LockedAvailable");
    *ptrImplemented0Available = 0;
    CPPUNIT_ASSERT_EQUAL( true, IsImplemented( *ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( *ptrNode00 ) );
    *ptrImplemented0Available = 1;
    CPPUNIT_ASSERT_EQUAL( true, IsImplemented( *ptrNode00 ) );
   *ptrAvailableAvailable = 0;
    CPPUNIT_ASSERT_EQUAL( true, IsImplemented( ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsReadable( *ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsWritable( *ptrNode00 ) );
    *ptrAvailableAvailable = 1;
    CPPUNIT_ASSERT_EQUAL( true, IsWritable( *ptrNode00 ) );

    // WO flags
    CPPUNIT_ASSERT_EQUAL( false, IsReadable( *ptrWONode ) );
    CPPUNIT_ASSERT_EQUAL( NA, ptrNodeImplWO->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( NA, ptrNodeAvailWO->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( NA, ptrNodeLockedWO->GetAccessMode() );

    *ptrLockedAvailable = 0;
    CPPUNIT_ASSERT_EQUAL( NA, ptrNode00->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( false, IsReadable( ptrNode00 ) );
    CPPUNIT_ASSERT_EQUAL( false, IsWritable( ptrNode00 ) );
    *ptrLockedAvailable = 1;
    CPPUNIT_ASSERT_EQUAL( true, IsWritable( ptrNode00 ) );
    // Do similar checks for category
    *ptrCatImplemented = 0;
    CPPUNIT_ASSERT_EQUAL( NI, ptrCat->GetAccessMode() );
    *ptrCatImplemented = 1;
    *ptrCatAvailable = 0;
    CPPUNIT_ASSERT_EQUAL( NI, ptrCat->GetAccessMode() );
    *ptrCatAvailable = 1;
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( *ptrCat ) );
    CIntegerPtr ptrCatImplementedAvailable = Camera._GetNode("CatImplementedAvailable");
    CIntegerPtr ptrCatAvailableAvailable = Camera._GetNode("CatAvailableAvailable");
    *ptrCatImplementedAvailable = 0;
    CPPUNIT_ASSERT_EQUAL( false, IsImplemented( *ptrCat ) );
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( *ptrCat ) );
    *ptrCatImplementedAvailable = 1;
    *ptrCatAvailableAvailable = 0;
    IsImplemented( *ptrCat );
    IsAvailable( *ptrCat );
    CPPUNIT_ASSERT_EQUAL( false, IsImplemented( *ptrCat ) );
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( *ptrCat ) );
    CPPUNIT_ASSERT_EQUAL( false, IsReadable( *ptrCat ) );
    CPPUNIT_ASSERT_EQUAL( false, IsWritable( *ptrCat ) );
    *ptrCatAvailableAvailable = 1;
    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( ptrCat ) );

    // Check visibility
    CPPUNIT_ASSERT_EQUAL( true, IsVisible( Beginner, Guru ) );
    CPPUNIT_ASSERT_EQUAL( false, IsVisible( Invisible, Expert ) );

    // Check flag combinations
    CPPUNIT_ASSERT_EQUAL( NI, Combine(RO, NI) );
    CPPUNIT_ASSERT_EQUAL( NA, Combine(RO, NA) );
    CPPUNIT_ASSERT_EQUAL( NA, Combine(WO, RO) );

    // check what happens if a WO node is locked
    *ptrLockedAvailable = 0;
    CPPUNIT_ASSERT( IsWritable(ptrWONode2) );
    CPPUNIT_ASSERT( !IsReadable(ptrWONode2) );
    *ptrLockedAvailable = 1;
    CPPUNIT_ASSERT( !IsWritable(ptrWONode2) );
    CPPUNIT_ASSERT( !IsReadable(ptrWONode2) );


    // to complete happy path, check also locked node
    // whose value had different access mode than RW
    CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Integer Name=\"Value\">                        \n"
    "    <pIsLocked>Locked</pIsLocked>                \n"
    "    <pValue>ValueReg</pValue>                    \n"
    "    <Min>0</Min>                                \n"
    "    <Max>4294967296</Max>                        \n"
    "    <Inc>1</Inc>                                \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<IntReg Name=\"ValueReg\">                        \n"
    "        <Address>0x0104</Address>                \n"
    "        <Length>4</Length>                        \n"
    "        <AccessMode>WO</AccessMode>                \n"
     "        <pPort>Port</pPort>                        \n"
    "        <Sign>Unsigned</Sign>                    \n"
     "        <Endianess>LittleEndian</Endianess>        \n"
    "        <Representation>Linear</Representation>    \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "    <Integer Name=\"Locked\">                    \n"
    "        <Value>1</Value>                        \n"
    "    </Integer>                                    \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera2;
    Camera2._LoadXMLFromString(CameraDescriptionFile);

    CTestPort Port2;
    Port2.CreateEntry(0x0104, (uint32_t)1024, RW);
    Camera2._Connect(&Port2, "Port");

    CIntegerPtr ptrValue = Camera2._GetNode("Value");

    CPPUNIT_ASSERT_THROW_EX(
        ptrValue->GetValue(),
        GenICam::AccessException
    );

    CPPUNIT_ASSERT_EQUAL( false, IsAvailable( ptrValue ) );
}


void NodeTestSuite::TestStaticNameSpace()
{
    {
        using namespace Test_TestCamera_v1_0;

        CTestCamera_v1_0 Camera;
        Camera._LoadDLL();

        // Same tests as in TestNameSpace
        CNodePtr ptrNode;

        ptrNode = Camera._GetNode("Cust::MyDefault");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        ptrNode = Camera._GetNode("Std::MyDefault");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
        ptrNode = Camera._GetNode("MyDefault");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        CPPUNIT_ASSERT_EQUAL(Custom, ptrNode->GetNameSpace() );
        CPPUNIT_ASSERT_EQUAL(gcstring("MyDefault"), ptrNode->GetName() );
        CPPUNIT_ASSERT_EQUAL(gcstring("Cust::MyDefault"), ptrNode->GetName(true) );

        ptrNode = Camera._GetNode("Cust::MyStandard");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
        ptrNode = Camera._GetNode("Std::MyStandard");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        ptrNode = Camera._GetNode("MyStandard");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        CPPUNIT_ASSERT_EQUAL(Standard, ptrNode->GetNameSpace() );
        CPPUNIT_ASSERT_EQUAL(gcstring("MyStandard"), ptrNode->GetName() );
        CPPUNIT_ASSERT_EQUAL(gcstring("Std::MyStandard"), ptrNode->GetName(true) );

        ptrNode = Camera._GetNode("Cust::MyCustom");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        ptrNode = Camera._GetNode("Std::MyCustom");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
        ptrNode = Camera._GetNode("MyCustom");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        CPPUNIT_ASSERT_EQUAL(Custom, ptrNode->GetNameSpace() );
        CPPUNIT_ASSERT_EQUAL(gcstring("MyCustom"), ptrNode->GetName() );
        CPPUNIT_ASSERT_EQUAL(gcstring("Cust::MyCustom"), ptrNode->GetName(true) );

        ptrNode = Camera._GetNode("Trallala::MyDefault");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );

        ptrNode = Camera._GetNode("Std::Trallala");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );

        ptrNode = Camera._GetNode("Trallala");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
    }
    {
        using namespace Test_TestCamera_v1_1;

        CTestCamera_v1_1 Camera;
        Camera._LoadDLL();

        // Same tests as in TestNameSpace
        CNodePtr ptrNode;

        ptrNode = Camera._GetNode("Cust::MyDefault");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        ptrNode = Camera._GetNode("Std::MyDefault");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
        ptrNode = Camera._GetNode("MyDefault");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        CPPUNIT_ASSERT_EQUAL(Custom, ptrNode->GetNameSpace() );
        CPPUNIT_ASSERT_EQUAL(gcstring("MyDefault"), ptrNode->GetName() );
        CPPUNIT_ASSERT_EQUAL(gcstring("Cust::MyDefault"), ptrNode->GetName(true) );

        ptrNode = Camera._GetNode("Cust::MyStandard");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
        ptrNode = Camera._GetNode("Std::MyStandard");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        ptrNode = Camera._GetNode("MyStandard");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        CPPUNIT_ASSERT_EQUAL(Standard, ptrNode->GetNameSpace() );
        CPPUNIT_ASSERT_EQUAL(gcstring("MyStandard"), ptrNode->GetName() );
        CPPUNIT_ASSERT_EQUAL(gcstring("Std::MyStandard"), ptrNode->GetName(true) );

        ptrNode = Camera._GetNode("Cust::MyCustom");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        ptrNode = Camera._GetNode("Std::MyCustom");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
        ptrNode = Camera._GetNode("MyCustom");
        CPPUNIT_ASSERT( ptrNode.IsValid() );
        CPPUNIT_ASSERT_EQUAL(Custom, ptrNode->GetNameSpace() );
        CPPUNIT_ASSERT_EQUAL(gcstring("MyCustom"), ptrNode->GetName() );
        CPPUNIT_ASSERT_EQUAL(gcstring("Cust::MyCustom"), ptrNode->GetName(true) );

        ptrNode = Camera._GetNode("Trallala::MyDefault");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );

        ptrNode = Camera._GetNode("Std::Trallala");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );

        ptrNode = Camera._GetNode("Trallala");
        CPPUNIT_ASSERT( !ptrNode.IsValid() );
    }
}

void NodeTestSuite::TestNameSpace()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <Category Name=\"Root\">                            \n"
    "        <pFeature>MyDefault</pFeature>                    \n"
    "        <pFeature>MyStandard</pFeature>                    \n"
    "        <pFeature>MyCustom</pFeature>                    \n"
    "    </Category>                                            \n"
    "    <Node Name=\"MyDefault\"/>                            \n"
    "    <Node Name=\"MyStandard\" NameSpace=\"Standard\"/>    \n"
    "    <Node Name=\"MyCustom\" NameSpace=\"Custom\"/>        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrNode;

    ptrNode = Camera._GetNode("Cust::MyDefault");
    CPPUNIT_ASSERT( ptrNode.IsValid() );
    ptrNode = Camera._GetNode("Std::MyDefault");
    CPPUNIT_ASSERT( !ptrNode.IsValid() );
    ptrNode = Camera._GetNode("MyDefault");
    CPPUNIT_ASSERT( ptrNode.IsValid() );
    CPPUNIT_ASSERT_EQUAL(Custom, ptrNode->GetNameSpace() );
    CPPUNIT_ASSERT_EQUAL(gcstring("MyDefault"), ptrNode->GetName() );
    CPPUNIT_ASSERT_EQUAL(gcstring("Cust::MyDefault"), ptrNode->GetName(true) );

    ptrNode = Camera._GetNode("Cust::MyStandard");
    CPPUNIT_ASSERT( !ptrNode.IsValid() );
    ptrNode = Camera._GetNode("Std::MyStandard");
    CPPUNIT_ASSERT( ptrNode.IsValid() );
    ptrNode = Camera._GetNode("MyStandard");
    CPPUNIT_ASSERT( ptrNode.IsValid() );
    CPPUNIT_ASSERT_EQUAL(Standard, ptrNode->GetNameSpace() );
    CPPUNIT_ASSERT_EQUAL(gcstring("MyStandard"), ptrNode->GetName() );
    CPPUNIT_ASSERT_EQUAL(gcstring("Std::MyStandard"), ptrNode->GetName(true) );

    ptrNode = Camera._GetNode("Cust::MyCustom");
    CPPUNIT_ASSERT( ptrNode.IsValid() );
    ptrNode = Camera._GetNode("Std::MyCustom");
    CPPUNIT_ASSERT( !ptrNode.IsValid() );
    ptrNode = Camera._GetNode("MyCustom");
    CPPUNIT_ASSERT( ptrNode.IsValid() );
    CPPUNIT_ASSERT_EQUAL(Custom, ptrNode->GetNameSpace() );
    CPPUNIT_ASSERT_EQUAL(gcstring("MyCustom"), ptrNode->GetName() );
    CPPUNIT_ASSERT_EQUAL(gcstring("Cust::MyCustom"), ptrNode->GetName(true) );

    ptrNode = Camera._GetNode("Trallala::MyDefault");
    CPPUNIT_ASSERT( !ptrNode.IsValid() );

    ptrNode = Camera._GetNode("Std::Trallala");
    CPPUNIT_ASSERT( !ptrNode.IsValid() );

    ptrNode = Camera._GetNode("Trallala");
    CPPUNIT_ASSERT( !ptrNode.IsValid() );
}

void NodeTestSuite::TestPropertyAccess()
{
    // Note that DisplayName is handled slightly different way for EnumEntry
    // where Symbolic is also considered
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <Node Name=\"MyName\" NameSpace=\"Standard\">                \n"
    "        <ToolTip>MyToolTip</ToolTip>                            \n"
    "        <Description>MyDescription</Description>                    \n"
    "        <DisplayName>MyDisplayName</DisplayName>                \n"
    "        <Visibility>Guru</Visibility>                            \n"
    "    </Node>                                                        \n"
    ""
    "<Enumeration Name=\"Enum0\">                            \n"
    "    <EnumEntry Name=\"EnumEntry0\">                    \n"
    "        <DisplayName>EnumDisplayName</DisplayName>                                \n"
    "        <Value>0</Value>                                \n"
    "    </EnumEntry>                                        \n"
    "    <EnumEntry Name=\"EnumEntry1\">                    \n"
    "        <Value>1</Value>                                \n"
    "        <Symbolic>EnumSymbolic</Symbolic>                                \n"
    "    </EnumEntry>                                        \n"
    "    <EnumEntry Name=\"EnumEntry2\">                    \n"
    "        <Value>2</Value>                                \n"
    "    </EnumEntry>                                        \n"
    "    <Value>0</Value>                                \n"
    "</Enumeration>                                            \n"
    ""
   + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrMyName = Camera._GetNode("MyName");
    CPPUNIT_ASSERT( ptrMyName.IsValid() );

    CPPUNIT_ASSERT_EQUAL(RW, ptrMyName->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL(gcstring("MyName"), ptrMyName->GetName() );
    CPPUNIT_ASSERT_EQUAL(gcstring("Std::MyName"), ptrMyName->GetName(true) );
    CPPUNIT_ASSERT_EQUAL(Standard, ptrMyName->GetNameSpace() );
    CPPUNIT_ASSERT_EQUAL(gcstring("MyDisplayName"), ptrMyName->GetDisplayName() );
    CPPUNIT_ASSERT_EQUAL(Guru, ptrMyName->GetVisibility() );
    CPPUNIT_ASSERT_EQUAL(gcstring("MyToolTip"), ptrMyName->GetToolTip() );
    CPPUNIT_ASSERT_EQUAL(gcstring("MyDescription"), ptrMyName->GetDescription() );
}

void NodeTestSuite::TestCachingMode()
{
  gcstring CameraDescriptionFile =

    CDFHeader +

    " <IntReg Name=\"Int01\" > \n"
    "   <Address>0x10</Address> \n"
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>NoCache</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int02\" > \n"
    "   <Address>0x12</Address> \n"
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteThrough</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int03\" > \n"
    "   <Address>0x14</Address> \n"
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteAround</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int04\" > \n"
    "   <pAddress>Int01</pAddress> \n"   // note: a connection to a node with NoCache
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteAround</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int05\" > \n"
    "   <pAddress>Int02</pAddress> \n"  // note: a connection to a node with WriteThrough
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteAround</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int06\" > \n"
    "   <pAddress>Int03</pAddress> \n"  // note: a connection to a node with WriteAround
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteAround</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int07\" > \n"
    "   <pAddress>Int01</pAddress> \n"   // note: a connection to a node with NoCache
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteThrough</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int08\" > \n"
    "   <pAddress>Int02</pAddress> \n"  // note: a connection to a node with WriteThrough
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteThrough</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int09\" > \n"
    "   <pAddress>Int03</pAddress> \n"   // note: a connection to a node with WriteAround
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Cachable>WriteThrough</Cachable> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int10\" > \n" // note : no explicite CachingMode given
    "   <Address>0</Address> \n"
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    " <IntReg Name=\"Int11\" > \n"
    "   <pAddress>Int10</pAddress> \n"  // note: a connection to a node with implicite WriteThrough
    "   <Length>2</Length> \n"
    "   <AccessMode>RW</AccessMode> \n"
    "   <pPort>MyPort</pPort> \n"
    "   <Sign>Unsigned</Sign> \n"
    "   <Endianess>LittleEndian</Endianess> \n"
    " </IntReg> \n"

    "    <Integer Name=\"IntNoCache\">                               \n"
    "    <pValue>Int01</pValue>                                       \n"
    "    </Integer>                                             \n"

    "    <Integer Name=\"IntThrough\">                               \n"
    "    <pValue>Int02</pValue>                                       \n"
    "    </Integer>                                             \n"

    "    <Integer Name=\"IntAround\">                               \n"
    "    <pValue>Int03</pValue>                                       \n"
    "    </Integer>                                             \n"

    "    <Converter Name=\"ConvAround\">                            \n"
    "        <FormulaTo> FROM </FormulaTo>                \n"
    "        <FormulaFrom> TO </FormulaFrom>                \n"
    "        <pValue>Int03</pValue>                                \n"
    "        <Slope>Increasing</Slope>                                \n"
    "    </Converter>                                                \n"

    "    <Float Name=\"FloatAround\">                               \n"
    "    <pValue>ConvAround</pValue>                                       \n"
    "    </Float>                                             \n"

    " <Port Name=\"MyPort\"/> \n"

    + CDFFooter;

  CNodeMapRef Camera;
  Camera._LoadXMLFromString( CameraDescriptionFile );

  CNodePtr ptrInt01 = Camera._GetNode( gcstring( "Int01" ) );
  CNodePtr ptrInt02 = Camera._GetNode( gcstring( "Int02" ) );
  CNodePtr ptrInt03 = Camera._GetNode( gcstring( "Int03" ) );
  CNodePtr ptrInt04 = Camera._GetNode( gcstring( "Int04" ) );
  CNodePtr ptrInt05 = Camera._GetNode( gcstring( "Int05" ) );
  CNodePtr ptrInt06 = Camera._GetNode( gcstring( "Int06" ) );
  CNodePtr ptrInt07 = Camera._GetNode( gcstring( "Int07" ) );
  CNodePtr ptrInt08 = Camera._GetNode( gcstring( "Int08" ) );
  CNodePtr ptrInt09 = Camera._GetNode( gcstring( "Int09" ) );
  CNodePtr ptrInt10 = Camera._GetNode( gcstring( "Int10" ) );
  CNodePtr ptrInt11 = Camera._GetNode( gcstring( "Int11" ) );
  CPPUNIT_ASSERT( ptrInt01.IsValid() );
  CPPUNIT_ASSERT( ptrInt02.IsValid() );
  CPPUNIT_ASSERT( ptrInt03.IsValid() );
  CPPUNIT_ASSERT( ptrInt04.IsValid() );
  CPPUNIT_ASSERT( ptrInt05.IsValid() );
  CPPUNIT_ASSERT( ptrInt06.IsValid() );
  CPPUNIT_ASSERT( ptrInt07.IsValid() );
  CPPUNIT_ASSERT( ptrInt08.IsValid() );
  CPPUNIT_ASSERT( ptrInt09.IsValid() );
  CPPUNIT_ASSERT( ptrInt10.IsValid() );
  CPPUNIT_ASSERT( ptrInt11.IsValid() );

  CPPUNIT_ASSERT_EQUAL( NoCache, ptrInt01->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteThrough, ptrInt02->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteAround, ptrInt03->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( NoCache, ptrInt04->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteAround, ptrInt05->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteAround, ptrInt06->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( NoCache, ptrInt07->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteThrough, ptrInt08->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteAround, ptrInt09->GetCachingMode() );
  // ensure that the default caching mode is WriteThrough (single node)
  CPPUNIT_ASSERT_EQUAL( WriteThrough, ptrInt10->GetCachingMode() );
  // ensure that the default caching mode is WriteThrough (for both, child and parent node, there is no caching mode specified
  CPPUNIT_ASSERT_EQUAL( WriteThrough, ptrInt11->GetCachingMode() );

     // create and initialize a test port
    CTestPort Port;
//    short s = (short)0xfdfd; // *JS* removed warning
    short s = (short)(unsigned short)0xfdfd;
    Port.CreateEntry(0x0010, 2UL, &s, RW);
    Port.CreateEntry(0x0012, 2UL, &s, RW);
    Port.CreateEntry(0x0014, 2UL, &s, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  // No cache
  CIntegerPtr ptrInt(ptrInt01);
  CPPUNIT_ASSERT( ptrInt.IsValid() );
  ptrInt->SetValue( 42 );
  CPPUNIT_ASSERT_EQUAL ( (int64_t)42LL, ptrInt->GetValue() );
  // Write through
  ptrInt = ptrInt02;
  const int64_t n1(4711);
  *ptrInt = n1;
  CPPUNIT_ASSERT_EQUAL( n1, ptrInt->GetValue() );
  // Write around
  ptrInt = ptrInt03;
  const int64_t n2(123);
  *ptrInt = n2;
  CPPUNIT_ASSERT_EQUAL( n2, ptrInt->GetValue() );

  // Let's play directly with the values within the port
  CIntegerPtr ptrNoCache = Camera._GetNode("Int01");
  CIntegerPtr ptrThrough = Camera._GetNode("Int02");
  CIntegerPtr ptrAround = Camera._GetNode("Int03");
  CPPUNIT_ASSERT_EQUAL( NoCache, ptrNoCache->GetNode()->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteThrough, ptrThrough->GetNode()->GetCachingMode() );
  CPPUNIT_ASSERT_EQUAL( WriteAround, ptrAround->GetNode()->GetCachingMode() );
  CIntegerPtr ptrIntNoCache = Camera._GetNode("IntNoCache");
  CIntegerPtr ptrIntThrough = Camera._GetNode("IntThrough");
  CIntegerPtr ptrIntAround = Camera._GetNode("IntAround");
  *ptrNoCache = 0;
  *ptrThrough = 0;
  *ptrAround = 0;
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrNoCache->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrAround->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrThrough->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrIntNoCache->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrIntAround->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrIntThrough->GetValue() );
  // Volatile change of value is visible only to NoCache nodes
  short Value = 1;
  Port.Write (&Value, 0x10, sizeof(Value));
  Port.Write (&Value, 0x12, sizeof(Value));
  Port.Write (&Value, 0x14, sizeof(Value));
  CPPUNIT_ASSERT_EQUAL( (int64_t)1LL, ptrNoCache->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrAround->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrThrough->GetValue() );
  Value = 2;
  Port.Write (&Value, 0x10, sizeof(Value));
  Port.Write (&Value, 0x12, sizeof(Value));
  Port.Write (&Value, 0x14, sizeof(Value));
  CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, ptrIntNoCache->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrIntAround->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrIntThrough->GetValue() );
  // Adapting the value after writing is visible to all but WriteThrough node
  *ptrNoCache = 3;
  *ptrThrough = 3;
  *ptrAround = 3;
  Value = 4;
  Port.Write (&Value, 0x10, sizeof(Value));
  Port.Write (&Value, 0x12, sizeof(Value));
  Port.Write (&Value, 0x14, sizeof(Value));
  CPPUNIT_ASSERT_EQUAL( (int64_t)4LL, ptrNoCache->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)4LL, ptrAround->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)3LL, ptrThrough->GetValue() );
  *ptrIntNoCache = 5;
  *ptrIntThrough = 5;
  *ptrIntAround = 5;
  Value = 6;
  Port.Write (&Value, 0x10, sizeof(Value));
  Port.Write (&Value, 0x12, sizeof(Value));
  Port.Write (&Value, 0x14, sizeof(Value));
  CPPUNIT_ASSERT_EQUAL( (int64_t)6LL, ptrIntNoCache->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)6LL, ptrIntAround->GetValue() );
  CPPUNIT_ASSERT_EQUAL( (int64_t)5LL, ptrIntThrough->GetValue() );
  // make sure the cache is avoided if the write fails
  *ptrThrough = 7;
  CPPUNIT_ASSERT_THROW (*ptrThrough = 0xFFFFFFFF, OutOfRangeException);
  CPPUNIT_ASSERT_EQUAL( (int64_t)7LL, ptrThrough->GetValue() );
  *ptrThrough = 7;
  CPPUNIT_ASSERT_THROW (*ptrThrough = 0xFFFFFFFF, OutOfRangeException);
  CPPUNIT_ASSERT_EQUAL( (int64_t)7LL, ptrThrough->GetValue() );

  // repeat the write-around tests also with float
  CFloatPtr ptrFloatAround = Camera._GetNode("FloatAround");
  *ptrFloatAround = 50.0;
  Value = 60;
  Port.Write (&Value, 0x14, sizeof(Value));
  CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)60.0, ptrFloatAround->GetValue(), std::numeric_limits<double>::epsilon() );
}

void NodeTestSuite::TestDeviceInformation()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <Node Name=\"Dummy\"/>                            \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CDeviceInfoPtr ptrDeviceInfo(Camera._Ptr);

    Version_t Version;
    ptrDeviceInfo->GetDeviceVersion(Version);
    CPPUNIT_ASSERT_EQUAL( (uint16_t)2, Version.Major );
    CPPUNIT_ASSERT_EQUAL( (uint16_t)3, Version.Minor );
    CPPUNIT_ASSERT_EQUAL( (uint16_t)4, Version.SubMinor );

    uint16_t Build;
    ptrDeviceInfo->GetGenApiVersion(Version, Build);
    CPPUNIT_ASSERT_EQUAL( (uint16_t)GENICAM_VERSION_MAJOR, Version.Major );
    CPPUNIT_ASSERT_EQUAL( (uint16_t)GENICAM_VERSION_MINOR, Version.Minor );
    CPPUNIT_ASSERT_EQUAL( (uint16_t)GENICAM_VERSION_SUBMINOR, Version.SubMinor );
    CPPUNIT_ASSERT_EQUAL( (uint16_t)GENICAM_VERSION_BUILD, Build );

    ptrDeviceInfo->GetSchemaVersion(Version);
    switch(GenApiSchemaVersion)
    {
    case v1_0:
        CPPUNIT_ASSERT_EQUAL( (uint16_t)1, Version.Major );
        CPPUNIT_ASSERT_EQUAL( (uint16_t)0, Version.Minor );
        CPPUNIT_ASSERT_EQUAL( (uint16_t)1, Version.SubMinor );
        break;
    case v1_1:
        CPPUNIT_ASSERT_EQUAL( (uint16_t)1, Version.Major );
        CPPUNIT_ASSERT_EQUAL( (uint16_t)1, Version.Minor );
        CPPUNIT_ASSERT_EQUAL( (uint16_t)0, Version.SubMinor );
        break;
    default:
        CPPUNIT_ASSERT(false);
    }
    CPPUNIT_ASSERT_EQUAL( gcstring("GenApiTest"), ptrDeviceInfo->GetModelName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Generic"), ptrDeviceInfo->GetVendorName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("2D932CC6-EB68-40bd-B6CC-F03B55B7D653"), ptrDeviceInfo->GetProductGuid() );
    CPPUNIT_ASSERT_EQUAL( gcstring("02A8C268-BEE8-463b-A6C0-53ED8256E3D8"), ptrDeviceInfo->GetVersionGuid() );
    CPPUNIT_ASSERT_EQUAL( gcstring("nodes for testing the GenApi reference implementation"), ptrDeviceInfo->GetToolTip() );
    CPPUNIT_ASSERT_EQUAL( gcstring("GEV"), ptrDeviceInfo->GetStandardNameSpace() );
}



void NodeTestSuite::TestValueNode()
{
    {
        using namespace Test_TestCamera_v1_0;

        CTestCamera_v1_0 Camera;
        Camera._LoadDLL();

        CValuePtr ptrValue = Camera._GetNode("MyHelpNode1");

        ptrValue->FromString ("foobar");
        CPPUNIT_ASSERT_EQUAL( gcstring("foobar"), ptrValue->ToString() );
    }
    {
        using namespace Test_TestCamera_v1_1;

        CTestCamera_v1_1 Camera;
        Camera._LoadDLL();

        CValuePtr ptrValue = Camera._GetNode("MyHelpNode1");

        ptrValue->FromString ("foobar");
        CPPUNIT_ASSERT_EQUAL( gcstring("foobar"), ptrValue->ToString() );
    }
}


void NodeTestSuite::TestTheRest()
{
     gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        " <IntReg Name=\"IntChild\" > \n"
        "   <Address>0</Address> \n"
        "   <Length>2</Length> \n"
        "   <AccessMode>RW</AccessMode> \n"
        "   <pPort>MyPort</pPort> \n"
        "   <Sign>Unsigned</Sign> \n"
        "   <Endianess>LittleEndian</Endianess> \n"
        " </IntReg> \n"
        " <IntReg Name=\"IntParent\" > \n"
        "   <pAlias>IntChild</pAlias> \n"
        "   <pAddress>IntChild</pAddress> \n"
        "   <Length>2</Length> \n"
        "   <AccessMode>RW</AccessMode> \n"
        "   <pPort>MyPort</pPort> \n"
        "   <Sign>Unsigned</Sign> \n"
        "   <Endianess>LittleEndian</Endianess> \n"
        " </IntReg> \n"
        " <Port Name=\"MyPort\"/> \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString( CameraDescriptionFile );

    CNodePrivatePtr ptrIntChild = Camera._GetNode( gcstring( "IntChild" ) );
    CNodePrivatePtr ptrIntParent = Camera._GetNode( gcstring( "IntParent" ) );

    NodeList_t NodeList;
    ptrIntChild->GetParents (NodeList);
    CPPUNIT_ASSERT( NodeList.size()==1 );
    CPPUNIT_ASSERT_EQUAL( gcstring("IntParent"), NodeList[0]->GetName() );

    CNodeMapPtr ptrNodeMap = ptrIntChild->GetNodeMap();
    CPPUNIT_ASSERT ( ptrNodeMap.IsValid() );

    CPPUNIT_ASSERT_EQUAL( gcstring("IntChild"), ptrIntParent->GetAlias()->GetName() );
    CPPUNIT_ASSERT_EQUAL( (INode*)NULL, ptrIntChild->GetAlias() );

}

void NodeTestSuite::TestStringConversions()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Integer Name=\"TestInt\">                \n"
    "    <Value>0</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Register Name=\"TestReg\">            \n"
    "    <Address>0x0000</Address>            \n"
    "    <Length>1</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "</Register>                            \n"
    "                                        \n"
    "<Register Name=\"TestReg2L\">            \n"
    "    <Address>0x0110</Address>            \n"
    "    <Length>2</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "</Register>                            \n"
    "                                        \n"
    "<Register Name=\"TestReg2B\">            \n"
    "    <Address>0x0120</Address>            \n"
    "    <Length>2</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "</Register>                            \n"
    "                                        \n"
    "<Register Name=\"TestRegBig\">            \n"
    "    <Address>0x0100</Address>            \n"
    "    <Length>128</Length>                \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "</Register>                            \n"
    "                                        \n"
    "<Port Name=\"Port\"/>                    \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrInt = Camera._GetNode("TestInt");

    uint8_t RegVal = 0;
    uint16_t RegVal2L = 0;
    uint16_t RegVal2B = 0;
    uint8_t BigRegVal[128];
    memset (BigRegVal, 0, sizeof(BigRegVal));

    CTestPort Port;
    Port.CreateEntry(0x0000, RegVal, RW);
    Port.CreateEntry(0x0110, RegVal2L, RW);
    Port.CreateEntry(0x0120, RegVal2B, RW);
    Port.CreateEntry(0x0100, sizeof(BigRegVal), BigRegVal, RW);
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("TestReg");
    CRegisterPtr ptrRegister2L = Camera._GetNode("TestReg2L");
    CRegisterPtr ptrRegister2B = Camera._GetNode("TestReg2B");
    CRegisterPtr ptrBigRegister = Camera._GetNode("TestRegBig");

    CPPUNIT_ASSERT_EQUAL (gcstring("IRegister"), GetInterfaceName(ptrRegister));

    // test options not yet covered by other tests
    ptrInt->FromString("0X100");
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0x100LL, ptrInt->GetValue() );
    ptrInt->FromString("0x200");
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0x200LL, ptrInt->GetValue() );
    // and the old '&h' options, no more supported
    CPPUNIT_ASSERT_THROW( ptrInt->FromString("&h1"), GenICam::InvalidArgumentException );
    CPPUNIT_ASSERT_THROW( ptrInt->FromString("&HFF"), GenICam::InvalidArgumentException );
    CPPUNIT_ASSERT_THROW( ptrInt->FromString("&FFF"), GenICam::InvalidArgumentException );

    // the same for byte arrays
    CPPUNIT_ASSERT_EQUAL( gcstring("0x00"), ptrRegister->ToString() );
    gcstring bigRegisterStr = ptrBigRegister->ToString();
    CPPUNIT_ASSERT_EQUAL( 2+ 2*sizeof BigRegVal, bigRegisterStr.length() );
    CPPUNIT_ASSERT_EQUAL( gcstring("0x00"), bigRegisterStr.substr(0,4 ) );
    for(int i=1; i<sizeof BigRegVal;++i)
        CPPUNIT_ASSERT_EQUAL( gcstring("00"), bigRegisterStr.substr(2*(i+1),2) );


    // single byte register, including some invalid strings
    // (the string should start with 0x following by one or more pairs of hexa-digits)
    CPPUNIT_ASSERT_THROW( ptrRegister->FromString ("1"), GenICam::InvalidArgumentException);
    CPPUNIT_ASSERT_THROW( ptrRegister->FromString ("0x"), GenICam::InvalidArgumentException);
    CPPUNIT_ASSERT_THROW( ptrRegister->FromString ("0x1"), GenICam::InvalidArgumentException);
    CPPUNIT_ASSERT_THROW( ptrRegister->FromString ("0x123"), GenICam::InvalidArgumentException);

    ptrRegister->FromString ("0xAB");
    ptrRegister->Get((uint8_t*)&RegVal, sizeof(RegVal));
    CPPUNIT_ASSERT_EQUAL( (uint8_t)0xAB, RegVal );
    CPPUNIT_ASSERT_EQUAL( gcstring("0xab"), ptrRegister->ToString() );

    // 2-byte registers, both little and big endian
    ptrRegister2L->FromString ("0xaabb");
    ptrRegister2L->Get((uint8_t*)&RegVal2L, sizeof(RegVal2L));
    CPPUNIT_ASSERT_EQUAL( (uint16_t)0xBBAA, RegVal2L );
    CPPUNIT_ASSERT_EQUAL( gcstring("0xaabb"), ptrRegister2L->ToString() );

    ptrRegister2B->FromString ("0xaabb");
    ptrRegister2B->Get((uint8_t*)&RegVal2B, sizeof(RegVal2B));
    CPPUNIT_ASSERT_EQUAL( (uint16_t)0xBBAA, RegVal2B );
    CPPUNIT_ASSERT_EQUAL( gcstring("0xaabb"), ptrRegister2B->ToString() );

}

void NodeTestSuite::TestBool()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                        \n"
    "    <Node Name=\"Node\">                                \n"
    "        <pIsImplemented>Implemented</pIsImplemented>    \n"
    "        <pIsAvailable>Available</pIsAvailable>            \n"
    "        <pIsLocked>Locked</pIsLocked>                    \n"
    "    </Node>                                                \n"
    "                                                        \n"
    "    <Boolean Name=\"Implemented\">                        \n"
    "        <Value>true</Value>                                \n"
    "    </Boolean>                                            \n"
    "                                                        \n"
    "    <Boolean Name=\"Available\">                        \n"
    "        <Value>true</Value>                                \n"
    "    </Boolean>                                            \n"
    "                                                        \n"
    "    <Boolean Name=\"Locked\">                            \n"
    "        <Value>false</Value>                            \n"
    "    </Boolean>                                            \n"
    "                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrNode = Camera._GetNode("Node");
    CBooleanPtr ptrImplemented = Camera._GetNode("Implemented");
    CBooleanPtr ptrAvailable = Camera._GetNode("Available");
    CBooleanPtr ptrLocked = Camera._GetNode("Locked");

    // Check Implemented
    *ptrImplemented = false;
    CPPUNIT_ASSERT_EQUAL( NI, ptrNode->GetAccessMode() );
    *ptrImplemented = true;

    // Check Available
    *ptrAvailable = false;
    CPPUNIT_ASSERT_EQUAL( NA, ptrNode->GetAccessMode() );
    *ptrAvailable = true;

    // Check Locked
    *ptrLocked = true;
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode->GetAccessMode() );
    *ptrLocked = false;
}

void NodeTestSuite::TestImposeAccessMode()
{

   CallbackObject TestRegCb;
   gcstring CameraDescriptionFile =
        CDFHeader +
        "                                       \n"
        "<Integer Name=\"TestInt\">              \n"
        "    <Value>0</Value>                    \n"
        "</Integer>                              \n"
        "                                        \n"
        "<Register Name=\"TestReg\">             \n"
        "    <Address>0x0000</Address>           \n"
        "    <Length>1</Length>                  \n"
        "    <AccessMode>RW</AccessMode>         \n"
        "    <pPort>Port</pPort>                 \n"
        "</Register>                             \n"
        "<Category Name=\"Root\">               \n"
        "     <pFeature>TestInt</pFeature>      \n"
        "</Category>                            \n"
        "                                       \n"
        "<Port Name=\"Port\"/>                  \n"
        + CDFFooter;

    CNodeMapRef Camera;
    CTestPort Port;

    // now initialize map
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CNodeMapDynPtr pNodeMapDyn(Camera._Ptr);
    CPPUNIT_ASSERT_NO_THROW( Camera._Connect(&Port, "Port") );


    // test the access mode
    CNodePtr ptrValue = Camera._GetNode( "TestReg" );
    GenApi::Register( ptrValue, TestRegCb, &CallbackObject::Callback );
    CPPUNIT_ASSERT_EQUAL_MESSAGE( "Expected Imposed Access mode is taken in account.", RW, ptrValue->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL_MESSAGE( "Expected Imposed Access mode is taken in account.", RW, ptrValue->GetAccessMode() );
    ptrValue->ImposeAccessMode( RO );
    CPPUNIT_ASSERT_EQUAL_MESSAGE( "Callback not fired by ImposeAccessMode.", (uint32_t)1, TestRegCb.Count() );
    CPPUNIT_ASSERT_EQUAL_MESSAGE( "Expected Imposed Access mode is taken in account.", RO, ptrValue->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL_MESSAGE( "Expected Imposed Access mode is taken in account.", RO, ptrValue->GetAccessMode() );
}


void NodeTestSuite::TestConst()
{
   gcstring CameraDescriptionFile =
        CDFHeader +
        "                                       \n"
        "<Integer Name=\"TestInt\">              \n"
        "    <Value>0</Value>                    \n"
        "</Integer>                              \n"
        "                                        \n"
        "<Register Name=\"TestReg\">             \n"
        "    <Address>0x0000</Address>           \n"
        "    <Length>1</Length>                  \n"
        "    <AccessMode>RW</AccessMode>         \n"
        "    <pPort>Port</pPort>                 \n"
        "</Register>                             \n"
        "<Category Name=\"Root\">               \n"
        "     <pFeature>TestInt</pFeature>      \n"
        "</Category>                            \n"
        "                                       \n"
        "<Port Name=\"Port\"/>                  \n"
        + CDFFooter;

    CNodeMapRef Camera;
    CTestPort Port;
//    NodeList_t NodeList;

    // now test initialized map
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CNodeMapDynPtr pNodeMapDyn(Camera._Ptr);
    CPPUNIT_ASSERT_NO_THROW( Camera._Connect(&Port, "Port") );

    // node pointer
    const INode * const pNode = Camera._GetNode( "Root" );
    if (IsReadable( pNode )) ;
    else if (IsWritable( pNode )) ;
    else if (IsAvailable( pNode ));
    else if (IsImplemented( pNode ));
    else CPPUNIT_ASSERT( 0 && "unexpected AccessMode" );

    // node reference
    const CNodePtr ptrNode = Camera._GetNode( "Port" );
    const INode& rNode = *ptrNode;
    if (IsReadable( rNode )) ;
    else if (IsWritable( rNode )) ;
    else if (IsAvailable( rNode ));
    else if (IsImplemented( rNode ));
    else CPPUNIT_ASSERT( 0 && "unexpected AccessMode" );

}

void NodeTestSuite::TestReadOnlyPort()
{
   const gcstring CameraDescriptionFile =
        CDFHeader +
        "                                       \n"
        "<Register Name=\"TestReg\">             \n"
        "    <Address>0x0000</Address>           \n"
        "    <Length>1</Length>                  \n"
        "    <AccessMode>RW</AccessMode>         \n"
        "    <pPort>Port</pPort>                 \n"
        "</Register>                             \n"
        "                                       \n"
        "<Port Name=\"Port\"/>                  \n"
        + CDFFooter;

    CNodeMapRef Camera;
    // Extended Test Port declaring that read access is allowed only.
    class CReadonlyTestPort : public CTestPort {
    public:
        GenApi::EAccessMode GetAccessMode() const   { return RO; }
    } Port;
    // Initial register value
    const unsigned char RegVal = 0xcd;
    Port.CreateEntry( 0x0000, RegVal, RW );

    Camera._LoadXMLFromString( CameraDescriptionFile );
    CPPUNIT_ASSERT_NO_THROW( Camera._Connect(&Port, "Port") );
    CPPUNIT_ASSERT_EQUAL( RO, Port.GetAccessMode() );

    CRegisterPtr ptrReg = Camera._GetNode( "TestReg" );
    CPPUNIT_ASSERT( ptrReg.IsValid() );
    unsigned char CurRegVal = 0xff;
    CPPUNIT_ASSERT_EQUAL_MESSAGE( "Access mode is not R0!", RO, ptrReg->GetAccessMode() );
    CPPUNIT_ASSERT_THROW( ptrReg->Set( &CurRegVal, sizeof CurRegVal ), GenICam::AccessException );
    CPPUNIT_ASSERT_NO_THROW( ptrReg->Get( &CurRegVal, sizeof CurRegVal ) );
    CPPUNIT_ASSERT_EQUAL( RegVal, CurRegVal );

}

void NodeTestSuite::TestInfiniteRecursion()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
"                                                           \n"
"    <IntSwissKnife Name=\"WidthMax\">                      \n"
"    <pVariable Name=\"OFFSET\">OffsetValue</pVariable>     \n"
"    <Formula>1000-OFFSET</Formula>                         \n"
"    </IntSwissKnife>                                       \n"
"                                                           \n"
"    <Integer Name=\"WidthValue\">                          \n"
"    <Value>1000</Value>                                    \n"
"    <pMax>WidthMax</pMax>                                  \n"
"    </Integer>                                             \n"
"                                                           \n"
"    <IntSwissKnife Name=\"OffsetMax\">                     \n"
"    <pVariable Name=\"WIDTH\">WidthValue</pVariable>       \n"
"    <Formula>1000-WIDTH</Formula>                          \n"
"    </IntSwissKnife>                                       \n"
"                                                           \n"
"    <Integer Name=\"OffsetValue\">                         \n"
"    <Value>0</Value>                                       \n"
"    <pMax>OffsetMax</pMax>                                 \n"
"    </Integer>                                             \n"
"                                                           \n"
    + CDFFooter;

    CNodeMapRef Camera;
    if(GenApiSchemaVersion == v1_0)
    {
        // for v1.0 schema the loader does not check for write cycles
        CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

        // for backward compatibility this code must run. It works because there
        // are caches in the cycle

        CIntegerPtr ptrWidthValue = Camera._GetNode("WidthValue");
        CPPUNIT_ASSERT( ptrWidthValue.IsValid() );

        CIntegerPtr ptrOffsetMax = Camera._GetNode("OffsetMax");
        CPPUNIT_ASSERT( ptrOffsetMax.IsValid() );

        CPPUNIT_ASSERT_EQUAL ( (int64_t)1000LL, ptrWidthValue->GetValue( true ));
        CPPUNIT_ASSERT_EQUAL ( (int64_t)0LL, ptrOffsetMax->GetValue( true ));
        *ptrWidthValue = 900;
        CPPUNIT_ASSERT_EQUAL ( (int64_t)100LL, ptrOffsetMax->GetValue( true ));

    }
    else
    {
        // for all other schema versions it does
        CPPUNIT_ASSERT_THROW_EX(Camera._LoadXMLFromString(CameraDescriptionFile), GenICam::RuntimeException);

        // catching the error makes sure no node is created
        CIntegerPtr ptrWidthValue = Camera._GetNode("WidthValue");
        CPPUNIT_ASSERT( !ptrWidthValue.IsValid() );
    }



}

void NodeTestSuite::TestURL()
{

    if(GenApiSchemaVersion == v1_0)
        return;


    gcstring CameraDescriptionFile =

        CDFHeader +
        "                                     \n"
        "    <Node Name=\"TheNodeA\">          \n"
        "        <DocuURL><![CDATA[http://www.mycompany.com/docu/MyCamera.pdf#MyFeature]]></DocuURL>\n"
        "    </Node>                          \n"
        "                                     \n"
        "    <Node Name=\"TheNodeB\">          \n"
        "        <DocuURL><![CDATA[http://www.mycompany.com/docu.php?Feature=$(Sys::NodeName)&Value=$(Value)]]></DocuURL>\n"
        "    </Node>                          \n"
        "                                     \n"
        "    <Node Name=\"TheNodeC\">          \n"
        "        <DocuURL><![CDATA[http://www.mycompany.com/docu.php?Vendor=$(Sys::VendorName)&Model=$(Sys::ModelName)&GenApiVersion=$(Sys::GenApiVersion)&DeviceVersion=$(Sys::DeviceVersion)&SchemaVersion=$(Sys::SchemaVersion)]]></DocuURL>\n"
        "    </Node>                          \n"
        "                                     \n"
        "    <Node Name=\"TheNodeD\">          \n"
        "        <DocuURL><![CDATA[http://www.mycompany.com/docu.php?Unk1=$(Sys::NotKnown)&Unk=$(Bla)]]></DocuURL>\n"
        "    </Node>                          \n"
        "                                     \n"
        "    <Node Name=\"TheNodeE\">          \n"
        "        <DocuURL><![CDATA[http://www.mycompany.com/docu.php?App=$(Sys::Application)&OS=$(Sys::OperatingSystem)&Language=$(Sys::Language)]]></DocuURL>\n"
        "    </Node>                          \n"
        "                                     \n"
        "  <Integer Name=\"Value\">            \n"
        "    <Value>10</Value>                    \n"
        "  </Integer>                                \n"
        "                                     \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrTheNodeA = Camera._GetNode("TheNodeA");
    CPPUNIT_ASSERT_EQUAL (gcstring("http://www.mycompany.com/docu/MyCamera.pdf#MyFeature"), ptrTheNodeA->GetDocuURL());
    cout << ptrTheNodeA->GetDocuURL() << "\n";

    CNodePtr ptrTheNodeB = Camera._GetNode("TheNodeB");
    CPPUNIT_ASSERT_EQUAL (gcstring("http://www.mycompany.com/docu.php?Feature=TheNodeB&Value=10"), ptrTheNodeB->GetDocuURL());
    cout << ptrTheNodeB->GetDocuURL() << "\n";

    CNodePtr ptrTheNodeC = Camera._GetNode("TheNodeC");
    CPPUNIT_ASSERT_EQUAL (gcstring("http://www.mycompany.com/docu.php?Vendor=Generic&Model=GenApiTest&GenApiVersion=" GENICAM_VERSION_MAJOR_STR "." GENICAM_VERSION_MINOR_STR "." GENICAM_VERSION_SUBMINOR_STR "&DeviceVersion=2.3.4&SchemaVersion=1.1.0"), ptrTheNodeC->GetDocuURL());
    cout << ptrTheNodeC->GetDocuURL() << "\n";

    CNodePtr ptrTheNodeD = Camera._GetNode("TheNodeD");
    CPPUNIT_ASSERT_EQUAL (gcstring("http://www.mycompany.com/docu.php?Unk1=Unknown&Unk=Unknown"), ptrTheNodeD->GetDocuURL());
    cout << ptrTheNodeD->GetDocuURL() << "\n";

    CNodePtr ptrTheNodeE = Camera._GetNode("TheNodeE");
    cout << "!!!!!! Check manually : " << ptrTheNodeE->GetDocuURL() << "\n";
}

void NodeTestSuite::TestDeprecated()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    {
        gcstring CameraDescriptionFile =

            CDFHeader +
            "                                     \n"
            "    <Node Name=\"TheNode\">          \n"
            "    </Node>                          \n"
            "                                     \n"
            "    <Node Name=\"TheDeprecatedNode\">\n"
            "    <IsDeprecated>Yes</IsDeprecated> \n"
            "    </Node>                          \n"
            "                                     \n"
            + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        CNodePtr ptrTheNode = Camera._GetNode("TheNode");
        CNodePtr ptrTheDeprecatedNode = Camera._GetNode("TheDeprecatedNode");
        CPPUNIT_ASSERT_EQUAL(false, ptrTheNode->IsDeprecated());
        CPPUNIT_ASSERT_EQUAL(true, ptrTheDeprecatedNode->IsDeprecated());
    }
    {
        using namespace Test_TestCamera_v1_1;

        CTestCamera_v1_1 Camera;
        Camera._LoadDLL();

        // this should wire a warning 4995 "name has been marked as deprecated"
        Camera.DeprecatedNode = 42;
    }
}

void NodeTestSuite::TestStructReg()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<StructReg Comment=\"MyStruct\" >       \n"
        "   <Address>2</Address>                 \n"
        "   <Length>2</Length>                   \n"
        "   <AccessMode>RW</AccessMode>          \n"
        "   <pPort>MyPort</pPort>                \n"
        "   <Endianess>BigEndian</Endianess>     \n"
        "   <StructEntry Name=\"Bit0\" NameSpace=\"Custom\" >  \n"
        "      <Bit>0</Bit>                      \n"
        "   </StructEntry>                       \n"
        "   <StructEntry Name=\"Bit1\" NameSpace=\"Custom\" >  \n"
        "      <Bit>1</Bit>                      \n"
        "   </StructEntry>                       \n"
        "   <StructEntry Name=\"Bit2\" NameSpace=\"Custom\" >  \n"
        "      <Bit>2</Bit>                      \n"
        "   </StructEntry>                       \n"
        "</StructReg>                            \n"

        "<Port Name=\"MyPort\"/>                 \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x02, 0xFFFFFFFF, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

    CIntegerPtr ptrBit0(Camera._GetNode( "Bit0" ));
    CPPUNIT_ASSERT( (bool) ptrBit0 );
    CIntegerPtr ptrBit1(Camera._GetNode( "Bit1" ));
    CPPUNIT_ASSERT( (bool) ptrBit1 );
    CIntegerPtr ptrBit2(Camera._GetNode( "Bit2" ));
    CPPUNIT_ASSERT( (bool) ptrBit2 );
    int64_t Bit0 = ptrBit0->GetValue ();
    int64_t Bit1 = ptrBit1->GetValue ();
    int64_t Bit2 = ptrBit2->GetValue ();

}


void NodeTestSuite::TestExtension()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "                                                           \n"
        "    <Node Name=\"CatNode\">                                \n"
        "       <Extension>                                         \n"
        "           <MyFavourite>123</MyFavourite>                  \n"
        "           <AnotherNode>                                   \n"
        "               <WithSubNode>bla</WithSubNode>              \n"
        "               <WithSubNode>blub</WithSubNode>             \n"
        "           </AnotherNode>                                  \n"
        "       </Extension>                                        \n"
        "    </Node>                                                \n"
        "                                                           \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);


}

void NodeTestSuite::TestAccessModeCache()

{

    CTestPort Port;
    uint32_t RegisterA = 0;
    uint32_t RegisterB = 0;
    Port.CreateEntry(0x4000, RegisterA, RW);
    Port.CreateEntry(0x4004, RegisterB, RW);


    gcstring CameraDescriptionFile =

        CDFHeader +

        "<IntReg Name=\"A\">                    \n"
        "   <Address>0x4000</Address>           \n"
        "   <Length>4</Length>                  \n"
        "   <AccessMode>RO</AccessMode>         \n"
        "   <pPort>MyPort</pPort>               \n"
        "   <Cachable>NoCache</Cachable>        \n"
        "   <Sign>Unsigned</Sign>               \n"
        "   <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                              \n"

        "<IntReg Name=\"B\">                    \n"
        "   <Address>0x4004</Address>           \n"
        "   <Length>4</Length>                  \n"
        "   <AccessMode>RO</AccessMode>         \n"
        "   <pPort>MyPort</pPort>               \n"
        "   <Cachable>NoCache</Cachable>        \n"
        "   <Sign>Unsigned</Sign>               \n"
        "   <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                              \n"

        "<IntSwissKnife Name=\"C\">                     \n"
        "   <pVariable Name=\"VAR_A\">A</pVariable>     \n"
        "   <pVariable Name=\"VAR_B\">B</pVariable>     \n"
        "   <Formula>(VAR_A + VAR_B) > 0</Formula>      \n"
        "</IntSwissKnife>                               \n"

        "<Integer Name=\"D\">                           \n"
        "   <pIsAvailable>C</pIsAvailable>              \n"
        "   <Value>0</Value>                            \n"
        "</Integer>                                     \n"

        "<Port Name=\"MyPort\"/>                        \n"

        + CDFFooter;


    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    Camera._Connect(&Port, "MyPort");

    GenApi::IInteger *lA = dynamic_cast<IInteger *>(Camera._GetNode("A"));
    GenApi::IInteger *lB = dynamic_cast<IInteger *>(Camera._GetNode("B"));
    GenApi::IInteger *lD = dynamic_cast<IInteger *>(Camera._GetNode("D"));

    // Validate registers initial state
    CPPUNIT_ASSERT(lA->GetValue() == 0);
    CPPUNIT_ASSERT(lB->GetValue() == 0);

    // Validate expected original access mode: NA.
    // Required as it populates the access mode cache.
    CPPUNIT_ASSERT(!IsAvailable(lD->GetAccessMode()));

    // Directly change A value
    uint32_t lNewAVal = 4;
    EAccessMode lAccessA; // Out, irrelevant
    Port.UpdateEntry(0x4000, sizeof(lNewAVal), &lNewAVal, lAccessA);

    // Directly change B value
    uint32_t lNewBVal = 2;
    EAccessMode lAccessB; // Out, irrelevant
    Port.UpdateEntry(0x4004, sizeof(lNewBVal), &lNewBVal, lAccessB);

    // Now the big test - with 0000378 fixed, should pass
    CPPUNIT_ASSERT(IsAvailable(lD->GetAccessMode()));

}
/* ================== type definitions ========================== */
// have to be defined at file scope for VC71
#   pragma pack(push, 1)
    typedef struct _TestIsUncached_t
    {                                               // addr:len
        PACK_STRUCT_FIELD(uint32_t Value);          // 0:4
    } PACK_STRUCT_STRUCT TestIsUncached_t;          // 4
#   pragma pack(pop)

void NodeTestSuite::TestIsUncached()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescriptionFile =

        CDFHeader +
        "                                               \n"
        "<Integer Name=\"Value\">                       \n"
        "    <pValue>ValueReg</pValue>               \n"
        "</Integer>                                     \n"
        "                                               \n"
        "<IntReg Name=\"ValueReg\">                     \n"
        "    <pBlockPolling>PollingDisabler</pBlockPolling>   \n"
        "    <Address>0x0000</Address>                  \n"
        "    <Length>4</Length>                         \n"
        "    <AccessMode>RW</AccessMode>                \n"
        "    <pPort>Port</pPort>                        \n"
        "    <PollingTime>1000</PollingTime>                \n"
        "    <Sign>Unsigned</Sign>                      \n"
        "    <Endianess>LittleEndian</Endianess>        \n"
        "</IntReg>                                      \n"
        "                                               \n"
        "<Integer Name=\"PollingDisabler\">               \n"
        "    <Value>0</Value>                           \n"
        "</Integer>                                     \n"
        "                                               \n"
        "<Port Name=\"Port\"/>                    \n"
        "                                               \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // type definition of TestIsUncached is above
    CTestPortStruct<TestIsUncached_t> Port;
    Port.Value = 42;
    Camera._Connect(&Port, "Port");

    CIntegerPtr ptrValue = Camera._GetNode("Value");
    CIntegerPtr ptrValueReg = Camera._GetNode("ValueReg");
    CIntegerPtr ptrPollingDisabler = Camera._GetNode("PollingDisabler");

    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrValue() );

    // Change register value; since the register is cached the Value node will not notice...
    Port.Value = 13;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrValue() );

    // ...until the register is polled
    Camera._Poll(2000);
    CPPUNIT_ASSERT_EQUAL( (int64_t) 13LL, *ptrValue() );

    // Now we do it again but first we block the polling
    Port.Value = 42;
    *ptrPollingDisabler = 1;
    // setting the disabler invalidates the nodes so we have to fill the caches again by reading the Value
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrValue() );

    // Change register value; since the register is cached the Value node will not notice...
    Port.Value = 13;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrValue() );

    // ...until the register is polled
    Camera._Poll(2000);
    // But the polling didn't happen so the cached value is still valid
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrValue() );

    // If we invalidate the Value node explicitly...
    ptrValue->GetNode()->InvalidateNode();
    // ...nothing happens because the cache of the register node was not invalidated
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrValue() );

    // However if we invalidate the ValueReg node explicitely...
    ptrValueReg->GetNode()->InvalidateNode();
    // ... we finally get the value
    CPPUNIT_ASSERT_EQUAL( (int64_t) 13LL, *ptrValue() );

}
