//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "PerformanceTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

#include <cppunitex/LogEx.h>
#include "StopWatch.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( PerformanceTestSuite );

void PerformanceTestSuite::TestDcamScalar()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                    \n"
    "<Integer Name=\"Value\">                            \n"
    "    <pIsImplemented>ValueInq</pIsImplemented>        \n"
    "    <pValue>ValueValue</pValue>                        \n"
    "    <pMin>ValueMin</pMin>                            \n"
    "    <pMax>ValueMax</pMax>                            \n"
    "</Integer>                                            \n"
    "                                                    \n"
    "<MaskedIntReg Name=\"ValueInq\">                    \n"
    "    <Address>0x0000</Address>                        \n"
    "    <Length>4</Length>                                \n"
    "    <AccessMode>RO</AccessMode>                        \n"
    "    <pPort>Port</pPort>                                \n"
    "    <Bit>31</Bit>                                    \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                \n"
    "</MaskedIntReg>                                    \n"
    "                                                    \n"
    "<MaskedIntReg Name=\"ValueValue\">                    \n"
    "    <Address>0x0000</Address>                        \n"
    "    <Length>4</Length>                                \n"
    "    <AccessMode>RW</AccessMode>                        \n"
    "    <pPort>Port</pPort>                                \n"
    "    <LSB>0</LSB>                                    \n"
    "    <MSB>11</MSB>                                    \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                \n"
    "</MaskedIntReg>                                    \n"
    "                                                    \n"
    "<MaskedIntReg Name=\"ValueMin\">                    \n"
    "    <Address>0x0010</Address>                        \n"
    "    <Length>4</Length>                                \n"
    "    <AccessMode>RO</AccessMode>                        \n"
    "    <pPort>Port</pPort>                                \n"
    "    <LSB>0</LSB>                                    \n"
    "    <MSB>11</MSB>                                    \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                \n"
    "</MaskedIntReg>                                    \n"
    "                                                    \n"
    "<MaskedIntReg Name=\"ValueMax\">                    \n"
    "    <Address>0x0010</Address>                        \n"
    "    <Length>4</Length>                                \n"
    "    <AccessMode>RO</AccessMode>                        \n"
    "    <pPort>Port</pPort>                                \n"
    "    <LSB>12</LSB>                                    \n"
    "    <MSB>23</MSB>                                    \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                \n"
    "</MaskedIntReg>                                    \n"
    "                                                    \n"
    "<Port Name=\"Port\"/>                                \n"
    "                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x0000, 0x80000fffUL, RW);
    Port.CreateEntry(0x0010, 0x00fff000UL, RO);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CNodePtr ptrPort = Camera._GetNode("Port");
    CPPUNIT_ASSERT( ptrPort.IsValid() );

    CIntegerPtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    CPPUNIT_ASSERT_EQUAL( RW, ptrValue->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, ptrValue->GetMin() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)4095, ptrValue->GetMax() );

    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : Integer <=> MaskedIntReg(Malue) <=> Port");
    LOG_PERFORMANCE("                <=> MaskedIntReg(Max)   <=> Port");
    LOG_PERFORMANCE("                <=> MaskedIntReg(Min)   <=> Port");
    LOG_PERFORMANCE("                <=> MaskedIntReg(Inq)    <=> Port");

    // Test SetValue with caching
    ptrValue->SetValue(42); // fill cache
    TEST_BEGIN(1000)
        ptrValue->SetValue(42);
    TEST_END( Integer::SetValue (with caching) )

    // Test SetValue without caching
    ptrPort->InvalidateNode();
    TEST_BEGIN(1000)
        ptrValue->SetValue(42);
        StopWatch.PauseBegin();
        ptrPort->InvalidateNode();
        StopWatch.PauseEnd();
    TEST_END( Integer::SetValue (without caching) )

    // Test SetValue with caching and without verify
    ptrValue->SetValue(42); // fill cache
    TEST_BEGIN(1000)
        ptrValue->SetValue(42, false);
    TEST_END( Integer::SetValue (with caching, without verify) )

    // Test GetValue with caching
    int64_t Result = ptrValue->GetValue(); // fill cache
    TEST_BEGIN(1000)
        int64_t Result = ptrValue->GetValue();
    TEST_END( Integer::GetValue (with caching) )

    // Test GetValue without caching
    ptrPort->InvalidateNode();
    TEST_BEGIN(1000)
        int64_t Result = ptrValue->GetValue();
        StopWatch.PauseBegin();
        ptrPort->InvalidateNode();
        StopWatch.PauseEnd();
    TEST_END( Integer::GetValue (without caching) )

}


void PerformanceTestSuite::TestStopWatch()
{
    // test performance measurement
    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Timer resolution = %f us per tick", 1000000.0 * CStopWatch::SecondsPerTick());

    TEST_BEGIN(1)
    TEST_END( Empty Test (1 run) )

    TEST_BEGIN(1000)
    TEST_END( Empty Test (1000 runs) )

    TEST_BEGIN(1)
        StopWatch.PauseBegin();
        for(int k=0; k<100; k++)
            ;
        StopWatch.PauseEnd();
    TEST_END( Empty Test with pause (1 run) )

    CTestPort Port;
    Port.CreateEntry(0x0000, 0x12345678UL, RW);

    // Dummy port test
    uint8_t Buffer[4];
    TEST_BEGIN(1000)
        Port.Read(&Buffer, 0x0000, 4);
    TEST_END( Dummy Port read )

    TEST_BEGIN(1000)
        Port.Write(&Buffer, 0x0000, 4);
    TEST_END( Dummy Port write )

}


void PerformanceTestSuite::TestIntReg()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<IntReg Name=\"IntReg\">                \n"
    "    <Address>0x0010</Address>            \n"
    "    <Length>4</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<Port Name=\"Port\"/>                    \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x0010, (int32_t)117, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CNodePtr ptrPort = Camera._GetNode("Port");
    CPPUNIT_ASSERT( ptrPort.IsValid() );

    CIntegerPtr ptrIntReg = Camera._GetNode("IntReg");
    CPPUNIT_ASSERT( ptrIntReg.IsValid() );

    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : IntReg <=> Port");

    // Test SetValue with caching
    ptrIntReg->SetValue(42); // fill cache
    TEST_BEGIN(1000)
        ptrIntReg->SetValue(42);
    TEST_END( IntReg::SetValue (with caching) )

    // Test SetValue without caching
    ptrPort->InvalidateNode();
    TEST_BEGIN(1000)
        ptrIntReg->SetValue(42);
        StopWatch.PauseBegin();
        ptrPort->InvalidateNode();
        StopWatch.PauseEnd();
    TEST_END( IntReg::SetValue (without caching) )

    // Test GetValue with caching
    int64_t Result = ptrIntReg->GetValue(); // fill cache
    TEST_BEGIN(1000)
        int64_t Result = ptrIntReg->GetValue();
    TEST_END( IntReg::GetValue (with caching) )

    // Test GetValue without caching
    ptrPort->InvalidateNode();
    TEST_BEGIN(1000)
        int64_t Result = ptrIntReg->GetValue();
        StopWatch.PauseBegin();
        ptrPort->InvalidateNode();
        StopWatch.PauseEnd();
    TEST_END( IntReg::GetValue (without caching) )

}

void PerformanceTestSuite::TestXMLLoading()
{

    CLog::GetLogger("CppUnit.Performance").info( "PerformanceTestSuite::TestXMLLoading" );

    CNodeMapRef::_ClearXMLCache();

    //! Gets a list of files or directories matching a given FileTemplate
    gcstring_vector FileNames;
    gcstring FileFolder( XMLPATH "Test/Performance/" );
    ReplaceEnvironmentVariables(FileFolder);

    GetFiles( FileFolder + "*.xml", FileNames );

    for( gcstring_vector::iterator itFileName=FileNames.begin(); itFileName!=FileNames.end(); itFileName++ )
    {
        gcstring FileName(FileFolder + *itFileName);
        CLog::GetLogger("CppUnit.Performance").info( "Loading file '%s'", FileName.c_str() );
        CLog::GetLogger("CppUnit.Performance").info( " " );

        CStopWatch StopWatch;
        StopWatch.Start();
        {
            CNodeMapRef Camera;
            Camera._LoadXMLFromFile(FileName);
        }
        double ElapsedTime1 = StopWatch.Stop();
        CLog::GetLogger("CppUnit.Performance").info( "First time : t = %.1f s", ElapsedTime1 );

        StopWatch.Start();
        {
            CNodeMapRef Camera;
            Camera._LoadXMLFromFile(FileName);
        }
        double ElapsedTime2 = StopWatch.Stop();
        CLog::GetLogger("CppUnit.Performance").info( "Second time (from cache) : t = %.1f s", ElapsedTime2 );
        CLog::GetLogger("CppUnit.Performance").info( "With caching the XML loades %.2f times faster", ElapsedTime1/ElapsedTime2 );
        CLog::GetLogger("CppUnit.Performance").info( " " );
    }
}
