//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "PersistenceTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include <strstream>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( PersistenceTestSuite );

void PersistenceTestSuite::TestSimple()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                               \n"
    "<Category Name=\"Root\">                       \n"
    " <pFeature>IntegerValue</pFeature>             \n"
    " <pFeature>IntegerValueNotStreamable</pFeature>\n"
    " <pFeature>FloatValue</pFeature>               \n"
    " <pFeature>EnumerationValue</pFeature>         \n"
    "</Category>                                    \n"
    "                                               \n"
    "<Integer Name=\"IntegerValue\">                \n"
    "    <Streamable>Yes</Streamable>               \n"
    "    <Value>17</Value>                          \n"
    "</Integer>                                     \n"
    "                                               \n"
    "<Integer Name=\"IntegerValueNotStreamable\">   \n"
    "    <Streamable>No</Streamable>                \n"
    "    <Value>17</Value>                          \n"
    "</Integer>                                     \n"
    "                                               \n"
    "<Float Name=\"FloatValue\">                    \n"
    "    <Streamable>Yes</Streamable>               \n"
    "    <Value>17.0</Value>                        \n"
    "</Float>                                       \n"
    "                                               \n"
    "<Enumeration Name=\"EnumerationValue\">        \n"
    "    <Streamable>Yes</Streamable>               \n"
    "    <EnumEntry Name=\"EnumValue1\">            \n"
    "        <Value>10</Value>                      \n"
    "   </EnumEntry>                                \n"
    "    <EnumEntry Name=\"EnumValue2\">            \n"
    "        <Value>20</Value>                      \n"
    "   </EnumEntry>                                \n"
    "    <pValue>EnumInt</pValue>                   \n"
    "</Enumeration>                                 \n"
    "                                               \n"
    "<Integer Name=\"EnumInt\">                     \n"
    "    <Value>10</Value>                          \n"
    "</Integer>                                     \n"
    "                                               \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrIntegerValue = Camera._GetNode("IntegerValue");
    CPPUNIT_ASSERT( ptrIntegerValue.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)17, ptrIntegerValue->GetValue() );

    CIntegerPtr ptrIntegerValueNotStreamable = Camera._GetNode("IntegerValueNotStreamable");
    CPPUNIT_ASSERT( ptrIntegerValueNotStreamable.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)17, ptrIntegerValueNotStreamable->GetValue() );

    CFloatPtr ptrFloatValue = Camera._GetNode("FloatValue");
    CPPUNIT_ASSERT( ptrFloatValue.IsValid() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 17.0, ptrFloatValue->GetValue(), 0.001 );

    CEnumerationPtr ptrEnumerationValue = Camera._GetNode("EnumerationValue");
    CPPUNIT_ASSERT( ptrEnumerationValue.IsValid() );
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue1"), ptrEnumerationValue->ToString() );

    // persist the values to staorage
    CFeatureBag FeatureBag;
    FeatureBag.StoreToBag(Camera._Ptr);

    // stream content of feature bag to string
    strstream File;
    File << FeatureBag;

    // print the string's content to stdout (don't forget to reset the filte pointer to the begin)
    cout << File.rdbuf();
    File.seekg( ios::beg );

    // load string into another(!) bag
    CFeatureBag FeatureBag2;
    File >> FeatureBag2;

    // change the values
    *ptrIntegerValue = 13;
    CPPUNIT_ASSERT_EQUAL( (int64_t)13, ptrIntegerValue->GetValue() );

    *ptrIntegerValueNotStreamable = 13;
    CPPUNIT_ASSERT_EQUAL( (int64_t)13, ptrIntegerValueNotStreamable->GetValue() );

    *ptrFloatValue = 13.0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 13.0, ptrFloatValue->GetValue(), 0.001 );

    *ptrEnumerationValue = "EnumValue2";
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue2"), ptrEnumerationValue->ToString() );

    // load the values from FeatureBag and verify
    FeatureBag2.LoadFromBag(Camera._Ptr);
    CPPUNIT_ASSERT_EQUAL( (int64_t)17, ptrIntegerValue->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)13, ptrIntegerValueNotStreamable->GetValue() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 17.0, ptrFloatValue->GetValue(), 0.001 );
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue1"), ptrEnumerationValue->ToString() );

}

void PersistenceTestSuite::TestSelector()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                               \n"
    "<Category Name=\"Root\">                       \n"
    "   <pFeature>Entry</pFeature>                  \n"
    "   <pFeature>Selector</pFeature>               \n"
    "</Category>                                    \n"
    "                                               \n"
    "<IntReg Name=\"Entry\">                        \n"
    "   <Streamable>Yes</Streamable>                \n"
    "   <pAddress>Selector</pAddress>               \n"
    "   <Length>4</Length>                          \n"
    "   <AccessMode>RW</AccessMode>                 \n"
    "   <pPort>MyPort</pPort>                       \n"
    "   <Sign>Signed</Sign>                         \n"
    "   <Endianess>LittleEndian</Endianess>         \n"
    "</IntReg>                                      \n"
    "                                               \n"
    "<Integer Name=\"Selector\">                    \n"
// BUG : this should work also with Streamable=No but it does not :-(
    "   <Streamable>Yes</Streamable>                \n"
    "   <Value>48</Value>                           \n"
    "   <Min>40</Min>                               \n"
    "   <Max>49</Max>                               \n"
    "   <pSelected>Entry</pSelected>                \n"
    "</Integer>                                     \n"
    "                                               \n"
    "<Port Name=\"MyPort\"/>                        \n"
    "                                               \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    for (int i=40; i<50; ++i)
        Port.CreateEntry(i, i-40UL, RW);

    // connect the node map to the port
    bool ret = Camera._Connect(&Port, "MyPort");
    CPPUNIT_ASSERT(ret);

    // verify data
    CIntegerPtr ptrEntry = Camera._GetNode("Entry");
    CPPUNIT_ASSERT( ptrEntry.IsValid() );
    CIntegerPtr ptrSelector = Camera._GetNode("Selector");
    CPPUNIT_ASSERT( ptrSelector.IsValid() );
    for (int64_t i=ptrSelector->GetMin(); i<=ptrSelector->GetMax(); i+=ptrSelector->GetInc())
    {
        *ptrSelector = i;
        CPPUNIT_ASSERT_EQUAL( (int64_t) (i-40), ptrEntry->GetValue() );
    }
    *ptrSelector = 45;

    // persist the values to staorage
    CFeatureBag FeatureBag;
    FeatureBag.StoreToBag(Camera._Ptr);

    // stream content of feature bag to string
    strstream File;
    File << FeatureBag;

    // print the string's content to stdout (don't forget to reset the filte pointer to the begin)
    cout << File.rdbuf();
    File.seekg( ios::beg );

    // load string into another(!) bag
    CFeatureBag FeatureBag2;
    File >> FeatureBag2;

    // change the values
    for (int64_t i=ptrSelector->GetMin(); i<=ptrSelector->GetMax(); i+=ptrSelector->GetInc())
        ptrEntry->SetValue(-1);
    *ptrSelector = 40;

    // load the values from FeatureBag and verify
    FeatureBag2.LoadFromBag(Camera._Ptr);
    CPPUNIT_ASSERT_EQUAL( (int64_t)45, ptrSelector->GetValue() );
    for (int64_t i=ptrSelector->GetMin(); i<=ptrSelector->GetMax(); i+=ptrSelector->GetInc())
    {
        *ptrSelector = i;
        CPPUNIT_ASSERT_EQUAL( (int64_t)(i-40), ptrEntry->GetValue() );
    }

}

void PersistenceTestSuite::TestAccessMode()
{
    // initialize the node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                               \n"
    "<Category Name=\"Root\">                       \n"
    " <pFeature>IntegerValueWasNI</pFeature>        \n"
    "</Category>                                    \n"
    "                                               \n"
    "<Integer Name=\"IntegerValueWasNI\">           \n"
    "    <pIsImplemented>Implemented</pIsImplemented> \n"
    "    <Streamable>Yes</Streamable>               \n"
    "    <Value>17</Value>                          \n"
    "</Integer>                                     \n"
    "                                               \n"
    "<Integer Name=\"Implemented\">                 \n"
    "    <Value>0</Value>                           \n"
    "</Integer>                                     \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrIntegerValueWasNI = Camera._GetNode("IntegerValueWasNI");
    CPPUNIT_ASSERT( ptrIntegerValueWasNI.IsValid() );
    CIntegerPtr ptrImplemented = Camera._GetNode("Implemented");
    CPPUNIT_ASSERT( ptrImplemented.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, ptrImplemented->GetValue() );

    // persist the values to storage
    CFeatureBag FeatureBag;
    FeatureBag.StoreToBag(Camera._Ptr);

    // stream content of feature bag to string
    strstream File;
    File << FeatureBag;

    // print the string's content to stdout (don't forget to reset the file pointer to the begin)
    cout << File.rdbuf();
    File.seekg( ios::beg );

    // load string into another(!) bag
    CFeatureBag FeatureBag2;
    File >> FeatureBag2;

    // now make the before-NI node implemented, its value should not be altered by the persistence
    ptrImplemented->SetValue (1);
    CPPUNIT_ASSERT_EQUAL( (int64_t)17, ptrIntegerValueWasNI->GetValue() );
    ptrIntegerValueWasNI->SetValue (23);

    // load the values from FeatureBag and verify
    FeatureBag2.LoadFromBag(Camera._Ptr);

    CPPUNIT_ASSERT_EQUAL( (int64_t)23, ptrIntegerValueWasNI->GetValue() );
}


void PersistenceTestSuite::TestMisc()
{
    // EatComments should work also if at eof
    strstream File;
    CPPUNIT_ASSERT_NO_THROW (EatComments(File));
    CPPUNIT_ASSERT_NO_THROW (EatComments(File));

    // On the other hand, FeatureBag's stream input should fail
    CFeatureBag FeatureBag;
    CPPUNIT_ASSERT_THROW_EX (File >> FeatureBag, GenICam::RuntimeException);

    // now send the (empty) bag to a stream, skip the magic line, and observe the exception
    strstream File2;
    File2 << FeatureBag;
    File2.seekg( ios::beg );
    File2.ignore(1024, '\n');
    CFeatureBag FeatureBag2;
    CPPUNIT_ASSERT_THROW_EX (File2 >> FeatureBag2, GenICam::RuntimeException);

    // try to use a node map without "Root"
    gcstring CameraDescriptionFile3 = CDFHeader + CDFFooter;
    CNodeMapRef Camera3;
    Camera3._LoadXMLFromString(CameraDescriptionFile3);
    CFeatureBag FeatureBag3;
    CPPUNIT_ASSERT_THROW_EX (FeatureBag3.StoreToBag(Camera3._Ptr), GenICam::RuntimeException);

    // try to load from an invalid pointer
    CFeatureBag FeatureBag4;
    CPPUNIT_ASSERT_THROW_EX (FeatureBag4.LoadFromBag(NULL), GenICam::InvalidArgumentException);

    // if a node name is not found, the LoadFromBag() should fail
    gcstring CameraDescriptionFile5 =
    CDFHeader +
    "<Category Name=\"Root\">                       \n"
    " <pFeature>IntegerValue1</pFeature>            \n"
    "</Category>                                    \n"
    "                                               \n"
    "<Integer Name=\"IntegerValue1\">               \n"
    "    <Streamable>Yes</Streamable>               \n"
    "    <Value>17</Value>                          \n"
    "</Integer>                                     \n"
    + CDFFooter;
    CNodeMapRef Camera5;
    Camera5._LoadXMLFromString(CameraDescriptionFile5);

    gcstring CameraDescriptionFile6 =
    CDFHeader +
    "<Category Name=\"Root\">                       \n"
    " <pFeature>IntegerValue2</pFeature>            \n"
    "</Category>                                    \n"
    "                                               \n"
    "<Integer Name=\"IntegerValue2\">               \n"
    "    <Streamable>Yes</Streamable>               \n"
    "    <Value>17</Value>                          \n"
    "</Integer>                                     \n"
    + CDFFooter;
    CNodeMapRef Camera6;
    Camera6._LoadXMLFromString(CameraDescriptionFile6);

    CFeatureBag FeatureBag5;
    FeatureBag5.StoreToBag(Camera5._Ptr);
    gcstring_vector ErrorList;
    FeatureBag5.LoadFromBag(Camera6._Ptr, true, &ErrorList);
    gcstring_vector::iterator pError;
    for( pError = ErrorList.begin(); pError != ErrorList.end(); pError++ )
    {
        cout << (*pError).c_str() << "\n";
    }
    CPPUNIT_ASSERT_EQUAL( (size_t)1, ErrorList.size() );
}


void PersistenceTestSuite::FailBit_Mantis356()
{
    // create and initialize the node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                               \n"
    "<Category Name=\"Root\">                       \n"
    " <pFeature>IntegerValue</pFeature>             \n"
    "</Category>                                    \n"
    "                                               \n"
    "<Integer Name=\"IntegerValue\">                \n"
    "    <Streamable>Yes</Streamable>               \n"
    "    <Value>4717</Value>                        \n"
    "</Integer>                                     \n"
    "                                               \n"
    + CDFFooter;

    strstream File;
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    {   // Lokal Scope just to create locale String Stream containing the node map
        CFeatureBag FeatureBag;

        // stream content of feature bag to string
        // persist the values to storage
        FeatureBag.StoreToBag(Camera._Ptr);

        File << FeatureBag;
        CPPUNIT_ASSERT (!File.fail());

    }   // Destroys FeatureBag

     // load string into another(!) bag
    CFeatureBag FeatureBag;
    File >> FeatureBag;             // This causes Mantia #356, because fail bit ist set after correct read operation!
    CPPUNIT_ASSERT (!File.fail());
}



void PersistenceTestSuite::LongFeatures_Mantis356()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                               \n"
    "<Category Name=\"Root\">                       \n"
    " <pFeature>Register</pFeature>                 \n"
    "</Category>                                    \n"
    "                                               \n"
    "<Register Name=\"Register\">                   \n"
    "   <Streamable>Yes</Streamable>                \n"
    "   <Address>0xff00</Address>                   \n"
    "   <Length>1024</Length>                       \n"
    "   <AccessMode>RW</AccessMode>                 \n"
    "   <pPort>Port</pPort>                         \n"
    "</Register>                                    \n"
    "                                               \n"
    "<Port Name=\"Port\"/>                          \n"
    "                                               \n"

    + CDFFooter;


    strstream File;

    {   // Local scope just to create locale String Stream containing the node map
        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        // create and initialize a test memory
        char TestData[1024];
        memset(TestData, 0x55, 1024);

        // create and initialize a test port
        CTestPort Port;
        Port.CreateEntry(0xff00, 1024, (void*) TestData ,RW);

        // conntect the node map to the port
        Camera._Connect(&Port, "Port");
        CFeatureBag FeatureBag;

        // stream content of feature bag to string
        // persist the values to storage
        FeatureBag.StoreToBag(Camera._Ptr);

        File << FeatureBag;

    }   // Destroys Camera include Nodemap, FeatureBag, and Test memory


    ///////////////////////////////////////
    //
    // create and initialize a test memory
    char TestData[1024];
    memset(TestData, 0xAA, 1024);

    // load string into another(!) bag
    CFeatureBag FeatureBag;
    File >> FeatureBag;                             // This causes Mantia #356, because Register Value is 1024 Bytes long and can not be read!

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");
    Port.CreateEntry(0xff00, 1024, (void*) TestData ,RW);

    // Restore camera node map from Stream!!!!!
    FeatureBag.LoadFromBag (Camera._Ptr);

    // Get out register value(s)
    GenApi::EAccessMode AccessMode;
    Port.LookupEntry(0xff00, 1024, TestData, AccessMode);

    // Check if read from stream to camera was successful
    CPPUNIT_ASSERT_EQUAL((char)0x55, TestData[1023]);
}


void PersistenceTestSuite::TestEnumerationSelector()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                               \n"
    "<Category Name=\"Root\">                       \n"
    "   <pFeature>Entry</pFeature>                  \n"
    "   <pFeature>Selector</pFeature>               \n"
    "</Category>                                    \n"
    "                                               \n"
    "<Integer Name=\"Entry\">                       \n"
    "   <Streamable>Yes</Streamable>                \n"
    "   <Value>192</Value>                          \n"
    "   <Min>192</Min>                              \n"
    "   <Max>1023</Max>                             \n"
    "   <Representation>Linear</Representation>     \n"
    "</Integer>                                     \n"
    "                                               \n"
    "<Enumeration Name=\"Selector\">                \n"
    "   <EnumEntry Name=\"Entry0\" NameSpace=\"Standard\">      \n"
    "       <Value>0</Value>                                    \n"
    "   </EnumEntry>                                            \n"
    "                                                           \n"
    "   <EnumEntry Name=\"Entry1\" NameSpace=\"Standard\">      \n"
    "       <Value>1</Value>                                    \n"
    "   </EnumEntry>                                            \n"
    "                                                           \n"
    "   <EnumEntry Name=\"Entry2\" NameSpace=\"Standard\">      \n"
    "       <Value>2</Value>                                    \n"
    "   </EnumEntry>                                            \n"
    "                                                           \n"
    "                                                           \n"
    "   <Value>1</Value>                                        \n"
    "   <pSelected>Entry</pSelected>                            \n"
    "</Enumeration>                                             \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // persist the values to feature bag
    CFeatureBag FeatureBag;
    FeatureBag.StoreToBag(Camera._Ptr);

    // stream content of feature bag to string
    strstream File;
    File << FeatureBag;

    gcstring csRead = File.str();

    // Check if all selector values were streamed out
    CPPUNIT_ASSERT (csRead.find("Entry0") != gcstring::npos);
    CPPUNIT_ASSERT (csRead.find("Entry1") != gcstring::npos);
    CPPUNIT_ASSERT (csRead.find("Entry2") != gcstring::npos);
}
