/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "PointerTestSuite.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( PointerTestSuite );

void PointerTestSuite::TestPointer()
   {
    // create and initialize a test port
    CTestPort Port;
    uint32_t MyRegister = 42;
    Port.CreateEntry(0x00ff, MyRegister, RW);

    // create a camera description file
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <Port Name=\"MyPort\" >                \n"
    "    </Port>                                    \n"
    "    <Port Name=\"AnotherPort\" >       \n"
    "    </Port>                                    \n"
    + CDFFooter;

    // create and connect a camera object
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    Camera._Connect(&Port, "MyPort");

    CPortPtr ptrPort = Camera._GetNode("MyPort");
   CPortPtr equivPort = Camera._GetNode("MyPort");
   CPortPtr diffPort = Camera._GetNode("AnotherPort");

   //Test pointer validity:
    CPPUNIT_ASSERT_MESSAGE( "The pointer was unexpectably found to be NULL during a cast to bool", (bool)ptrPort );
   CPPUNIT_ASSERT_MESSAGE( "The pointer was found to be NULL unexpectedly", ptrPort.IsValid() );
   CPPUNIT_ASSERT_MESSAGE( "Two pointers created to point to the same node do not match", ptrPort == &(*equivPort) );
   CPPUNIT_ASSERT_MESSAGE( "Two pointers created to point to the same node do not match", ptrPort == (IPort*)equivPort );
   CPPUNIT_ASSERT_MESSAGE( "Two pointers originally assigned to different nodes are pointing to the same node", !(ptrPort == (IPort*)diffPort) );

   //Set one of the pointers to NULL.
   equivPort = NULL;

    //write some values, then read them back, tests dereferencing
    uint32_t MyNewRegister = 99;
   uint32_t Buffer = 0;
    CPPUNIT_ASSERT_NO_THROW( ptrPort->Write(&MyNewRegister, 0x00ff, sizeof(Buffer)) );
    CPPUNIT_ASSERT_NO_THROW( ptrPort->Read(&Buffer, 0x00ff, sizeof(Buffer)) );
    CPPUNIT_ASSERT_EQUAL(MyNewRegister, Buffer);

   MyNewRegister = 42;
   Buffer = 25;
   CPPUNIT_ASSERT_NO_THROW( ptrPort->Write(&MyNewRegister, 0x00ff, sizeof(Buffer)) );
    CPPUNIT_ASSERT_NO_THROW( ptrPort->Read(&Buffer, 0x00ff, sizeof(Buffer)) );
   CPPUNIT_ASSERT_NO_THROW( ptrPort());
    CPPUNIT_ASSERT_EQUAL(MyNewRegister, Buffer);

   //Test null pointer exception generation:
   CPPUNIT_ASSERT_THROW( (IPort*)equivPort, GenICam::GenericException );
   CPPUNIT_ASSERT_THROW( equivPort(), GenICam::GenericException );
   CPPUNIT_ASSERT_THROW( *equivPort, GenICam::GenericException );
   CPPUNIT_ASSERT_THROW( (IPort*)equivPort, GenICam::GenericException );
   CPPUNIT_ASSERT_THROW( ptrPort == 42, GenICam::GenericException );
   CPPUNIT_ASSERT_NO_THROW( ptrPort == 0 );
   CPPUNIT_ASSERT_THROW( equivPort->Read(&Buffer, 0x00ff, sizeof(Buffer)), GenICam::GenericException );
   }
