//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------

/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "PollTest.h"
#include "CallbackHelper.h"
#include "GenApi/GenApi.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( PollTestSuite );

void IntegerTest()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "<Integer Name=\"Value\">                        \n"
    "   <pValue>Register</pValue>                    \n"
    "</Integer>                                      \n"
    "                                                \n"
    "                                                \n"
    "<IntReg Name=\"Register\">                    \n"
    "    <Address>0x0f00</Address>                    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <PollingTime>1000</PollingTime>                \n"
    "    <Sign>Unsigned</Sign>                    \n"
     "   <Endianess>LittleEndian</Endianess>        \n"
    "</IntReg>                                    \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CNodePtr ptrNode = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)1000, ptrNode->GetPollingTime() );

    CNodePtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT( ptrNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)-1, ptrValue->GetPollingTime() );

    CallbackObject CallBackTarget;
    Register( ptrNode, CallBackTarget, &CallbackObject::Callback );
    Register( ptrValue, CallBackTarget, &CallbackObject::Callback );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0, CallBackTarget.Count() );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallBackTarget.Count() );


}

void PollTestSuite::TestPolling()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                \n"
    "<Register Name=\"Register\">                    \n"
    "    <Address>0x0f00</Address>                    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <PollingTime>1000</PollingTime>                \n"
    "</Register>                                    \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrNode = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)1000, ptrNode->GetPollingTime() );

    CallbackObject CallBackTarget;
    Register( ptrNode, CallBackTarget, &CallbackObject::Callback );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0, CallBackTarget.Count() );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallBackTarget.Count() );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallBackTarget.Count() );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallBackTarget.Count() );

    IntegerTest();
}

