//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#  include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#else
#  include <sys/wait.h>
#endif
#include <stdlib.h>

#include "GenApiTest.h"
#include "PreprocessorTest.h"
#include "GenApi/GenApi.h"
#include "GenApi/GenApiVersion.h"
#include "GenApi/impl/DllManager.h"
#include "Base/GCLinkage.h"
#include "Base/GCTypes.h"


using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( PreProcessorTestSuite );


void PreProcessorTestSuite::TestSyntaxCheck()
{
    CNodeMapRef Camera;
    gcstring XMLFileName;
    gcstring OutputFileName;
    gcstring StyleSheetFileName;

    // make sure the test is reproducible
    CNodeMapRef::_ClearXMLCache();

    // Load a valid XML file and check sytax
    XMLFileName = XMLPATH "Test/TestCamera_v1_0.xml";
    Camera._PreprocessXMLFromFile( XMLFileName, "", "" );

    // Load an ivalid XML file and check sytax
    XMLFileName = XMLPATH "Test/InvalidXMLFile.xml";
    CPPUNIT_ASSERT_THROW( Camera._PreprocessXMLFromFile( XMLFileName, "", "" ), GenICam::RuntimeException);

    // Load a valid XML file and write it to file
    XMLFileName = XMLPATH "Test/TestCamera_v1_0.xml";
    OutputFileName = XMLPATH "Temp/TestSyntaxCheck__TestCamera_v1_0__Preprocessed.xml";
    Camera._PreprocessXMLFromFile( XMLFileName, "", OutputFileName );

    // Load a valid XML file, apply a stylesheet, and write it to file
    XMLFileName = XMLPATH "Test/TestCamera_v1_0.xml";
    StyleSheetFileName = XMLPATH "Test/TestTransform.xsl";
    OutputFileName = XMLPATH "Temp/TestSyntaxCheck__TestTransform.txt";
    Camera._PreprocessXMLFromFile( XMLFileName, StyleSheetFileName, OutputFileName );

    // Load a valid XML file, apply a stylesheet, and write it to file
    XMLFileName = XMLPATH "Test/TestCamera_v1_0.xml";
    StyleSheetFileName = XMLPATH "GenApi_Ptr_h.xsl";
    OutputFileName = XMLPATH "Temp/TestSyntaxCheck__TestCamera_v1_0__Ptr_h.txt";
    Camera._PreprocessXMLFromFile( XMLFileName, StyleSheetFileName, OutputFileName );
}

int InvokePreProcessor( const char* pXMLFileName = NULL, const char *pStyleSheetFileName = NULL, const char* pOutputFileName = NULL,
                        const bool Help = false, const bool Verbose = false  )
{
    int status;

#ifdef _MSC_VER
    gcstring Command = EXE_NAME("GenApiPreProcessor");
#else
    gcstring Command = "GenApiPreProcessor";
#endif

    if( pXMLFileName )
        Command += " -x \"" + gcstring(pXMLFileName) + "\"";
    if( pStyleSheetFileName )
        Command += " -s \"" + gcstring(pStyleSheetFileName) + "\"";
    if( pOutputFileName )
        Command += " -o \"" + gcstring(pOutputFileName) + "\"";
    if( Help )
        Command += " -h";
    if( Verbose )
        Command += " -v";

    ReplaceEnvironmentVariables(Command);
    printf("Invoking command '%s'\n", Command.c_str() );

    status = system( Command.c_str() );
#ifdef _MSC_VER
    return status;
#else
    /* system() modifies the exit status. Undo this modifications. */
    return (int8_t) WEXITSTATUS(status);
#endif
}


void PreProcessorTestSuite::TestGenApiPreprocessor()
{
    gcstring XMLFileName;
    gcstring OutputFileName;
    gcstring StyleSheetFileName;

#ifdef  _WIN64

    // The Win64 test installation does not contain the GenApiPreProcessor
    // Reason: since Win64 is always(!) cross compiled the Win32 version of the
    // GenApiPreProcessor would have to be installed even with Win64 which is too complicated wiht CMake
    return;
#endif

    // make sure the test is reproducible
    CNodeMapRef::_ClearXMLCache();

    // Show help text
    CPPUNIT_ASSERT_EQUAL( 0, InvokePreProcessor( NULL, NULL, NULL, true) );

    // Load a valid XML file and check
    CPPUNIT_ASSERT_EQUAL( 0, InvokePreProcessor( XMLPATH "Test/TestCamera_v1_0.xml" ) );

    // Load a valid XML file and write it to file
    CPPUNIT_ASSERT_EQUAL( 0, InvokePreProcessor( XMLPATH "Test/TestCamera_v1_0.xml",
                                                 NULL,
                                                 XMLPATH "Temp/TestGenApiPreprocessor__TestCamera_v1_0__Preprocessed.xml") );

    // Load a valid XML file, apply a stylesheet, and write it to file
    CPPUNIT_ASSERT_EQUAL( 0, InvokePreProcessor( XMLPATH "Test/TestCamera_v1_0.xml",
                                                 XMLPATH "Test/TestTransform.xsl",
                                                 XMLPATH "Temp/TestGenApiPreprocessor__TestTransform.txt") );

    // Load a valid XML file, apply a stylesheet, and write it to file
    CPPUNIT_ASSERT_EQUAL( 0, InvokePreProcessor( XMLPATH "Test/TestCamera_v1_0.xml",
                                                 XMLPATH "GenApi_Ptr_h.xsl",
                                                 XMLPATH "Temp/TestGenApiPreprocessor__TestCamera_v1_0__Ptr_h.txt") );

    // Do it verbose
    CPPUNIT_ASSERT_EQUAL( 0, InvokePreProcessor( XMLPATH "Test/TestCamera_v1_0.xml",
                                                 XMLPATH "Test/TestTransform.xsl",
                                                 XMLPATH "Temp/TestGenApiPreprocessor__TestTransform.txt", false, true) );


    // Dont do this :-)
    CPPUNIT_ASSERT_EQUAL( -1, InvokePreProcessor() );
}
