//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "PropertiesTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( PropertiesTestSuite );

void PropertiesTestSuite::TestPropertyAccess()
{

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Category Name=\"Root\">                \n"
    "    <pFeature>Register</pFeature>                \n"
    "</Category>                                    \n"
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <pIsImplemented>Implemented</pIsImplemented>        \n"
    "    <pAddress>BaseAddress</pAddress>            \n"
    "    <pAddress>AddressOffset</pAddress>            \n"
    "    <Address>0x0100</Address>                    \n"
    "    <Address>0x0e00</Address>                    \n"
    "    <IntSwissKnife Name=\"Dummy\">                                \n"
    "        <pVariable Name=\"I\">Index</pVariable>    \n"
    "        <Formula>0x0080 * I</Formula>            \n"
    "    </IntSwissKnife>                            \n"
    "    <pIndex Offset=\"0x0080\">Index</pIndex>    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <pInvalidator>Invalidated</pInvalidator>        \n"
    "    <Sign>Unsigned</Sign>        \n"
    "    <Endianess>LittleEndian</Endianess>        \n"
    "    <pSelected>Changer</pSelected>                \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"BaseAddress\">                    \n"
    "    <Value>0x2000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"AddressOffset\">                \n"
    "    <Value>0xd000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Index\">                        \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Changer\">                        \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Selector\">                    \n"
    "    <Value>0</Value>                            \n"
    "    <pSelected>Register</pSelected>                \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Implemented\">                        \n"
    "    <Value>1</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Invalidated\">                        \n"
    "    <Value>1</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera("TestCamera");
    Camera._LoadXMLFromString(CameraDescriptionFile);

    printf("Dumping PropertyNames:\n");
    NodeList_t Nodes;
    Camera._GetNodes(Nodes);
    for( NodeList_t::iterator itNode=Nodes.begin(); itNode!=Nodes.end(); itNode++ )
    {
        cout << "Node '" << (*itNode)->GetName().c_str() << "'\n";

        GenICam::gcstring_vector PropertyNames;
        (*itNode)->GetPropertyNames(PropertyNames);

        gcstring ValueStr;
        gcstring AttributeStr;
        GenICam::gcstring_vector::iterator ptr;
        for(ptr=PropertyNames.begin(); ptr!=PropertyNames.end(); ptr++)
        {
            bool res = (*itNode)->GetProperty((*ptr), ValueStr, AttributeStr);
            if(!res)
                printf("\t  Property '%s' is not available\n", (*ptr).c_str());
            else if(AttributeStr.empty())
                printf("\t  Property '%s' = '%s'\n", (*ptr).c_str(), ValueStr.c_str());
            else
                printf("\t  Property '%s' = '%s' [%s]\n", (*ptr).c_str(), ValueStr.c_str(), AttributeStr.c_str());
        }
    }

    CNodePtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    GenICam::gcstring_vector PropertyNames;
    ptrRegister->GetPropertyNames(PropertyNames);

    int i=0;
    CPPUNIT_ASSERT_EQUAL( gcstring("Name"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("NameSpace"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("DeviceName"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("NodeType"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pIsImplemented"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pAddress"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("Address"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pIndex"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("Length"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("AccessMode"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pPort"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pInvalidator"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("Sign"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("Endianess"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pSelected"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pSelecting"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pTerminal"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pDependent"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("IsFeature"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( (size_t)i, PropertyNames.size() );

    gcstring ValueStr;
    gcstring AttributeStr;
    CPPUNIT_ASSERT( ptrRegister->GetProperty("NodeType", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("IntReg"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("Name", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Register"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("NameSpace", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Custom"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("DeviceName", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("TestCamera"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("pIsImplemented", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Implemented"), ValueStr);

    CPPUNIT_ASSERT( ptrRegister->GetProperty("pAddress", ValueStr, AttributeStr) );
    // the number is generated by the pre-processor and subject to change
    CPPUNIT_ASSERT_EQUAL( gcstring("BaseAddress\tAddressOffset\t" /*N0117FA60.01285508*/), ValueStr.substr(0, 26) );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("Address", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("256\t3584"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("pIndex", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Index"), ValueStr );
    // the number is generated by the pre-processor and subject to change
    CPPUNIT_ASSERT_EQUAL( gcstring("_Register_IndexOffset_" /*N0154B6E0N015CF978*/ ), AttributeStr.substr(0, 22) );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("Length", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("4"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("AccessMode", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("RW"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("pPort", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Port"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("pInvalidator", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Invalidated"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("Sign", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Unsigned"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("Endianess", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("LittleEndian"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("pSelected", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Changer"), ValueStr );

    CPPUNIT_ASSERT( ptrRegister->GetProperty("pSelecting", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Selector"), ValueStr );
}

void PropertiesTestSuite::TestSwissKnifePropertyAccess()
{

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntSwissKnife Name=\"Result\">                \n"
    "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
    "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
    "    <Formula> X * Y + 12 </Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueX\">                        \n"
    "    <Value>3</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueY\">                        \n"
    "    <Value>10</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrResult = Camera._GetNode("Result");
    CPPUNIT_ASSERT( ptrResult.IsValid() );

    GenICam::gcstring_vector PropertyNames;
    ptrResult->GetPropertyNames(PropertyNames);

    printf("Dumping PropertyNames:\n");
    gcstring ValueStr;
    gcstring AttributeStr;
    GenICam::gcstring_vector::iterator ptr;
    for(ptr=PropertyNames.begin(); ptr!=PropertyNames.end(); ptr++)
    {
        bool res = ptrResult->GetProperty((*ptr), ValueStr, AttributeStr);
        if(!res)
            printf("\tProperty '%s' is not available\n", (*ptr).c_str());
        else if(AttributeStr.empty())
            printf("\tProperty '%s' = '%s'\n", (*ptr).c_str(), ValueStr.c_str());
        else
            printf("\tProperty '%s' = '%s' [%s]\n", (*ptr).c_str(), ValueStr.c_str(), AttributeStr.c_str());
    }

    int i=0;
    CPPUNIT_ASSERT_EQUAL( gcstring("Name"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("NameSpace"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("DeviceName"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("NodeType"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("pVariable"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("Formula"), PropertyNames[i++] );
    CPPUNIT_ASSERT_EQUAL( (size_t)i, PropertyNames.size() );

    CPPUNIT_ASSERT( ptrResult->GetProperty("NodeType", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("IntSwissKnife"), ValueStr );

    CPPUNIT_ASSERT( ptrResult->GetProperty("Name", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Result"), ValueStr );

    CPPUNIT_ASSERT( ptrResult->GetProperty("NameSpace", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Custom"), ValueStr );

    CPPUNIT_ASSERT( ptrResult->GetProperty("DeviceName", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("Device"), ValueStr );

    CPPUNIT_ASSERT( ptrResult->GetProperty("pVariable", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("ValueX\tValueY"), ValueStr );
    CPPUNIT_ASSERT_EQUAL( gcstring("X\tY"), AttributeStr );

    CPPUNIT_ASSERT( ptrResult->GetProperty("Formula", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring(" X * Y + 12 "), ValueStr );
}

