//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApi
//  Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
\brief    Definition of CRegisterEntry and CRegisterMap
*/

#ifndef GENAPI_REGISTERMAP_H
#define GENAPI_REGISTERMAP_H

#include "GenApi/Types.h"
#include <string.h>

struct CValueCacheImpl;

/**
\brief Describes a dummy register
*/
class CRegisterEntry
{
public:
    //! address of the data
    int64_t m_Address;

    //! data size in bytes
    int64_t m_DataSize;

    //! Data
    void *m_pData;

    //! Acess mode of the data
    GenApi::EAccessMode m_AccessMode;

    //! Constructor
    CRegisterEntry() :
        m_Address(0),
        m_DataSize(0),
        m_AccessMode(GenApi::NI),
        m_pData(NULL)
    {}

    //! Destructor
    ~CRegisterEntry()
    {
        delete[] reinterpret_cast<uint8_t*>(m_pData);
        m_pData = NULL;
    }

    //! Initializing Constructor
    CRegisterEntry( int64_t Address, int64_t DataSize, GenApi::EAccessMode AccessMode, const void *pData ) :
        m_Address(Address),
        m_DataSize(DataSize),
        m_AccessMode(AccessMode),
        m_pData(memcpy( new uint8_t[(size_t)DataSize], pData, (size_t)DataSize))
    {}

    //! Copy Constructor
    CRegisterEntry( const CRegisterEntry& v ) :
        m_Address(v.m_Address),
        m_DataSize(v.m_DataSize),
        m_AccessMode(v.m_AccessMode),
        m_pData(memcpy( new uint8_t[(size_t)v.m_DataSize], v.m_pData, (size_t)v.m_DataSize ))
    {}

    //! Assignment operator
    CRegisterEntry& operator=( CRegisterEntry& v )
    {
        if (this != &v)
        {
            void *pv = memcpy( new uint8_t[(size_t)v.m_DataSize], v.m_pData, (size_t)v.m_DataSize );
            if (! pv)
                throw RUNTIME_EXCEPTION( "Memory exception" );

            delete [] reinterpret_cast<uint8_t*>(m_pData);
            m_pData       = pv;
            m_DataSize = v.m_DataSize;
            m_AccessMode = v.m_AccessMode;
            m_Address     = v.m_Address;
        }

        return *this;
    }

};

/**
\brief Holds dummy registers
*/
class CRegisterMap
{
public:
    //! Constructor
    CRegisterMap();

    //! Destructor
    ~CRegisterMap();

    //! Look up an entry
    void LookupEntry( int64_t Address, int64_t Size, void *pValue, GenApi::EAccessMode &AccessMode );

    //! Create a new entry
    void CreateEntry( int64_t Address, int64_t Size, void *pValue, GenApi::EAccessMode AccessMode);

    /**
    \brief Create a new entry from a type
    */
    template<class T>
    void CreateEntry( int64_t Address, const T Value, GenApi::EAccessMode AccessMode)
    {
        T _Value = Value;
        CreateEntry( Address, sizeof(_Value), &_Value, AccessMode);
    }

    //! Update entry value
    void UpdateEntry( int64_t Address, int64_t Size, const void *pValue, GenApi::EAccessMode &AccessMode);

    /**
    \brief Create a new entry from a type
    */
    template<class T>
    void UpdateEntry( int64_t Address, const T Value, GenApi::EAccessMode AccessMode)
    {
        T _Value = Value;
        UpdateEntry( Address, sizeof(_Value), &_Value, AccessMode);
    }

    //! Clear all entries
    void ClearAllEntries();

protected:
    //! Map type
    typedef std::map< int64_t, CRegisterEntry > Map_t;

    //! map holding the register entries
    Map_t m_Map;

private:
    //! not implemented: copy constructor
    CRegisterMap( const CRegisterMap& );

    //! not implmeneted: assignment operator
    CRegisterMap& operator=( const CRegisterMap& );

};


#endif // GENAPI_REGISTERMAP_H


