//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "RegisterTest.h"
#include "TestPort.h"
#include "StopWatch.h"
#include "GenApi/GenApi.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( RegisterTestSuite );

void RegisterTestSuite::TestValueAccessSwiss()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <pIsAvailable>RegAvailable</pIsAvailable>    \n"
    "    <pIsLocked>RegLocked</pIsLocked>            \n"
    "    <pAddress>BaseAddress</pAddress>            \n"
    "    <Address>0x0f00</Address>                    \n"
    "    <IntSwissKnife Name=\"Dummy\">                \n"
    "        <pVariable Name=\"I\">Index</pVariable>    \n"
    "        <Formula>0x00f0 * I</Formula>            \n"
    "    </IntSwissKnife>                            \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                        \n"
    "    <Endianess>LittleEndian</Endianess>            \n"

    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"BaseAddress\">                    \n"
    "    <Value>0xf000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Index\">                        \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"RegAvailable\">                \n"
    "    <Value>1</Value>                            \n"
    "</Integer>                                        \n"
    "<Integer Name=\"RegLocked\">                    \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0xff00, (uint32_t)0x12345678, RW);
    Port.CreateEntry(0xfff0, (uint32_t)0x87654321, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    CIntegerPtr ptrIndex = Camera._GetNode("Index");
    CPPUNIT_ASSERT( ptrIndex.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrRegister->GetLength() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)0xff00, ptrRegister->GetAddress() );

    uint32_t Result;
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );

    *ptrIndex = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrIndex->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xfff0, ptrRegister->GetAddress() );

    // from the other one read less bytes
    Result = 0xFEDCBA98;
    ptrRegister->Get((uint8_t*)&Result, 2);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0xFEDC4321, Result );

    // test the same functionality also with CRegisterRef
    // (first uninitialized, then initialized reference)
    CRegisterRef refReg;
    uint32_t Value = 0xAABBCCDD;
    CPPUNIT_ASSERT_THROW( refReg.GetLength(), AccessException );
    CPPUNIT_ASSERT_THROW( refReg.GetAddress(), AccessException );
    CPPUNIT_ASSERT_THROW( refReg.Set((uint8_t*)&Value, 4), AccessException );
    CPPUNIT_ASSERT_THROW( refReg.Get((uint8_t*)&Result, 4), AccessException );

    CPPUNIT_ASSERT_NO_THROW( refReg.SetReference( ptrRegister ) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)4, refReg.GetLength() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xfff0, refReg.GetAddress() );
    CPPUNIT_ASSERT_NO_THROW( refReg.Set((uint8_t*)&Value, 4) );
    CPPUNIT_ASSERT_NO_THROW( refReg.Get((uint8_t*)&Result, 4) );
    CPPUNIT_ASSERT( Value == Result );

    // make the register "not available" and "locked"
    CIntegerPtr ptrAvailable = Camera._GetNode("RegAvailable");
    CIntegerPtr ptrLocked = Camera._GetNode("RegLocked");
    *ptrAvailable = 0;
    CPPUNIT_ASSERT_THROW( refReg.Set((uint8_t*)&Value, 4), AccessException );
    CPPUNIT_ASSERT_THROW( refReg.Get((uint8_t*)&Result, 4, true), AccessException );
    *ptrAvailable = 1;
    *ptrLocked = 1;
    CPPUNIT_ASSERT_THROW( refReg.Set((uint8_t*)&Value, 4), AccessException );
    CPPUNIT_ASSERT_NO_THROW( refReg.Get((uint8_t*)&Result, 4) );
    *ptrLocked = 0;


    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : Register <=> Integer (BaseAddress)");
    LOG_PERFORMANCE("                 <=> Integer (Index voa Swissknife)");

    int Index = 0;
    *ptrIndex = Index;
    ptrIndex->GetNode()->InvalidateNode();
    TEST_BEGIN(1000)
        ptrRegister->Get((uint8_t*)&Result, 4);
        StopWatch.PauseBegin();
        Index = Index ? 0 : 1;
        *ptrIndex = Index;
        StopWatch.PauseEnd();
    TEST_END( Register::GetValue (with index computed via SwissKnife) )


}

void RegisterTestSuite::TestValueAccessIndex()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <pAddress>BaseAddress</pAddress>            \n"
    "    <Address>0x0f00</Address>                    \n"
    "    <pIndex Offset=\"0x00f0\">Index</pIndex>    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                            \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"BaseAddress\">                    \n"
    "    <Value>0xf000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Index\">                        \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0xff00, (uint32_t)0x12345678, RW);
    Port.CreateEntry(0xfff0, (uint32_t)0x87654321, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    CIntegerPtr ptrIndex = Camera._GetNode("Index");
    CPPUNIT_ASSERT( ptrIndex.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrRegister->GetLength() );

    uint32_t Result;
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );

    *ptrIndex = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrIndex->GetValue() );

    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x87654321, Result );

    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : Register <=> Integer (BaseAddress)");
    LOG_PERFORMANCE("                 <=> Integer (Index voa Swissknife)");

    int Index = 0;
    *ptrIndex = Index;
    ptrIndex->GetNode()->InvalidateNode();
    TEST_BEGIN(1000)
        ptrRegister->Get((uint8_t*)&Result, 4);
        StopWatch.PauseBegin();
        Index = Index ? 0 : 1;
        *ptrIndex = Index;
        StopWatch.PauseEnd();
    TEST_END( Register::GetValue (with index computed via Index) )


}

void RegisterTestSuite::TestValueAccessIndexPOffset()
{

    if(GenApiSchemaVersion == v1_0)
        return;


    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <pAddress>BaseAddress</pAddress>            \n"
    "    <Address>0x0f00</Address>                    \n"
    "    <pIndex pOffset=\"Offset\">Index</pIndex>    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                            \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"BaseAddress\">                    \n"
    "    <Value>0xf000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Index\">                        \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Offset\">                        \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0xff00, (uint32_t)0x12345678, RW);
    Port.CreateEntry(0xfff0, (uint32_t)0x87654321, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    CIntegerPtr ptrIndex = Camera._GetNode("Index");
    CPPUNIT_ASSERT( ptrIndex.IsValid() );

    CIntegerPtr ptrOffset = Camera._GetNode("Offset");
    CPPUNIT_ASSERT( ptrOffset.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrRegister->GetLength() );

    uint32_t Result;
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );

    *ptrIndex = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrIndex->GetValue() );
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );

    *ptrOffset = 0xf0;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xf0, ptrOffset->GetValue() );
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x87654321, Result );

}


void RegisterTestSuite::TestValueAccessIndexNoOffset()
{

    if(GenApiSchemaVersion == v1_0)
        return;


    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <pAddress>BaseAddress</pAddress>            \n"
    "    <Address>0x0f00</Address>                    \n"
    "    <pIndex>Index</pIndex>    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                            \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"BaseAddress\">                    \n"
    "    <Value>0xf000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Index\">                        \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0xff00, (uint32_t)0x12345678, RW);
    Port.CreateEntry(0xff04, (uint32_t)0x87654321, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    CIntegerPtr ptrIndex = Camera._GetNode("Index");
    CPPUNIT_ASSERT( ptrIndex.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrRegister->GetLength() );

    uint32_t Result;
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );

    *ptrIndex = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrIndex->GetValue() );

    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x87654321, Result );
}

void RegisterTestSuite::TestValueAccess()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <pAddress>BaseAddress</pAddress>            \n"
    "    <pAddress>Enum</pAddress>                    \n"
    "    <Address>0x0f00</Address>                    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                        \n"
    "    <Endianess>LittleEndian</Endianess>            \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"BaseAddress\">                    \n"
    "    <Value>0xf000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Enumeration Name=\"Enum\">                    \n"
    "    <EnumEntry Name=\"EnumValue1\">                \n"
    "        <Value>0x00</Value>                        \n"
    "   </EnumEntry>                                \n"
    "    <EnumEntry Name=\"EnumValue2\">                \n"
    "        <Value>0xf0</Value>                        \n"
    "   </EnumEntry>                                \n"
    "    <Value>0x00</Value>                            \n"
    "</Enumeration>                                    \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0xff00, (uint32_t)0x12345678, RW);
    Port.CreateEntry(0xfff0, (uint32_t)0x87654321, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CPPUNIT_ASSERT( ptrEnum.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrRegister->GetLength() );

    uint32_t Result;
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );

    *ptrEnum = "EnumValue2";

    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x87654321, Result );


}

void RegisterTestSuite::TestEmbeddedSwissKnife()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <pAddress>BaseAddress</pAddress>            \n"
    "    <pAddress>Enum</pAddress>                    \n"
    "    <IntSwissKnife Name=\"R1\" >                   \n"
    "    <Formula>0x0100</Formula>                    \n"
    "    </IntSwissKnife>                             \n"
    "    <IntSwissKnife Name=\"R2\" >                   \n"
    "    <Formula>0x0100</Formula>                    \n"
    "    </IntSwissKnife>                             \n"
    "    <IntSwissKnife Name=\"R3\" >                   \n"
    "    <Formula>0x0100</Formula>                    \n"
    "    </IntSwissKnife>                             \n"
    "    <IntSwissKnife Name=\"R4\" >                   \n"
    "    <Formula>0x0100</Formula>                    \n"
    "    </IntSwissKnife>                             \n"
    "    <Address>0x0b00</Address>                    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                        \n"
    "    <Endianess>LittleEndian</Endianess>            \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"BaseAddress\">                    \n"
    "    <Value>0xf000</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Enumeration Name=\"Enum\">                    \n"
    "    <EnumEntry Name=\"EnumValue1\">                \n"
    "        <Value>0x00</Value>                        \n"
    "   </EnumEntry>                                \n"
    "    <EnumEntry Name=\"EnumValue2\">                \n"
    "        <Value>0xf0</Value>                        \n"
    "   </EnumEntry>                                \n"
    "    <Value>0x00</Value>                            \n"
    "</Enumeration>                                    \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0xff00, (uint32_t)0x12345678, RW);
    Port.CreateEntry(0xfff0, (uint32_t)0x87654321, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CPPUNIT_ASSERT( ptrEnum.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrRegister->GetLength() );

    uint32_t Result;
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );

    *ptrEnum = "EnumValue2";

    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x87654321, Result );


}

void RegisterTestSuite::TestEmbeddedSwissKnife2()
{

    if(GenApiSchemaVersion == v1_0)
        return;


    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntReg Name=\"Register\">                        \n"
    "    <IntSwissKnife Name=\"Addr\" >                   \n"
    "    <pVariable Name=\"Var\">MyVar</pVariable>                    \n"
    "    <Constant Name=\"Const\">0x100</Constant>                    \n"
    "    <Expression Name=\"Expr\">Const+Var+0x100</Expression>                    \n"
    "    <Formula>Expr+0x100</Formula>                    \n"
    "    </IntSwissKnife>                             \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                        \n"
    "    <Endianess>LittleEndian</Endianess>            \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Integer Name=\"MyVar\">                    \n"
    "    <Value>0x100</Value>                        \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x400, (uint32_t)0x12345678, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CRegisterPtr ptrRegister = Camera._GetNode("Register");
    CPPUNIT_ASSERT( ptrRegister.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrRegister->GetLength() );

    uint32_t Result;
    ptrRegister->Get((uint8_t*)&Result, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0x12345678, Result );
}


void RegisterTestSuite::TestRegisterNode()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Register Name=\"Register\">            \n"
    "    <pAddress>Address</pAddress>            \n"
    "    <pLength>Length</pLength>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "</Register>                            \n"
    "                                                \n"
    "<IntReg Name=\"Length\">                        \n"
    "    <Address>0</Address>                    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                            \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<IntReg Name=\"Address\">                        \n"
    "    <Address>4</Address>                    \n"
    "    <Length>4</Length>                            \n"
    "    <AccessMode>RW</AccessMode>                    \n"
    "    <pPort>Port</pPort>                            \n"
    "    <Sign>Unsigned</Sign>                            \n"
    "    <Endianess>LittleEndian</Endianess>                            \n"
    "</IntReg>                                        \n"
    "                                        \n"
    "<Port Name=\"Port\"/>                    \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    uint8_t RegVal;
    uint16_t RegVal2;
    uint32_t LenVal = 1;
    uint32_t AddrVal = 0x100;

    CTestPort Port;
    Port.CreateEntry(0x0, LenVal, RW);
    Port.CreateEntry(0x4, AddrVal, RW);
    RegVal = 0x11;
    Port.CreateEntry(0x100, RegVal, RW);
    RegVal = 0x22;
    Port.CreateEntry(0x101, RegVal, RW);
    RegVal2 = 0x3344;
    Port.CreateEntry(0x102, RegVal2, RW);
    Camera._Connect(&Port, "Port");

    CIntegerPtr ptrLen = Camera._GetNode("Length");
    CIntegerPtr ptrAddr = Camera._GetNode("Address");
    CRegisterPtr ptrRegister = Camera._GetNode("Register");

    uint8_t Byte;
    uint16_t Word;

    // excercise pAddress and pLength
    ptrRegister->Get(&Byte, 1);
    CPPUNIT_ASSERT_EQUAL( (uint8_t)0x11, Byte );
    *ptrAddr = 0x101;
    ptrRegister->Get(&Byte, 1);
    CPPUNIT_ASSERT_EQUAL( (uint8_t)0x22, Byte );
    *ptrAddr = 0x102;
    *ptrLen = 2;
    ptrRegister->Get((uint8_t*)&Word, 2);
    CPPUNIT_ASSERT_EQUAL( (uint16_t)0x3344, Word );
    *ptrAddr = 0x100;
    *ptrLen = 1;
    // try to get more data than register length
    CPPUNIT_ASSERT_THROW (ptrRegister->Get((uint8_t*)&Word, 2), OutOfRangeException);

}
