//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Hartmut Nebelung
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of ISelector
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "SelectorTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "GenApi/NodeCallbackImpl.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

#undef min

CPPUNIT_TEST_SUITE_REGISTRATION( SelectorTestSuite );

struct cb
{
  unsigned m_Count;
  cb() : m_Count(0) {}
  void doCount( INode* ) { ++m_Count;    }
  unsigned Count() const { return m_Count; }
};

// Test a selector of integer type
void SelectorTestSuite::TestSelector01()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
  "    <Category Name=\"catUserSet\"> \n"
    "    <ToolTip>the one and only category</ToolTip> \n"
    "    <DisplayName>\"A\"</DisplayName>\n"
    "    <Visibility>Beginner</Visibility>\n"
    "    <pFeature>regUserSetName</pFeature>\n"
    "    <pFeature>UserSetVersion</pFeature>\n"
    "</Category>\n"
    "<Integer Name=\"selector\">\n"
    "    <Extension/>\n"
    "    <ToolTip>Integer is a selector for category a</ToolTip>\n"
    "    <DisplayName>Mode</DisplayName>\n"
    "    <Visibility>Beginner</Visibility>\n"
    "    <Value>0</Value>\n"
    "    <Min>0</Min>\n"
    "    <Max>1</Max>\n"
    "    <Inc>1</Inc>\n"
    "    <Representation>Linear</Representation>\n"
    "        <pSelected>catUserSet</pSelected>\n"
    "</Integer>\n"
    "<Float Name=\"UserSetVersion\">\n"
    "    <pValue>rVersion</pValue>\n"
    "</Float>\n"
    "<FloatReg Name=\"rVersion\">\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x0</Address>\n"
    "    <Length>4</Length>\n"
    "    <AccessMode>RO</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</FloatReg>\n"
    "<StringReg Name=\"regUserSetName\">\n"
    "    <DisplayName>Name</DisplayName>\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x4</Address>\n"
    "    <pLength>UserSetNameLen</pLength>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "</StringReg>\n"
  "<Integer Name=\"UserSetNameLen\">\n"
  " <Value>48</Value>\n"
  "</Integer>\n"
    "<IntSwissKnife Name=\"adrCategory_A\">\n"
    "    <Visibility>Invisible</Visibility>\n"
    "    <pVariable Name=\"INDEX\">selector</pVariable>\n"
    "    <Formula>0x1000+0x100*INDEX</Formula>\n"
    "</IntSwissKnife>\n"
    "<Port Name=\"myPort\"/>\n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x2000, 42ULL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  CSelectorPtr ptrSelector = Camera._GetNode( "selector" );
  CPPUNIT_ASSERT( ptrSelector.IsValid() );

  CPPUNIT_ASSERT( ptrSelector->IsSelector() );
  FeatureList_t selected;
  ptrSelector->GetSelectedFeatures( selected );
  CPPUNIT_ASSERT_EQUAL( (size_t)1, selected.size() );
  CNodePtr ptrNode01 = selected[0]->GetNode();
  CPPUNIT_ASSERT( ptrNode01.IsValid() );

  gcstring name01( "catUserSet" );
  CPPUNIT_ASSERT_EQUAL( name01, ptrNode01->GetName() );

  {
    FeatureList_t selected2, selected3;
    CSelectorRef refSelector;

    // Pleora FG - ref, GetSelectingFeatures and IsSelector, should throw
    CPPUNIT_ASSERT_THROW_EX( refSelector.GetSelectingFeatures( selected2 ), AccessException );
    CPPUNIT_ASSERT_THROW_EX( refSelector.IsSelector(), AccessException );

    refSelector.SetReference( ptrSelector );
    CPPUNIT_ASSERT_EQUAL( RW, refSelector.GetAccessMode() );
    CPPUNIT_ASSERT_NO_THROW( refSelector.GetSelectedFeatures( selected2 ) );
    CPPUNIT_ASSERT( selected.size() == selected2.size() );
//    for (int i=0; i<selected.size(); ++i) CPPUNIT_ASSERT_EQUAL( selected[i], selected2[i] ); // *JS* removed warning
    for (size_t i=0; i<selected.size(); ++i) CPPUNIT_ASSERT_EQUAL( selected[i], selected2[i] );
    CPPUNIT_ASSERT_NO_THROW( refSelector.SetReference( 0 ) );
    CPPUNIT_ASSERT_THROW_EX( refSelector.GetSelectedFeatures( selected2 ), AccessException );

    // Pleora FG - ref, GetSelectingFeatures and IsSelector, should throw
    refSelector.SetReference( 0 );
    CPPUNIT_ASSERT_THROW_EX( refSelector.GetSelectingFeatures( selected2 ), AccessException );
    CPPUNIT_ASSERT_THROW_EX( refSelector.IsSelector(), AccessException );

    // Pleora FG - ref, GetSelectingFeature and IsSelector - should work
    refSelector.SetReference( ptrSelector );
    CPPUNIT_ASSERT_NO_THROW( refSelector.GetSelectingFeatures( selected2 ) );
    CPPUNIT_ASSERT_NO_THROW( ptrSelector->GetSelectingFeatures( selected3 ) );
    CPPUNIT_ASSERT( selected2.size() == selected3.size() );
    CPPUNIT_ASSERT( refSelector.IsSelector() == ptrSelector->IsSelector() );
  }
}

// Test a selector of enum type
void SelectorTestSuite::TestSelector02()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "    <Category Name=\"catUserSet\"> \n"
    "    <Extension/>\n"
    "    <ToolTip>the one and only category</ToolTip>\n"
    "    <DisplayName>A</DisplayName>\n"
    "    <Visibility>Beginner</Visibility>\n"
    "    <pFeature>regUserSetName</pFeature>\n"
    "    <pFeature>UserSetVersion</pFeature>\n"
    "</Category>\n"
    "<Enumeration Name=\"selector\">\n"
    "    <EnumEntry Name=\"MySettings\">\n"
    "        <Value>0</Value>\n"
    "    </EnumEntry>\n"
    "    <EnumEntry Name=\"HansSettings\">\n"
    "        <Value>1</Value>\n"
    "    </EnumEntry>\n"
    "    <pValue>regSelector</pValue>\n"
    "        <pSelected>catUserSet</pSelected>\n"
    "        <pSelected>UserSetVersion</pSelected>\n"
    "</Enumeration>\n"
    "<IntReg Name=\"regSelector\">\n"
    "    <Address>0x0</Address>\n"
    "    <Length>2</Length>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Sign>Unsigned</Sign>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</IntReg>\n"
    "<Float Name=\"UserSetVersion\">\n"
    "    <pValue>rVersion</pValue>\n"
    "</Float>\n"
    "<FloatReg Name=\"rVersion\">\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x0</Address>\n"
    "    <Length>4</Length>\n"
    "    <AccessMode>RO</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</FloatReg>\n"
    "<StringReg Name=\"regUserSetName\">\n"
    "    <DisplayName>Name</DisplayName>\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x4</Address>\n"
    "    <pLength>UserSetNameLen</pLength>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "</StringReg>\n"
  "<Integer Name=\"UserSetNameLen\">\n"
  " <Value>48</Value>\n"
  "</Integer>\n"
    "<IntSwissKnife Name=\"adrCategory_A\">\n"
    "    <Visibility>Invisible</Visibility>\n"
    "    <pVariable Name=\"INDEX\">regSelector</pVariable>\n"
    "    <Formula>0x1000+0x100*INDEX</Formula>\n"
    "</IntSwissKnife>\n"
    "<Port Name=\"myPort\"/>\n"
+ CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x2000, 42ULL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  CSelectorPtr ptrSelector = Camera._GetNode( "selector" );
  CPPUNIT_ASSERT( ptrSelector.IsValid() );

  CPPUNIT_ASSERT( ptrSelector->IsSelector() );
  FeatureList_t selected;
  ptrSelector->GetSelectedFeatures( selected );
  CPPUNIT_ASSERT_EQUAL( (size_t)2, selected.size() );
  CNodePtr ptrNode01(selected[0]->GetNode());
  CPPUNIT_ASSERT( ptrNode01.IsValid() );
  CNodePtr ptrNode02(selected[1]->GetNode());
  CPPUNIT_ASSERT( ptrNode02.IsValid() );

  gcstring name01( "catUserSet" );
  gcstring name02( "UserSetVersion" );
  CPPUNIT_ASSERT_EQUAL( name01, ptrNode01->GetName() );
  CPPUNIT_ASSERT_EQUAL( name02, ptrNode02->GetName() );
}


// Test selector of masked int reg type
void SelectorTestSuite::TestSelector03()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "    <Category Name=\"catUserSet\"> \n"
    "    <Extension/>\n"
    "    <ToolTip>the one and only category</ToolTip>\n"
    "    <DisplayName>A</DisplayName>\n"
    "    <Visibility>Beginner</Visibility>\n"
    "    <pFeature>regUserSetName</pFeature>\n"
    "    <pFeature>UserSetVersion</pFeature>\n"
    "</Category>\n"
    "<MaskedIntReg Name=\"selector\"> \n"
    "    <Address>0x0</Address>\n"
    "    <Length>2</Length>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Bit>4</Bit>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "        <pSelected>catUserSet</pSelected>\n"
    "        <pSelected>UserSetVersion</pSelected>\n"
    "</MaskedIntReg>\n"
    "    <Float Name=\"UserSetVersion\">\n"
    "    <pValue>rVersion</pValue>\n"
    "</Float>\n"
    "<FloatReg Name=\"rVersion\">\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x0</Address>\n"
    "    <Length>4</Length>\n"
    "    <AccessMode>RO</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</FloatReg>\n"
    "<StringReg Name=\"regUserSetName\">\n"
    "    <DisplayName>Name</DisplayName>\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x4</Address>\n"
    "    <pLength>UserSetNameLen</pLength>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "</StringReg>\n"
  "<Integer Name=\"UserSetNameLen\">\n"
  " <Value>48</Value>\n"
  "</Integer>\n"
    "<IntSwissKnife Name=\"adrCategory_A\">\n"
    "    <Visibility>Invisible</Visibility>\n"
    "    <pVariable Name=\"INDEX\">selector</pVariable>\n"
    "    <Formula>0x1000+0x100*INDEX</Formula>\n"
    "</IntSwissKnife>\n"
    "<Port Name=\"myPort\"/>\n"
+ CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x2000, 42ULL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  CSelectorPtr ptrSelector(Camera._GetNode( "selector" ));
  CPPUNIT_ASSERT( ptrSelector.IsValid() );

  CPPUNIT_ASSERT( ptrSelector->IsSelector() );
  FeatureList_t selected;
  ptrSelector->GetSelectedFeatures( selected );
  CPPUNIT_ASSERT_EQUAL( (size_t)2, selected.size() );
  CNodePtr ptrNode01(selected[0]->GetNode());
  CPPUNIT_ASSERT( ptrNode01.IsValid() );
  CNodePtr ptrNode02(selected[1]->GetNode());
  CPPUNIT_ASSERT( ptrNode02.IsValid() );

  gcstring name01( "catUserSet" );
  gcstring name02( "UserSetVersion" );
  CPPUNIT_ASSERT_EQUAL( name01, ptrNode01->GetName() );
  CPPUNIT_ASSERT_EQUAL( name02, ptrNode02->GetName() );

  cb c;
  GenApi::Register( ptrNode01, c, &cb::doCount );
  GenApi::Register( ptrNode02, c, &cb::doCount );

  CNodePtr ptrSelectorNode(ptrSelector);
  ptrSelectorNode->InvalidateNode();
}

// Test selector of int reg type
void SelectorTestSuite::TestSelector04()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "    <Category Name=\"catUserSet\"> \n"
    "    <Extension/>\n"
    "    <ToolTip>the one and only category</ToolTip>\n"
    "    <DisplayName>A</DisplayName>\n"
    "    <Visibility>Beginner</Visibility>\n"
    "    <pFeature>regUserSetName</pFeature>\n"
    "    <pFeature>UserSetVersion</pFeature>\n"
    "</Category>\n"
    "<IntReg Name=\"selector\"> \n"
    "    <Address>0x0</Address>\n"
    "    <Length>3</Length>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Sign>Signed</Sign>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "        <pSelected>regUserSetName</pSelected>\n"
    "        <pSelected>UserSetVersion</pSelected>\n"
    "</IntReg> \n"
    "<Float Name=\"UserSetVersion\">\n"
    "    <pValue>rVersion</pValue>\n"
    "</Float>\n"
    "<FloatReg Name=\"rVersion\">\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x0</Address>\n"
    "    <Length>4</Length>\n"
    "    <AccessMode>RO</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</FloatReg>\n"
    "<StringReg Name=\"regUserSetName\">\n"
    "    <DisplayName>Name</DisplayName>\n"
    "    <pAddress>adrCategory_A</pAddress>\n"
    "    <Address>0x4</Address>\n"
    "    <pLength>UserSetNameLen</pLength>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "</StringReg>\n"
  "<Integer Name=\"UserSetNameLen\">\n"
  " <Value>48</Value>\n"
  "</Integer>\n"
    "<IntSwissKnife Name=\"adrCategory_A\">\n"
    "    <Visibility>Invisible</Visibility>\n"
    "    <pVariable Name=\"INDEX\">selector</pVariable>\n"
    "    <Formula>0x1000+0x100*INDEX</Formula>\n"
    "</IntSwissKnife>\n"
    "<Port Name=\"myPort\"/>\n"
+ CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x2000, 42ULL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  CSelectorPtr ptrSelector(Camera._GetNode( "selector" ));
  CPPUNIT_ASSERT( ptrSelector.IsValid() );

  CPPUNIT_ASSERT( ptrSelector->IsSelector() );
  FeatureList_t selected;
  ptrSelector->GetSelectedFeatures( selected );
  CPPUNIT_ASSERT_EQUAL( (size_t)2, selected.size() );
  CNodePtr ptrNode01(selected[0]->GetNode());
  CPPUNIT_ASSERT( ptrNode01.IsValid() );
  CNodePtr ptrNode02(selected[1]->GetNode());
  CPPUNIT_ASSERT( ptrNode02.IsValid() );

  gcstring name01( "regUserSetName" );
  gcstring name02( "UserSetVersion" );
  CPPUNIT_ASSERT_EQUAL( name01, ptrNode01->GetName() );
  CPPUNIT_ASSERT_EQUAL( name02, ptrNode02->GetName() );
}

// -------------------------------------------------------------
/**
*  \brief Test selector of an array
*
*  A minimal register description is created. The root category
*  contains two features only. The selector is a "floating" integer
*  node with range from 40 to 49. The integer register "entry" is
*  one of the ten elements of the array. The index and the addres
*  are directly determined by the selector.
*
*  The test iterates through the whole array and sets the value.
*  The registered callback object c counts the invalidation calls.
*/
void SelectorTestSuite::TestSelector05()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +

  "<Category Name=\"Root\"> \n"
  " <pFeature>Entry</pFeature> \n"
  " <pFeature>selector</pFeature> \n"
  "</Category> \n"

    "<IntReg Name=\"Entry\"> \n"
  " <pAddress>selector</pAddress>\n"
    "    <Length>4</Length>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Sign>Signed</Sign>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</IntReg> \n"

  "<Integer Name=\"selector\">\n"
  " <Value>48</Value>\n"
  " <Min>40</Min>\n"
  " <Max>49</Max>\n"
  " <pSelected>Entry</pSelected> \n"
  "</Integer>\n"

    "<Port Name=\"myPort\"/>\n"
+ CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    for (int i=40; i<50; ++i) Port.CreateEntry(i, 42UL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  CSelectorPtr ptrSelector = Camera._GetNode( "selector" );
  CPPUNIT_ASSERT( ptrSelector.IsValid() );

  CPPUNIT_ASSERT( ptrSelector->IsSelector() );
  FeatureList_t selected;
  ptrSelector->GetSelectedFeatures( selected );
  CPPUNIT_ASSERT_EQUAL( (size_t)1, selected.size() );
  CNodePtr ptrNode01(selected[0]->GetNode());
  CPPUNIT_ASSERT( ptrNode01.IsValid() );

  gcstring name01( "Entry" );
  CPPUNIT_ASSERT_EQUAL( name01, ptrNode01->GetName() );

  cb c;
  GenApi::Register( ptrNode01, c, &cb::doCount );

  CIntegerPtr ptrInteger(ptrSelector);
  CValuePtr ptrValue(ptrNode01);
  for (int64_t i=ptrInteger->GetMin(); i<=ptrInteger->GetMax(); i+=ptrInteger->GetInc())
  {
    ptrInteger->SetValue( i );
    // gcstring str = ptrValue->ToString();
   }
  CPPUNIT_ASSERT_EQUAL( (unsigned) 10, c.Count() );
}


// -------------------------------------------------------------
/**
*  \brief Test selector of a catgory
*
*  A category must not become a selected item even if one of its features is selected
*/
void SelectorTestSuite::TestSelector06()
{

  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "<Category Name=\"ACategory\"> \n"
    "    <pFeature>ASelectedFeature</pFeature>\n"
    "    <pFeature>ASelector</pFeature>\n"
    "</Category>\n"
    "<Integer Name=\"ASelectedFeature\">\n"
    "    <Value>0</Value>\n"
    "</Integer>\n"
    "<Integer Name=\"ASelector\">\n"
    "    <Value>0</Value>\n"
    "    <pSelected>ASelectedFeature</pSelected>\n"
    "</Integer>\n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CSelectorPtr ptrSelectedFeature = Camera._GetNode("ASelectedFeature");
    CPPUNIT_ASSERT( ptrSelectedFeature.IsValid() );

    FeatureList_t SelectingFeatures;
    ptrSelectedFeature->GetSelectingFeatures(SelectingFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)1, SelectingFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( SelectingFeatures[0]->GetNode()->GetName(), gcstring("ASelector") );

    CSelectorPtr ptrCategory = Camera._GetNode("ACategory");
    CPPUNIT_ASSERT( ptrCategory.IsValid() );
    ptrCategory->GetSelectingFeatures(SelectingFeatures);
    // nobody is selecting a category
    CPPUNIT_ASSERT_EQUAL( (size_t)0, SelectingFeatures.size() );

}

/**
*  \brief Test selector of a catgory
*
*  A dependencies must be unique
*/
void SelectorTestSuite::TestSelector07()
{

  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "<Integer Name=\"Integer\">\n"
    "    <pValue>Value</pValue>\n"
    "    <pMin>Min</pMin>\n"
    "</Integer>\n"
    "<Integer Name=\"Value\">\n"
    "    <Value>0</Value>\n"
    "</Integer>\n"
    "<Integer Name=\"Min\">\n"
    "    <Value>0</Value>\n"
    "</Integer>\n"
    "<Integer Name=\"Selector\">\n"
    "    <Value>0</Value>\n"
    "    <pSelected>Value</pSelected>\n"
    "    <pSelected>Min</pSelected>\n"
    "</Integer>\n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CSelectorPtr ptrInteger = Camera._GetNode("Integer");
    CPPUNIT_ASSERT( ptrInteger.IsValid() );

    FeatureList_t SelectingFeatures;
    ptrInteger->GetSelectingFeatures(SelectingFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)1, SelectingFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( SelectingFeatures[0]->GetNode()->GetName(), gcstring("Selector") );
}

void SelectorTestSuite::TestBracketOperator()
{

    // TODO : implement bracket operator
#if 0

    // create and initilize node map
    using namespace Test_TestCamera_v1_1;
    CTestCamera_v1_1 Camera;
    Camera._LoadDLL();

    Camera.Gain[ GainSelector_Green ].SetValue( 42 );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, dynamic_cast<IInteger*>(Camera._GetNode("GainGreen"))->GetValue() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Green"), Camera.GainSelector.ToString() );
    Camera.Gain[ GainSelector_Green ].SetValue( 815 );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 815LL, dynamic_cast<IInteger*>(Camera._GetNode("GainGreen"))->GetValue() );

    Camera.Gain[ GainSelector_Red ].SetValue( 42 );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, dynamic_cast<IInteger*>(Camera._GetNode("GainRed"))->GetValue() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Red"), Camera.GainSelector.ToString() );
    Camera.Gain[ GainSelector_Red ].SetValue( 815 );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 815LL, dynamic_cast<IInteger*>(Camera._GetNode("GainRed"))->GetValue() );

    Camera.Gain[ GainMode_Analog ].SetValue( 42 );
    Camera.Gain[ GainSelector_Green ][ GainMode_Analog ].SetValue( 42 );

    Camera.Gain[ GainMode_Digital ][ GainSelector_Blue ].SetValue( 12345 );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 12345LL, dynamic_cast<IInteger*>(Camera._GetNode("GainBlue"))->GetValue() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Blue"), Camera.GainSelector.ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Digital"), Camera.GainMode.ToString() );

    Camera.Gain.SetSelector( GainSelector_Green ).SetSelector( GainMode_Analog ).SetSelector( 3 ).SetValue( 42 );
    Camera.Gain[ GainSelector_Green ][ GainMode_Analog ][3] = 42;
#endif
}

void SelectorTestSuite::TestBooleanSelector()
{
    if(GenApiSchemaVersion < v1_1)
        return;

  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "<Boolean Name=\"selector\">\n"
    "    <pValue>regSelector</pValue>\n"
    "    <OnValue>0x1000</OnValue>\n"
    "    <OffValue>0x2000</OffValue>\n"
    "        <pSelected>UserSetVersion</pSelected>\n"
    "</Boolean>\n"
    "<IntReg Name=\"regSelector\">\n"
    "    <Address>0x0</Address>\n"
    "    <Length>4</Length>\n"
    "    <AccessMode>RW</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Sign>Unsigned</Sign>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</IntReg>\n"
    "<Integer Name=\"UserSetVersion\">\n"
    "    <pValue>rVersion</pValue>\n"
    "</Integer>\n"
    "<IntReg Name=\"rVersion\">\n"
    "    <pAddress>regSelector</pAddress>\n"
    "    <Length>4</Length>\n"
    "    <AccessMode>RO</AccessMode>\n"
    "    <pPort>myPort</pPort>\n"
    "    <Endianess>LittleEndian</Endianess>\n"
    "</IntReg>\n"
    "<Port Name=\"myPort\"/>\n"
+ CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x0, 0x1000, RW);
    Port.CreateEntry(0x1000, 42ULL, RW);
    Port.CreateEntry(0x2000, 24ULL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  CSelectorPtr ptrSelector = Camera._GetNode( "selector" );
  CPPUNIT_ASSERT( ptrSelector.IsValid() );

  CPPUNIT_ASSERT( ptrSelector->IsSelector() );
  FeatureList_t selected;
  ptrSelector->GetSelectedFeatures( selected );
  CPPUNIT_ASSERT_EQUAL( (size_t)1, selected.size() );
  CNodePtr ptrNode01(selected[0]->GetNode());
  CPPUNIT_ASSERT( ptrNode01.IsValid() );

  gcstring name01( "UserSetVersion" );
  CPPUNIT_ASSERT_EQUAL( name01, ptrNode01->GetName() );
}

void SelectorTestSuite::TestSelecting()
{

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                \n"
        "<Category Name=\"Root\">                \n"
        "    <pFeature>Feature</pFeature>                \n"
        "</Category>                                    \n"
        "                                                \n"
        "<Integer Name=\"Feature\">                        \n"
        "    <Value>0</Value>                        \n"
        "    <pSelected>Changer</pSelected>                \n"
        "</Integer>                                        \n"
        "                                                \n"
        "<Integer Name=\"Changer\">                        \n"
        "    <Value>0</Value>                            \n"
        "</Integer>                                        \n"
        "                                                \n"
        + CDFFooter;

    CNodeMapRef Camera("TestCamera");
    Camera._LoadXMLFromString(CameraDescriptionFile);

    printf("Dumping PropertyNames:\n");
    NodeList_t Nodes;
    Camera._GetNodes(Nodes);
    for( NodeList_t::iterator itNode=Nodes.begin(); itNode!=Nodes.end(); itNode++ )
    {
        cout << "Node '" << (*itNode)->GetName().c_str() << "'\n";

        GenICam::gcstring_vector PropertyNames;
        (*itNode)->GetPropertyNames(PropertyNames);

        gcstring ValueStr;
        gcstring AttributeStr;
        GenICam::gcstring_vector::iterator ptr;
        for(ptr=PropertyNames.begin(); ptr!=PropertyNames.end(); ptr++)
        {
            bool res = (*itNode)->GetProperty((*ptr), ValueStr, AttributeStr);
            if(!res)
                printf("\t  Property '%s' is not available\n", (*ptr).c_str());
            else if(AttributeStr.empty())
                printf("\t  Property '%s' = '%s'\n", (*ptr).c_str(), ValueStr.c_str());
            else
                printf("\t  Property '%s' = '%s' [%s]\n", (*ptr).c_str(), ValueStr.c_str(), AttributeStr.c_str());
        }
    }

}

