//-----------------------------------------------------------------------------
//  (c) 2006 by MVTec Software GmbH
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Ivonne Puhlmann
//  $Header: /cvs/genicam/genicam/tests/GenApi/GenApiTest/SmartFeatureTest.cpp,v 1.0 2006/06/28 15:59:23
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of CSmartFeatures
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "SmartFeaturesTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "GenApi/impl/Compatibility.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( SmartFeaturesTestSuite );


class CTestPortEx : public GenApi::CPortImpl
{
public:
   //-------------------------------------------------------------
    // IBase implementation
    //-------------------------------------------------------------

    //! Get the access mode of the node
    virtual GenApi::EAccessMode GetAccessMode() const
    {
        return RW;
    }

    //! Get the type of the main interface of a node
    virtual GenApi::EInterfaceType GetPrincipalInterfaceType() const
    {
        return intfIPort;
    }

    //---------------------------------------------------------------
    // IPort implementation
    //---------------------------------------------------------------

    //! Reads a chunk of bytes from the port
    virtual void Read(void *pBuffer, int64_t Address, int64_t Length)
    {
        ReadPort.Read(pBuffer, Address, Length);
    }

    //! Writes a chunk of bytes to the port
    virtual void Write(const void *pBuffer, int64_t Address, int64_t Length)
    {
        WritePort.Write(pBuffer, Address, Length);
    }

    //! implements the read access
    CTestPort ReadPort;

    // implements the write access
    CTestPort WritePort;

};



void SmartFeaturesTestSuite::TestSmartFeature()
{
    gcstring CameraDescription =
    CDFHeader +

    "<SmartFeature Name=\"SmartFeature\">                            \n"
    "  <FeatureID>76879410-A3EE-4f80-911E-BE2FD0D56C04</FeatureID>    \n"
    "  <Address>0x0080</Address>                                    \n"
    "  <pPort>Port</pPort>                                            \n"
    "  <pIsImplemented>Implemented</pIsImplemented>                 \n"
    "  <pIsAvailable>Available</pIsAvailable>                       \n"
    "</SmartFeature>                                                \n"
    "                                                                \n"
    "<Integer Name=\"Available\">                                    \n"
    "  <Value>1</Value>                                                \n"
    "</Integer>                                                        \n"
    "                                                               \n"
    "<Integer Name=\"Implemented\">                                    \n"
    "  <Value>1</Value>                                                \n"
    "</Integer>                                                        \n"
    "                                                                \n"
    "<Port Name=\"Port\"/>                                            \n"

    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CIntegerPtr ptrSmartFeature = Camera._GetNode( "SmartFeature" );
    CIntegerPtr ptrImplemented = Camera._GetNode( "Implemented" );
    CIntegerPtr ptrAvailable = Camera._GetNode( "Available" );
    CPPUNIT_ASSERT( ptrAvailable.IsValid() );
    CPPUNIT_ASSERT( ptrImplemented.IsValid() );
    CPPUNIT_ASSERT( ptrSmartFeature.IsValid() );
    CPPUNIT_ASSERT_EQUAL (intfIInteger, ptrSmartFeature->GetNode()->GetPrincipalInterfaceType());


    // create and initialize a test port

    // {76879410-A3EE-4f80-911E-BE2FD0D56C04}
    _DEFINE_GUID(FeatureGuid, 0x76879410, 0xa3ee, 0x4f80, 0x91, 0x1e, 0xbe, 0x2f, 0xd0, 0xd5, 0x6c, 0x4);

    {
        typedef union { GUID g; uint32_t a[4]; } uguid;
        uguid &x(*(uguid *) &FeatureGuid);
        x.a[0] = bswap_32(x.a[0]);
        x.a[1] = bswap_32(x.a[1]);
        x.a[2] = bswap_32(x.a[2]);
        x.a[3] = bswap_32(x.a[3]);
    }

    // The address in the register consists of two big endian quadlets
    static const int32_t FeatureAddress[2] = { 0x01234567, 0x89ABCDEF };
    static const int64_t ExpectedAddress(0xEFCDAB8967452301LL);

    // The smart feautre CSR consists of the GUID and the smart feature's address
    uint8_t CSR[sizeof(FeatureGuid) + sizeof(FeatureAddress)];
    memcpy(CSR, &FeatureGuid, sizeof(FeatureGuid));
    memcpy((uint8_t*)CSR + sizeof(FeatureGuid), &FeatureAddress, sizeof(FeatureAddress));

    CTestPortEx Port;
    Port.WritePort.CreateEntry(0x0080, sizeof(FeatureGuid), &FeatureGuid, RW);
    Port.ReadPort.CreateEntry(0x0080, sizeof(CSR), &CSR, RW);
    //Port.WritePort.CreateEntry(0x080, sizeof(CSR), &CSR, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    // Get the address
    const int64_t Address(*ptrSmartFeature());
    CPPUNIT_ASSERT_EQUAL( Address, ExpectedAddress );

    // make sure the node has written the right GUID
    GUID TestGuid;
    EAccessMode TestAccessMode;
    Port.WritePort.LookupEntry(0x0080, sizeof(GUID), &TestGuid, TestAccessMode);
    CPPUNIT_ASSERT( IsEqualGUID(FeatureGuid, TestGuid) );

    Port.InvalidateNode();

    CPPUNIT_ASSERT_EQUAL( RO, ptrSmartFeature->GetAccessMode() );

    CPPUNIT_ASSERT_EQUAL( (int64_t) 1, ptrSmartFeature->GetInc() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0x7fffffffffffffffLL, ptrSmartFeature->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0, ptrSmartFeature->GetMin() );

    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrSmartFeature->GetRepresentation() );
    CPPUNIT_ASSERT_THROW( ptrSmartFeature->SetValue(11LL,false), AccessException);
    CPPUNIT_ASSERT_EQUAL( gcstring(""), ptrSmartFeature->GetUnit() );

    CPPUNIT_ASSERT_EQUAL( ExpectedAddress, ptrSmartFeature->GetValue() );
    CPPUNIT_ASSERT_THROW( ptrSmartFeature->SetValue(13LL), AccessException);;

    // test NI AccessMode
    *ptrImplemented = 0;
    CPPUNIT_ASSERT_THROW(ptrSmartFeature->GetValue(true), AccessException);
    CPPUNIT_ASSERT_THROW(ptrSmartFeature->SetValue(78LL), AccessException);
    *ptrImplemented = 1;

    // test NA AccessMode
    *ptrAvailable = 0;
    CPPUNIT_ASSERT_THROW(ptrSmartFeature->GetValue(true), AccessException);
    CPPUNIT_ASSERT_THROW(ptrSmartFeature->SetValue(78LL), AccessException);
    *ptrAvailable = 1;

}

void SmartFeaturesTestSuite::TestAdvancedFeatureNI()
{
    gcstring CameraDescription =
    CDFHeader +

    "<AdvFeatureLock Name=\"AdvancedFeature\">                        \n"
    "  <Address>0x0080</Address>                                    \n"
    "  <Length>8</Length>                                            \n"
    "  <AccessMode>RW</AccessMode>                                    \n"
    "  <pPort>Port</pPort>                                            \n"
    "  <FeatureID>0x0123456789AB</FeatureID>                        \n"
    "  <Timeout>0</Timeout>                                            \n"
    "</AdvFeatureLock>                                                \n"
    "                                                                \n"
    "<Port Name=\"Port\"/>                                            \n"

    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CIntegerPtr ptrAdvancedFeature = Camera._GetNode( "AdvancedFeature" );
    CPPUNIT_ASSERT( ptrAdvancedFeature.IsValid() );

    CTestPortEx Port;
    int64_t FeatureID(0x0123456789ABLL);

    Port.WritePort.CreateEntry(0x0080, sizeof(FeatureID), &FeatureID, RW);
    Port.InvalidateNode();

    // test NI AccessMode
    CPPUNIT_ASSERT_THROW(ptrAdvancedFeature->GetValue(), AccessException);

}

void SmartFeaturesTestSuite::TestAdvancedFeatureWO()
{
    gcstring CameraDescription =
    CDFHeader +

    "<AdvFeatureLock Name=\"AdvancedFeature\">                        \n"
    "  <Address>0x0020</Address>                                    \n"
    "  <Length>8</Length>                                            \n"
    "  <AccessMode>WO</AccessMode>                                    \n"
    "  <pPort>Port</pPort>                                            \n"
    "  <FeatureID>0x0123456789AB</FeatureID>                        \n"
    "  <Timeout>0</Timeout>                                            \n"
    "</AdvFeatureLock>                                                \n"
    "                                                                \n"
    "                                                               \n"
    "<Port Name=\"Port\"/>                                            \n"

    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CIntegerPtr ptrAdvancedFeature(Camera._GetNode( "AdvancedFeature" ));
    CPPUNIT_ASSERT( ptrAdvancedFeature.IsValid() );

    CTestPortEx Port;
    int64_t FeatureID(0x0123456789ABLL);

    uint32_t *p((uint32_t*)&FeatureID);
    for(int i = 0; i < sizeof(FeatureID) / sizeof(uint32_t); ++i )
        p[i] = bswap_32(p[i]);

    Port.WritePort.CreateEntry(0x0020, sizeof(FeatureID), &FeatureID, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    // test WO AccessMode
    CPPUNIT_ASSERT_THROW(ptrAdvancedFeature->GetValue(), AccessException);

}

void SmartFeaturesTestSuite::TestAdvancedFeatureRW()
{
    gcstring CameraDescription =
    CDFHeader +

    "<AdvFeatureLock Name=\"AdvancedFeature\">                        \n"
    "  <Address>0x0020</Address>                                    \n"
    "  <Length>8</Length>                                            \n"
    "  <AccessMode>RO</AccessMode>                                    \n"
    "  <pPort>Port</pPort>                                            \n"
    "  <FeatureID>0x0123456789AB</FeatureID>                        \n"
    "  <Timeout>0</Timeout>                                            \n"
    "</AdvFeatureLock>                                                \n"
    "                                                                \n"
    "<SmartFeature Name=\"SmartFeature\">                            \n"
    "  <FeatureID>76879410-A3EE-4f80-911E-BE2FD0D56C04</FeatureID>    \n"
    "  <Address>0x0080</Address>                                    \n"
    "  <pPort>Port</pPort>                                            \n"
    "  <pIsImplemented>Implemented</pIsImplemented>                 \n"
    "  <pIsAvailable>AdvancedFeature</pIsAvailable>                 \n"
    "</SmartFeature>                                                \n"
    "                                                                \n"
    "<Integer Name=\"Available\">                                    \n"
    "  <Value>1</Value>                                                \n"
    "</Integer>                                                        \n"
    "                                                               \n"
    "<Integer Name=\"Implemented\">                                    \n"
    "  <Value>1</Value>                                                \n"
    "</Integer>                                                        \n"
    "                                                                \n"
    "                                                               \n"
    "<Port Name=\"Port\"/>                                            \n"

    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CIntegerPtr ptrAdvancedFeature = Camera._GetNode( "AdvancedFeature" );
    CPPUNIT_ASSERT( ptrAdvancedFeature.IsValid() );

    CTestPortEx Port;
    int64_t FeatureID(0x0123456789ABLL);

    uint32_t *p = (uint32_t*)&FeatureID;
    for(int i = 0; i < sizeof(FeatureID) / sizeof(uint32_t); ++i )
        p[i] = bswap_32(p[i]);

    // The address in the register consists of two big endian quadlets
    const int32_t FeatureAddress2[1] = { 0x98765432 };
    const int32_t ExpectedAddress2(0x32547698);
    // The CSR2 consists of the FeatureID and it�s address
    uint8_t CSR2[sizeof(FeatureID) + sizeof(FeatureAddress2)];
    memcpy(CSR2, &FeatureID, sizeof(FeatureID));
    memcpy((uint8_t*)CSR2 + sizeof(FeatureID), &FeatureAddress2, sizeof(FeatureAddress2));

    Port.WritePort.CreateEntry(0x0020, sizeof(FeatureID), &FeatureID, RW);
    Port.ReadPort.CreateEntry(0x0020, sizeof(CSR2), &CSR2, RW);
    Port.WritePort.CreateEntry(0x0024, sizeof(FeatureID), &FeatureID, RW);// hack
    Port.InvalidateNode();

    // {76879410-A3EE-4f80-911E-BE2FD0D56C04}
    _DEFINE_GUID(FeatureGuid, 0x76879410, 0xa3ee, 0x4f80, 0x91, 0x1e, 0xbe, 0x2f, 0xd0, 0xd5, 0x6c, 0x4);

    // guid in register space must be quadlet-wise big endian
    uint32_t *q((uint32_t*)&FeatureGuid);
    for(int i = 0; i < sizeof(GUID) / sizeof(uint32_t); ++i )
        q[i] = bswap_32(q[i]);

    const int32_t FeatureAddress[2] = { 0x01234567, 0x89ABCDEF };
    const int64_t ExpectedAddress(0xEFCDAB8967452301LL);
    // The smart feautre CSR consists of the GUID and the smart feature's address
    uint8_t CSR[sizeof(FeatureGuid) + sizeof(FeatureAddress)];
    memcpy(CSR, &FeatureGuid, sizeof(FeatureGuid));
    memcpy((uint8_t*)CSR + sizeof(FeatureGuid), &FeatureAddress, sizeof(FeatureAddress));

    Port.WritePort.CreateEntry(0x0080, sizeof(FeatureGuid), &FeatureGuid, RW);
    Port.ReadPort.CreateEntry(0x0080, sizeof(CSR), &CSR, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");
    // Get the address
    int64_t Address(*ptrAdvancedFeature());
    //CPPUNIT_ASSERT_EQUAL( Address, ExpectedAddress2 );

    // remember: the node returns an address
    EAccessMode accessMode(RO);
    ERepresentation representation(PureNumber);
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrAdvancedFeature->GetInc() );
    CPPUNIT_ASSERT_EQUAL( accessMode, ptrAdvancedFeature->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)INT64_MAX, ptrAdvancedFeature->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, ptrAdvancedFeature->GetMin() );
    CPPUNIT_ASSERT_EQUAL( representation, ptrAdvancedFeature->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( gcstring("1"), ptrAdvancedFeature->ToString() );
    //ptrAdvancedFeature->FromString( gcstring("12") );
    //CPPUNIT_ASSERT_EQUAL( gcstring("12"), ptrAdvancedFeature->ToString() );
}
