//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "StreamingTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( StreamingTestSuite );

void StreamingTestSuite::TestSimpleStreaming()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Integer Name=\"Streaming\">            \n"
    "    <Streamable>Yes</Streamable>        \n"
    "    <Value>20</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Integer Name=\"NoStreaming\">            \n"
    "    <Streamable>No</Streamable>            \n"
    "    <Value>20</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Integer Name=\"SelectedA\">            \n"
    "    <Value>1</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Integer Name=\"SelectedAA\">            \n"
    "    <pValue>SelectedA</pValue>            \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Node Name=\"SelectedB\"/>                \n"
    "                                        \n"
    "<Integer Name=\"Selector1\">                \n"
    "    <Value>1</Value>                    \n"
    "    <pSelected>SelectedA</pSelected>    \n"
    "    <pSelected>SelectedB</pSelected>    \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Integer Name=\"Selector2\">                \n"
    "    <Value>1</Value>                    \n"
    "    <pSelected>SelectedA</pSelected>    \n"
    "</Integer>                                \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrStreaming = Camera._GetNode("Streaming");
    CPPUNIT_ASSERT( ptrStreaming.IsValid() );

    CIntegerPtr ptrNoStreaming = Camera._GetNode("NoStreaming");
    CPPUNIT_ASSERT( ptrNoStreaming.IsValid() );

    CSelectorPtr ptrSelector1 = Camera._GetNode("Selector1");
    CPPUNIT_ASSERT( ptrSelector1.IsValid() );

    CSelectorPtr ptrSelector2 = Camera._GetNode("Selector2");
    CPPUNIT_ASSERT( ptrSelector2.IsValid() );

    CSelectorPtr ptrSelectedA = Camera._GetNode("SelectedA");
    CPPUNIT_ASSERT( ptrSelectedA.IsValid() );

    CSelectorPtr ptrSelectedAA = Camera._GetNode("SelectedAA");
    CPPUNIT_ASSERT( ptrSelectedAA.IsValid() );

    CSelectorPtr ptrSelectedB = Camera._GetNode("SelectedB");
    CPPUNIT_ASSERT( ptrSelectedB.IsValid() );

    /*----- check the streaming flag ---*/

    CPPUNIT_ASSERT_EQUAL( true, ptrStreaming->GetNode()->IsStreamable() );
    CPPUNIT_ASSERT_EQUAL( false, ptrNoStreaming->GetNode()->IsStreamable() );

    /*----- check the Selector list---*/

    CPPUNIT_ASSERT_EQUAL( true, ptrSelector1->IsSelector() );
    CPPUNIT_ASSERT_EQUAL( true, ptrSelector2->IsSelector() );
    CPPUNIT_ASSERT_EQUAL( false, ptrSelectedA->IsSelector() );
    CPPUNIT_ASSERT_EQUAL( false, ptrSelectedAA->IsSelector() );
    CPPUNIT_ASSERT_EQUAL( false, ptrSelectedB->IsSelector() );

    int i;
    FeatureList_t SelectedFeatures;

    i=3;
    ptrSelector1->GetSelectedFeatures(SelectedFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)i, SelectedFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("SelectedAA"), SelectedFeatures[--i]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("SelectedB"), SelectedFeatures[--i]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("SelectedA"), SelectedFeatures[--i]->GetNode()->GetName() );

    i=2;
    ptrSelector2->GetSelectedFeatures(SelectedFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)i, SelectedFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("SelectedAA"), SelectedFeatures[--i]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("SelectedA"), SelectedFeatures[--i]->GetNode()->GetName() );

    ptrSelectedA->GetSelectedFeatures(SelectedFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)0, SelectedFeatures.size() );

    ptrSelectedAA->GetSelectedFeatures(SelectedFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)0, SelectedFeatures.size() );

    ptrSelectedB->GetSelectedFeatures(SelectedFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)0, SelectedFeatures.size() );

    /*----- check the Selecting list---*/

    FeatureList_t SelectingFeatures;
    ptrSelector1->GetSelectingFeatures(SelectingFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)0, SelectingFeatures.size() );

    ptrSelector2->GetSelectingFeatures(SelectingFeatures);
    CPPUNIT_ASSERT_EQUAL( (size_t)0, SelectingFeatures.size() );

    ptrSelectedA->GetSelectingFeatures(SelectingFeatures);
    i=2;
    CPPUNIT_ASSERT_EQUAL( (size_t)i, SelectingFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Selector2"), SelectingFeatures[--i]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Selector1"), SelectingFeatures[--i]->GetNode()->GetName() );

    ptrSelectedB->GetSelectingFeatures(SelectingFeatures);
    i=1;
    CPPUNIT_ASSERT_EQUAL( (size_t)i, SelectingFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Selector1"), SelectingFeatures[--i]->GetNode()->GetName() );

    ptrSelectedAA->GetSelectingFeatures(SelectingFeatures);
    i=2;
    CPPUNIT_ASSERT_EQUAL( (size_t)i, SelectingFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Selector2"), SelectingFeatures[--i]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Selector1"), SelectingFeatures[--i]->GetNode()->GetName() );

}

void StreamingTestSuite::VeryTestSimpleStreaming()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Integer Name=\"SelectedA\">            \n"
    "    <Value>1</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Integer Name=\"SelectedAA\">            \n"
    "    <pValue>SelectedA</pValue>            \n"
    "</Integer>                                \n"
    "                                        \n"
    "<Integer Name=\"Selector1\">                \n"
    "    <Value>1</Value>                    \n"
    "    <pSelected>SelectedA</pSelected>    \n"
    "</Integer>                                \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CSelectorPtr ptrSelector1 = Camera._GetNode("Selector1");
    CPPUNIT_ASSERT( ptrSelector1.IsValid() );

    CSelectorPtr ptrSelectedA = Camera._GetNode("SelectedA");
    CPPUNIT_ASSERT( ptrSelectedA.IsValid() );

    CSelectorPtr ptrSelectedAA = Camera._GetNode("SelectedAA");
    CPPUNIT_ASSERT( ptrSelectedAA.IsValid() );

    FeatureList_t SelectingFeatures;
    ptrSelectedAA->GetSelectingFeatures(SelectingFeatures);
    int i=1;
    CPPUNIT_ASSERT_EQUAL( (size_t)i, SelectingFeatures.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Selector1"), SelectingFeatures[--i]->GetNode()->GetName() );

}

