//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/


#ifndef __TESTPORT_H
#define __TESTPORT_H

#include "GenApiTest.h"
#include "RegisterMap.h"
#include "GenApi/IPortRecorder.h"

/**
\brief Dummy Port for testing nodes
*/
namespace GenApi
{

    class CTestPort : public CRegisterMap, public GenApi::CPortImpl
    {
    public:
        CTestPort(bool Verbose = false) :
          m_Verbose(Verbose)
          {}

        //-------------------------------------------------------------
        // IBase implementation
        //-------------------------------------------------------------

        //! Get the access mode of the node
        virtual GenApi::EAccessMode GetAccessMode() const;

        //! Get the type of the main interface of a node
        virtual GenApi::EInterfaceType GetPrincipalInterfaceType() const;

        //---------------------------------------------------------------
        // IPort implementation
        //---------------------------------------------------------------

        //! Reads a chunk of bytes from the port
        virtual void Read(void *pBuffer, int64_t Address, int64_t Length);

        //! Writes a chunk of bytes to the port
        virtual void Write(const void *pBuffer, int64_t Address, int64_t Length);

    protected:
        //! determine if the port access is loged to the console
        bool m_Verbose;
    };

    class CTestPortReplay : public CTestPort
    {
    public:

        //! constructir
        CTestPortReplay (bool Verbose = false) :
          CTestPort(Verbose)
          {}

        //---------------------------------------------------------------
        // IPortReplay implementation
        //---------------------------------------------------------------

        //! sends the commands as a block to the camera.
        virtual void Replay( IPortWriteList *pPortRecorder );

        class CPackerPort : public GenApi::CPortImpl
        {
            /*---------------------------------------------------------------*/
            // IBase ==> You need to override this method
            /*---------------------------------------------------------------*/

            //! Get the access mode of the node
            virtual EAccessMode GetAccessMode() const
            {
                return WO;
            }

            //---------------------------------------------------------------
            // IPort implementation
            //---------------------------------------------------------------

            //! Reads a chunk of bytes from the port
            virtual void Read(void * /*pBuffer*/, int64_t /*Address*/, int64_t /*Length*/ )
            {
                // not implemented
                assert(false);
            }

            //! Writes a chunk of bytes to the port
            virtual void Write(const void *pBuffer, int64_t Address, int64_t Length);

        };

    };

};

#endif // __TESTPORT_H

