//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------

/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "XMLCacheTest.h"
#include "GenApi/GenApi.h"

#include <stdlib.h>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_REGISTRATION( XMLCacheTestSuite );

#if defined (WIN32)
#   define setenv(key, val, repl) SetEnvironmentVariable(key, val)
#endif

void XMLCacheTestSuite::TestXMLCache()
{
    static const char cache_name[] = GENICAM_CACHE_VERSION;
    CNodeMapRef::_ClearXMLCache();

    // remember the cache directory
    const char * const pCacheDirectory(getenv(cache_name));
    CPPUNIT_ASSERT(pCacheDirectory);

    // clear the cache directory; this disables the caching
    setenv(cache_name, "", 1);

    // cache disabled
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                        \n"
        "    <Node Name=\"Node\">                                \n"
        "    </Node>                                             \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        const CNodePtr ptrNode(Camera._GetNode("Node"));
        CPPUNIT_ASSERT( ptrNode.IsValid() );
    }

    // set the cache directory to the old value; this enables the caching
    setenv(cache_name, pCacheDirectory, 1);

    // cache enabled - cache empty
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                        \n"
        "    <Node Name=\"Node\">                                \n"
        "    </Node>                                             \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        const CNodePtr ptrNode(Camera._GetNode("Node"));
        CPPUNIT_ASSERT( ptrNode.IsValid() );
    }

    // cache enabled - cache full
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                        \n"
        "    <Node Name=\"Node\">                                \n"
        "    </Node>                                             \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        const CNodePtr ptrNode(Camera._GetNode("Node"));
        CPPUNIT_ASSERT( ptrNode.IsValid() );
    }
}
