//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "gcstringTest.h"
#include "GenApi/GenApi.h"
#include <string.h>

using namespace std;
using namespace GenICam;
using namespace GenApi;



CPPUNIT_TEST_SUITE_REGISTRATION( gcstringTestSuite );

void gcstringTestSuite::Testgcstring()
{
    gcstring MyString(5, 'A');
    CPPUNIT_ASSERT_EQUAL((gcstring)"AAAAA", MyString);

    MyString.assign(3, '1');
    CPPUNIT_ASSERT_EQUAL((gcstring)"111", MyString);

    // Test self assignement
    MyString = MyString.assign(MyString);


    MyString = "0123456";
    CPPUNIT_ASSERT_EQUAL((unsigned long)3, (unsigned long)MyString.find('3'));
    CPPUNIT_ASSERT_EQUAL((unsigned long)2, (unsigned long)MyString.find("23"));
    CPPUNIT_ASSERT_EQUAL((unsigned long)2, (unsigned long)MyString.find("23999",1, 2));
    CPPUNIT_ASSERT_EQUAL( (unsigned long)gcstring::_npos(), (unsigned long)MyString.find("bla") );


    CPPUNIT_ASSERT_EQUAL(MyString.operator !=("7895"), true);
    gcstring strCopy = MyString;
    CPPUNIT_ASSERT_EQUAL(MyString.operator !=(strCopy), false);

    CPPUNIT_ASSERT_EQUAL(MyString.operator +=('7'), (gcstring)"01234567");

    MyString = "0123456";
    strCopy = MyString;
    strCopy = MyString.operator +=('7');
    CPPUNIT_ASSERT_EQUAL(strCopy, (gcstring)"01234567");


    const gcstring ConstString = "0123456";
    strCopy = MyString;
    // this is rather dobious interface
    //strCopy = ConstString.operator +=('7');
    // rather use this
    strCopy = MyString + "7";
    // but never this
    //strCopy = MyString + '7';



    CPPUNIT_ASSERT_EQUAL(strCopy.operator >(ConstString), true);


    void * pMem = gcstring::operator new(50);
    ::memset(pMem, '1', 50);

    void * pMem1 = gcstring::operator new(20, pMem);
    ::memset(pMem1, '2', 20);
    gcstring::operator delete(pMem1, pMem);


    gcstring::operator delete(pMem);


    // std::istream operator >>
    ostringstream ostr_stdin(ios::in | ios::out);
    istream istr_stdin(ostr_stdin.rdbuf());
    ostr_stdin.write("www", 3);
    std::string StdVal;
    istr_stdin >> StdVal;
    CPPUNIT_ASSERT_EQUAL(StdVal, (std::string)"www");

    // the same again and better for gcstring? :-)
    ostringstream ostr_stdin2(ios::in | ios::out);
    istream istr_stdin2(ostr_stdin2.rdbuf());
    ostr_stdin2.write("www", 3);
    gcstring GcVal;
    istr_stdin2 >> GcVal;
    CPPUNIT_ASSERT_EQUAL(GcVal, (gcstring)"www");

/*char buf[20];
    buf[0] = '\0';
    istr_stdin >> buf;
    cout << "read word: \"" << buf << "\"" << endl;*/

    std::string test ("abc#def");
    std::istringstream is(test);
    gcstring token;
    getline (is, token, '#');
    CPPUNIT_ASSERT_EQUAL (gcstring("abc"), token);
}




void gcstringTestSuite::TestgcstringVector()
{
    const gcstring ConstString("0123");

    gcstring_vector MyStrVect(3, ConstString);
    CPPUNIT_ASSERT_EQUAL((unsigned long)MyStrVect.size(), 3UL);

    gcstring strVal(MyStrVect.at(1));
    CPPUNIT_ASSERT_EQUAL(strVal, ConstString);


    const gcstring_vector ConstVect(MyStrVect);
    strVal = ConstVect.at(1);
    CPPUNIT_ASSERT_EQUAL(strVal, ConstString);

    CPPUNIT_ASSERT_EQUAL(ConstVect[2], (gcstring)"0123");

    CPPUNIT_ASSERT_EQUAL((unsigned long)ConstVect.size(), 3UL);


    MyStrVect[1] = "45";
    MyStrVect[2] = "67";

    strVal = MyStrVect.back();
    CPPUNIT_ASSERT_EQUAL(strVal, (gcstring)"67");

    strVal = ConstVect.back();
    CPPUNIT_ASSERT_EQUAL(strVal, (gcstring)"0123");

    strVal = MyStrVect.front();
    CPPUNIT_ASSERT_EQUAL(strVal, (gcstring)"0123");

    strVal = ConstVect.front();
    CPPUNIT_ASSERT_EQUAL(strVal, (gcstring)"0123");

    gcstring_vector::const_iterator it1(ConstVect.begin());
    CPPUNIT_ASSERT_EQUAL(*it1, (gcstring)"0123");

    gcstring_vector::iterator it2(MyStrVect.begin());
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"0123");

    for (it1 = ConstVect.begin(); it1 != ConstVect.end(); it1 ++)
    {
        strVal = *it1;
        CPPUNIT_ASSERT_EQUAL(strVal, (gcstring)"0123");
    }

    CPPUNIT_ASSERT_EQUAL((unsigned long)ConstVect.capacity(), 3UL);

    CPPUNIT_ASSERT_EQUAL(ConstVect.empty(), false);

    gcstring_vector EmptyVect;
    CPPUNIT_ASSERT_EQUAL(EmptyVect.empty(), true);

    it2 = MyStrVect.begin() + 1;
    MyStrVect.erase(it2);
    CPPUNIT_ASSERT_EQUAL((unsigned long)MyStrVect.size(), 2UL);

    MyStrVect.insert(it2, "AB");
    CPPUNIT_ASSERT_EQUAL(MyStrVect.at(1), (gcstring)"AB");
    CPPUNIT_ASSERT_EQUAL((unsigned long)MyStrVect.size(), 3UL);

    size_t TheSize(ConstVect.max_size());
    CPPUNIT_ASSERT_EQUAL((bool)(TheSize > 0), true);

    MyStrVect.pop_back();
    CPPUNIT_ASSERT_EQUAL((unsigned long)MyStrVect.size(), 2UL);

    MyStrVect.resize(4, "ZX");
    CPPUNIT_ASSERT_EQUAL((unsigned long)MyStrVect.size(), 4UL);
    CPPUNIT_ASSERT_EQUAL(MyStrVect.at(3), (gcstring)"ZX");

    MyStrVect.reserve(5);
    CPPUNIT_ASSERT_EQUAL((unsigned long)MyStrVect.size(), 4UL);

    MyStrVect = MyStrVect;
    CPPUNIT_ASSERT_EQUAL((unsigned long)MyStrVect.size(), 4UL);

//#ifdef WIN32
    void * const pMem(gcstring_vector::operator new(50));
    ::memset(pMem, '1', 50);
    gcstring_vector::operator delete(pMem);
//#endif


    // iterator
    it2 = MyStrVect.begin();
    it2.operator +=(2);
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"ZX");

    it2.operator --();
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"AB");

    it2.operator --(1);
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"0123");

    it2.operator +=(1);
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"AB");

    it2.operator -=(1);
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"0123");

    gcstring_vector::iterator it2ConstB(MyStrVect.begin());
    it2 = it2ConstB.operator +(2);
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"ZX");

    it2 = MyStrVect.begin();
    it2 ++;
    gcstring_vector::iterator it2ConstC(it2);
    it2 = it2ConstC.operator -(1);
    CPPUNIT_ASSERT_EQUAL(*it2, (gcstring)"0123");

    it2 = MyStrVect.begin();
    it2 ++;
    gcstring_vector::iterator it2ConstD(it2);
    it2ConstD.operator -=(1);
    CPPUNIT_ASSERT_EQUAL(*it2ConstD, (gcstring)"0123");

    it2 = MyStrVect.begin();
    it2 ++;
    const gcstring_vector::iterator it2ConstE(it2);
    strVal = it2ConstE.operator [](1);
    CPPUNIT_ASSERT_EQUAL(strVal, (gcstring)"ZX");

    const gcstring * pstrVal = it2ConstE.operator ->();
    CPPUNIT_ASSERT_EQUAL(*pstrVal, (gcstring)"AB");



    // const_iterator
    it1 = MyStrVect.begin();
    it1.operator +=(3);
    CPPUNIT_ASSERT_EQUAL(*it1, (gcstring)"ZX");

    it1.operator --();
    CPPUNIT_ASSERT_EQUAL(*it1, (gcstring)"ZX");

    it1.operator -=(1);
    CPPUNIT_ASSERT_EQUAL(*it1, (gcstring)"AB");

    it1.operator --(1);
    CPPUNIT_ASSERT_EQUAL(*it1, (gcstring)"0123");

    it1 = MyStrVect.begin();
    gcstring_vector::const_iterator it1ConstA(it1);
    it1ConstA.operator +=(2);
    CPPUNIT_ASSERT_EQUAL(*it1ConstA, (gcstring)"ZX");

    const gcstring_vector::const_iterator it1ConstB(MyStrVect.begin());
    it1 = it1ConstB.operator +(2);
    CPPUNIT_ASSERT_EQUAL(*it1, (gcstring)"ZX");

    it1 = MyStrVect.begin();
    it1 ++;
    const gcstring_vector::const_iterator it1ConstC = it1;
    it1 = it1ConstC.operator -(1);
    CPPUNIT_ASSERT_EQUAL(*it1, (gcstring)"0123");

    it1 = MyStrVect.begin();
    it1 ++;
    gcstring_vector::const_iterator it1ConstD(it1);
    it1ConstD.operator -=(1);
    CPPUNIT_ASSERT_EQUAL(*it1ConstD, (gcstring)"0123");

    it1 = MyStrVect.begin();
    it1 ++;
    const gcstring_vector::const_iterator it1ConstE(it1);
    strVal = it1ConstE.operator [](1);
    CPPUNIT_ASSERT_EQUAL(strVal, (gcstring)"ZX");

    pstrVal = it1ConstE.operator ->();
    CPPUNIT_ASSERT_EQUAL(*pstrVal, (gcstring)"AB");

    CPPUNIT_ASSERT_EQUAL( (int64_t) 3LL, (int64_t)(ConstVect.end()-ConstVect.begin()));
    CPPUNIT_ASSERT(ConstVect.end()>ConstVect.begin());
    CPPUNIT_ASSERT(ConstVect.end()>=ConstVect.begin());
    CPPUNIT_ASSERT(ConstVect.begin()<ConstVect.end());
    CPPUNIT_ASSERT(ConstVect.begin()<=ConstVect.end());





    // Miscellaneous
    gcstring_vector::const_iterator it1a(MyStrVect.begin());
    intptr_t iInc(2);
#ifdef WIN32
    it1 = ::operator +(iInc, it1a);
#else
    it1 = iInc + it1a;
#endif
    CPPUNIT_ASSERT_EQUAL(*it1, gcstring("ZX"));


    gcstring_vector::iterator it2a(MyStrVect.begin());
    iInc = 2;
#ifdef WIN32
    it2 = ::operator +(iInc, it2a);
#else
    it2 = iInc + it2a;
#endif
    CPPUNIT_ASSERT_EQUAL(*it2, gcstring("ZX"));

    MyStrVect.assign(4, ConstString);
    for (it1 = MyStrVect.begin(); it1 != MyStrVect.end(); it1++)
        CPPUNIT_ASSERT_EQUAL(*it1, ConstString);
}
